specification Q2931_specification[g] : noexit

library
	BasicNaturalNumber, Boolean    
endlib


(*******************************************************************)
(** Datatypes declaration                                         **)
(*******************************************************************)


Type NaturalNumber is BasicNaturalNumber,Boolean

	opns 
		pred : Nat -> Nat
		Global_call_ref  : 	   -> Nat
		Indefini         :     -> Nat
		_-_  : Nat,Nat -> Nat
		_eq_, _ne_, _lt_, _le_, _gt_, _ge_  : Nat, Nat -> Bool


	eqns forall n,m : Nat

	ofsort Nat

		pred(0) = 0;
		pred(succ(n)) = n;

		succ(n) - succ(m) = n - m ;
		n - 0 = n;
		0 - n = n;
		Global_call_ref = 0 ;
		Indefini  = succ(succ(0)) *succ(succ(succ(0)))*succ(succ(0))*
			succ(succ(succ(succ(0))));
		

	ofsort   Bool

		0 eq 0         = true         ;
		0 eq Succ(m)      = false        ;
		Succ(m) eq 0      = false        ;
		Succ(m) eq Succ(n)   = m eq n    ;
		m ne n         = not(m eq n)     ;
		0 lt 0         = false        ;
		0 lt Succ(n)      = true         ;
		Succ(n) lt 0      = false        ;
		Succ(m) lt Succ(n)   = m lt n    ;
		m le n         = (m lt n) or (m eq n)  ;
		m ge n         = not(m lt n)     ;
		m gt n         = not(m le n)     ;

EndType (*NaturalNumber*)


 
(* possible causes  *)
(********************)

  Type Cause is Boolean
	sorts Cause
	opns 
	  cause_96, 
	  cause_97,     (*message type non-existant or not implemented*)
	  cause_100, 
	  cause_101,	(*message not compatible with the call State*)
	  cause_102, 	(*recovery on timer expiry*)
	  cause_30,
	  cause_41,
	  cause_81, 	(* invalid call reference value*)
	  cause_reject:  ->Cause

	  isCause_30 : Cause    -> Bool

	  eqns
	   ofsort Bool
	 	
	  isCause_30(cause_96) = false;
	  isCause_30(cause_97) = false; 
	  isCause_30(cause_100) = false; 
	  isCause_30(cause_101) = false; 
	  isCause_30(cause_102) = false; 
	  isCause_30(cause_30) = true;
	  isCause_30(cause_41) = false;
	  isCause_30(cause_81) = false;
	  isCause_30(cause_reject) = false;
  EndType(*Cause*)
 
(*Timer of Q2931*)
(****************)
 
  Type Layer3_timer_name is
	sorts Layer3_timer_name
	opns
	  T303, (* Setup request*)
	  T308,  (*Release request*)
	  T310,  (*call_proc indication*)
	  T309,  (* SAAL failure re-establish request*)
	  T322,  (* Status_enquiry request*)
	  T313,   (*  state 8            *)
	  T316    (* Restart request*)  :	-> Layer3_timer_name
  EndType (* Layer3_timer_name *)

   Type Layer3_timeout is
	sorts Layer3_timeout
	opns
	  T303_Expired1, T303_Expired2,
	  T308_Expired1, T308_Expired2,
	  T310_Expired1, T310_Expired2,
	  T309_Expired1, T309_Expired2,
	  T322_Expired1, T322_Expired2,
	  T313_Expired1, T313_Expired2,
	  T316_Expired1 :	-> Layer3_timeout
  EndType (* Layer3_timeout *)


(*gestion des Timers*)
  Type SetofTimer is  Layer3_timer_name, NaturalNumber
	sorts SetofTimer
	opns 
	  TimerSet : Nat,Bool,Nat,Bool,Nat,Bool,Nat,Bool,Nat,Bool,Nat,Bool,Nat,Bool -> SetofTimer
	  StartTimer: Layer3_timer_name, SetofTimer 		-> SetofTimer
	  ResetTimer: Layer3_timer_name, SetofTimer 		-> SetofTimer
	  ResetallTimer : SetofTimer				-> SetofTimer
	  IncrTimer : Layer3_timer_name, SetofTimer 		-> SetofTimer
	  ValTimer   : Layer3_timer_name, SetofTimer 		-> Nat 
	  ActiveTimer: Layer3_timer_name, SetofTimer 		-> Bool
	eqns
	  forall n1,n2,n3,n4,n5,n6,n7:Nat,b1,b2,b3,b4,b5,b6,b7:Bool

	  ofsort Bool
	    ActiveTimer(T303,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b1;
	    ActiveTimer(T308,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b2;
	    ActiveTimer(T309,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b3;
	    ActiveTimer(T310,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b4;
	    ActiveTimer(T313,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b5;
	    ActiveTimer(T316,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b6;
	    ActiveTimer(T322,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = b7;
	    

	  ofsort Nat
	    ValTimer(T303,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = n1;
	    ValTimer(T308,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b7,n7,b7)) = n2;
	    ValTimer(T309,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = n3;
	    ValTimer(T310,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = n4;
	    ValTimer(T313,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = n5;
	    ValTimer(T316,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = n6;
	    ValTimer(T322,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7)) = n7;
	  ofsort SetofTimer

	    StartTimer(T303,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,true,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    StartTimer(T308,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,true,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    StartTimer(T309,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,true,n4,b4,n5,b5,n6,b6,n7,b7);
	    StartTimer(T310,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,true,n5,b5,n6,b6,n7,b7);
	    StartTimer(T313,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,true,n6,b6,n7,b7);
	    StartTimer(T316,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,true,n7,b7);
	    StartTimer(T322,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,true,n6,b6,n7,true);


	    	    
	     ResetTimer(T303,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,false,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    ResetTimer(T308,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,false,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    ResetTimer(T309,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,false,n4,b4,n5,b5,n6,b6,n7,b7);
	    ResetTimer(T310,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,false,n5,b5,n6,b6,n7,b7);
	    ResetTimer(T313,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,false,n6,b6,n7,b7);
	    ResetTimer(T316,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,false,n7,b7);
	    ResetTimer(T322,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,false);

	    ResetallTimer(TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,false,n2,false,n3,false,n4,false,n5,false,n6,false,n7,false);

	    IncrTimer(T303,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(succ(n1),b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    IncrTimer(T308,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,succ(n2),b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    IncrTimer(T309,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,succ(n3),b3,n4,b4,n5,b5,n6,b6,n7,b7);
	    IncrTimer(T310,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,succ(n4),b4,n5,b5,n6,b6,n7,b7);
	    IncrTimer(T313,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,succ(n5),b5,n6,b6,n7,b7);
	    IncrTimer(T316,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,succ(n6),b6,n7,b7);
	    IncrTimer(T322,TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,n7,b7))=
	         TimerSet(n1,b1,n2,b2,n3,b3,n4,b4,n5,b5,n6,b6,succ(n7),b7);

  EndType (*SetofTimer*)

 	  
(* Primitives between the Q2931 and the SSCF *)
(*********************************************)

  Type AAL_SP is boolean
	sorts AAL_SP
	opns 
	  AAL_Establish_Req,
	  AAL_Establish_Ind, 
	  AAL_Establish_Conf,  
	  AAL_Release_Req, 
	  AAL_Release_Ind, 
	  AAL_Release_Conf,
	  AAL_Data_Req, 
	  AAL_Data_Ind,
	  AAL_UnitData_Req, 
	  AAL_UnitData_Ind	:	-> AAL_SP
   EndType (*AAL_SP*)

(*Layer3 PDU*)
(************)

   Type Layer3_PDU is Boolean,NaturalNumber,Cause,Layer3_States
	sorts Layer3_PDU
	opns
	 Setup		  : Nat			->Layer3_PDU	
 	 Call_Proceeding  : Nat			-> Layer3_PDU
	 Connect	  : Nat			->Layer3_PDU	
	 ConnectAck	  : Nat			->Layer3_PDU
	 Release_Complete, 
 	 Release	  : Nat, Cause		->Layer3_PDU
	 Release_Complete,
	 Release	  : Nat			-> Layer3_PDU	
	 Status		  : Nat,Layer3_States   	-> Layer3_PDU	
	 Status		  : Nat,Layer3_States ,Cause  	-> Layer3_PDU
	 Status_Enquiry	  : Nat				-> Layer3_PDU  
	 Restart   	  : Nat		 		->Layer3_PDU	
         Restart_Ack      : Nat		 		->Layer3_PDU 
	 Unregonized_message : 				-> Layer3_PDU

	 get_call_state   : Layer3_PDU			-> Layer3_States
	 get_call_ref     : Layer3_Pdu			-> Nat
 	 get_cause	  : Layer3_PDU			-> Cause

	 isSetup,
	 isCall_Proceeding,
	 isConnect,
	 isConnectAck,
	 isRelease_Complete,
	 isRelease,
	 isStatus,
	 isStatus_Enquiry,
	 isRestart,
	 isRestart_Ack,
	 isUnregonized_message    : Layer3_PDU  -> Bool  
	 
	 eqns
	 forall n:Nat, c:Cause,cs:Layer3_States

	 ofsort Nat
	  get_call_ref(Setup(n)) = n;
	  get_call_ref(Call_Proceeding(n)) = n;
	  get_call_ref(Connect(n)) = n;
	  get_call_ref(ConnectAck(n)) = n;
	  get_call_ref(Release_Complete(n,c)) = n;
	  get_call_ref(Release_Complete(n)) = n;
	  get_call_ref(Release(n,c)) = n;
	  get_call_ref(Release(n)) = n;
	  get_call_ref(Status(n,cs)) = n;
	  get_call_ref(Status_Enquiry(n)) = n;
	  get_call_ref(Restart(n)) = n;
	  get_call_ref(Restart_Ack(n)) = n;
	  get_call_ref(Unregonized_message)  = 0;

	 ofsort Layer3_States
	   get_call_state(Status(n,cs)) = cs;
	   get_call_state(Status(n,cs,c)) = cs;

	 ofsort Cause
	   get_cause(Status(n,cs,c)) = c;


	 ofsort Bool
	  isSetup(Setup(n)) = True;
	  isSetup(Call_Proceeding(n)) = False;
	  isSetup(Connect(n))  = False;
	  isSetup(ConnectAck(n)) = False;
	  isSetup(Release_Complete(n,c)) = False;
	  isSetup(Release_Complete(n)) = False;
	  isSetup(Release(n,c)) = False;
	  isSetup(Release(n)) = False;
	  isSetup(Unregonized_message) = False;
	  isSetup(Status(n,cs)) = False;
	  isSetup(Status_Enquiry(n)) = False;

	  isCall_Proceeding(Call_Proceeding(n)) = True;
	  isCall_Proceeding(Setup(n)) = False;
	  isCall_Proceeding(Connect(n))  =False;
	  isCall_Proceeding(ConnectAck(n)) = False;
	  isCall_Proceeding(Release_Complete(n,c)) = False;
	  isCall_Proceeding(Release(n,c)) = False;
	  isCall_Proceeding(Release(n)) = False;
 	  
	  isConnect(Connect(n)) = True;
	  isConnect(Call_Proceeding(n)) = False;
	  isConnect(Setup(n)) = False;
	  isConnect(ConnectAck(n)) = False;
	  isConnect(Release_Complete(n,c)) = False;
	  isConnect(Release(n,c)) = False;
	  isConnect(Release(n)) = False;

	  isConnectAck(ConnectAck(n)) = True;
	  isConnectAck(Call_Proceeding(n)) = False;
	  isConnectAck(Setup(n)) = False;
	  isConnectAck(Connect(n))  =False;
	  isConnectAck(Release_Complete(n,c)) = False;
	  isConnectAck(Release(n,c)) = False;
	  isConnectAck(Release(n)) = False;

	  isRelease_Complete(Release_Complete(n,c)) = True;
	  isRelease_Complete(Release_Complete(n)) = True;
	  isRelease_Complete(Call_Proceeding(n)) = False;
	  isRelease_Complete(Setup(n)) = False;
	  isRelease_Complete(Connect(n))  =False;
	  isRelease_Complete(ConnectAck(n)) = False;
	  isRelease_Complete(Release(n,c)) = False;
	  isRelease_Complete(Release(n)) = False;


	  isRelease(Release(n,c)) = True;
	  isRelease(Release(n)) = True;
	  isRelease(Call_Proceeding(n)) = False;
	  isRelease(Setup(n)) = False;
	  isRelease(Connect(n))  =False;
	  isRelease(ConnectAck(n)) = False;
	  isRelease(Release_Complete(n,c)) = False;

	  
	  
	  isStatus(Status(n,cs)) = True;
	  isStatus(Status(n,cs,c)) = True;
	  isStatus(Call_Proceeding(n)) = False;
	  isStatus(Setup(n)) = False;
	  isStatus(Connect(n))  =False;
	  isStatus(ConnectAck(n)) = False;
	  isStatus(Release_Complete(n,c)) = False;
	  isStatus(Release(n,c)) = False;
	  isStatus(Release(n)) = False;

	  isStatus_Enquiry(Status_Enquiry(n)) = True;
	  isStatus_Enquiry(Call_Proceeding(n)) = False;
	  isStatus_Enquiry(Setup(n)) = False;
	  isStatus_Enquiry(Connect(n))  =False;
	  isStatus_Enquiry(ConnectAck(n)) = False;
	  isStatus_Enquiry(Release_Complete(n,c)) = False;
	  isStatus_Enquiry(Release(n,c)) = False;
	  isStatus_Enquiry(Release(n)) = False;
 
	  isRestart(Restart(n)) = True;
	  isRestart(Call_Proceeding(n)) = False;
	  isRestart(Setup(n)) = False;
	  isRestart(Connect(n))  =False;
	  isRestart(ConnectAck(n)) = False;

	  isRestart_Ack(Restart_Ack(n)) = True; 
	  isRestart_Ack(Call_Proceeding(n)) = False;
	  isRestart_Ack(Setup(n)) = False;
	  isRestart_Ack(Connect(n))  =False;
	  isRestart_Ack(ConnectAck(n)) = False;


	  isUnregonized_message(Unregonized_message) = True;
	  isUnregonized_message(Call_Proceeding(n)) = False;
	  isUnregonized_message(Setup(n)) = False;
	  isUnregonized_message(Connect(n))  =False;
	  isUnregonized_message(ConnectAck(n)) = False;
	  isUnregonized_message(Release_Complete(n,c)) = False;
	  isUnregonized_message(Release(n,c)) = False;
	  isUnregonized_message(Release(n)) = False;
 EndType (*Layer3_PDU *)
 

type Layer3_States is Boolean
	sorts Layer3_States
	opns
	  Connect_State,
	  NULL, 
	  Call_Initiated_1,
	  Outgoing_Call_Proceeding_3, 
	  (*Call_Delivered_4,*) 
	  Call_Present_6,
	  (*Call_Received_7,*)
	  Connect_Request_8,
	  Incoming_Call_Proceeding_9,
	  Release_Request_11,
	  Release_Ind_12,
	  Restart_Request,
	  Active_10 :	-> Layer3_States

	   
	  isNULL  : Layer3_States      -> Bool
	
	eqns
	ofsort Bool
	  isNULL(NULL) 		= True;
	  isNULL(Connect_State) = False;
	  isNULL(Call_Initiated_1) = False;
	  isNULL(Outgoing_Call_Proceeding_3) = False;
	  isNULL(Call_Present_6) = False;
	  isNULL(Connect_Request_8) = False;
	  isNULL(Incoming_Call_Proceeding_9) = False;
	  isNULL(Release_Request_11) = False;
	  isNULL(Release_Ind_12) = False;
	  isNULL(Restart_Request) = False;
	  isNULL(Active_10) = False;
endtype (* Layer3_States *)

(*******************************************************************)
(** End of data types declaration                                 **)
(*******************************************************************)	




behaviour
     		Q2931[g]
	 
        

where
    

(*******************************************************************)
(** Specification of the Q2931                                    **)
(*******************************************************************)

  Process Q2931[g] :noexit:=
     hide L3_I,L3_Timer in 
      (  
       	 Layer3[g,L3_I,L3_Timer](NULL, TimerSet(0,false,0,false,0,false, 0,false,0,false,0,false,0,false),Indefini)
       )
EndProc (*Q2931*)

   

  Process Layer3[g,L3_I,L3_Timer](S:Layer3_States, ts:SetofTimer, call_ref:Nat): noexit:=
    
    (    
       Layer3_Action[g,L3_I,L3_Timer](S,ts,call_ref) 
	 >> accept S:Layer3_States, ts:SetofTimer, call_ref:Nat 
         in  Layer3[g,L3_I,L3_Timer](S, ts, call_ref)
	 
     )
  EndProc(*Layer3*)
   



   Process Layer3_Action[g,L3_I,L3_Timer](S:Layer3_States,
	ts :SetofTimer,call_ref :Nat): exit(Layer3_States,SetofTimer,Nat):=
     
    [S = Connect_State ]->
	(
	  g !AAL_establish_Req;
	    g !AAL_Establish_Conf;
	     exit(NULL,ts,call_ref)
	[]
	  g !AAL_Establish_Ind;
	      exit(NULL,ts,call_ref)
	)  
[]
    [ S = NULL ] -> 
	( 
	    
	  (  L3_I ? select_call_ref:Nat;     (*Call reference selection*)
	       ( let call_ref:Nat  = select_call_ref in 
	          g !AAL_Data_Req !Setup(call_ref);     (*5.5.1.1*)
	              exit(Call_Initiated_1,StartTimer(T303,ts),call_ref)
	       )
	     
	  )
	[]
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	   [isSetup(pdu)]->  (*5.5.2.1*)
		(  
		   let call_ref:Nat = get_call_ref(pdu) in
		        ( 
			  [ call_ref eq Global_call_ref ]->   (*5.5.6.3.2 e*)
			     g !AAL_Data_Req !Status(Global_call_ref,S,cause_81);
			     exit(S,ts,call_ref)
				[]
		          [call_ref ne Global_call_ref]->
	  	             exit(Call_Present_6,ts,call_ref)
		        )
		 )  
		[]
	  [isRelease_Complete(pdu)]->  (*5.5.6.3.2 no respond*)
		(
	   	  exit(S,ts,call_ref)
		)
		[]
	  [isStatus(pdu)]->   
	   	    ( let cs:Layer3_States = get_call_state(pdu) in
			 [isNULL(cs)]->   (*5.5.6.12*)
	 		   exit(S,ts,call_ref)
				[]
			[not(isNULL(cs))]-> (*5.5.6.12 a*)
			  g !AAL_Data_Req !Release_Complete(call_ref,cause_101);
			  exit(S,ts,call_ref)
	     	    )
		[]
	  [isStatus_Enquiry(pdu)]->       (*5.5.6.11 and  5.5.6.3.2 g)
		(
	  	   g !AAL_Data_Req !Status(call_ref,S,cause_30);
	  	   exit(S,ts,call_ref)
		)
		[]
	  [isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(S,ts,Indefini)
		)
		[]
	  [isCall_Proceeding(pdu) or isConnect(pdu) or 
			isConnectAck(pdu) or isRelease(pdu) ]-> (*5.5.6.3.2*)
		(
	         g !AAL_Data_Req !Release_Complete(call_ref,cause_81);(*Invalid call reference value*)
	           exit(S,ts,call_ref)
		)
	   )
	[]
	  g !AAL_Release_Ind;
	  exit(NULL,ts,call_ref)
     )
[]
    [S = Call_Initiated_1] ->
     (
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	  [isCall_Proceeding(pdu)]->
		( let cr:Nat = get_Call_ref(pdu) in
		    [ cr eq Global_call_ref]->   (*5.5.6.3.2 e*)
			     g !AAL_Data_Req !Status(Global_call_ref,S,cause_81);
			     exit(S,ts,call_ref)
				[]
		    [ cr eq call_ref]->  (*5.5.1.5*)
		      (
		        let ts:SetofTimer = ResetTimer(T303,ts) in
	     	    	    let ts:SetofTimer = StartTimer(T310,ts) in
	      	               exit(Outgoing_Call_Proceeding_3,ts,call_ref)
		       )
				[]
		    [(cr ne call_ref) and (cr ne Global_call_ref)]->  (*5.5.6.3.2*)
			g !AAL_Data_Req !Release_Complete(call_ref,cause_81);
			(*Release call reference*)
			 exit(NULL,ts,Indefini)
	  	)
		[]
	  [isConnect(pdu)]-> 
	     ( let cr:Nat = get_Call_ref(pdu) in
	             [ cr eq Global_call_ref ]->   (*5.5.6.3.2 e*)
			     g !AAL_Data_Req !Status(Global_call_ref,S,cause_81);
			     exit(S,ts,call_ref)
				[]
		    [ cr eq call_ref]->  (*5.5.1.7*)
	       		g !AAL_Data_Req ! ConnectAck(call_ref);
	      		 exit(Active_10,ResetTimer(T303,ts),call_ref)
				[]
		    [(cr ne call_ref) and (cr ne Global_call_ref)]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ts,Indefini)
	     )
		[]
	  [isSetup(pdu)]->   (* no action is taken 5.5.6.3.2 d*)
		(
		  exit(S,ts,call_ref)
		)
		[]
	 [isRelease_Complete(pdu)]-> (*5.5.4.2 resources requested are not available*)
		(
	  	 (*Release call reference*)
	  	 exit(NULL,ResetTimer(T303,ts),Indefini)
		)
		[]
	  [isRelease(pdu)]->  (*5.5.6.4*)
		( let cr:Nat = get_Call_ref(pdu) in
		    [cr eq call_ref]-> 
		   	 (*Release call reference*)
		  	  g !AAL_Data_Req !Release_Complete(call_ref);
		  	    exit(NULL,ResetTimer(T303,ts),Indefini)
			[]
		   [cr ne call_ref]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ts,Indefini) 
		)
	 	[]
	  [isStatus(pdu)]->  (*5.5.6.12*)
	   	    ( let cs:Layer3_States = get_call_state(pdu) in
			 [isNULL(cs)]->  (*5.5.6.12 c*)
			   (*Release call reference*)
	 		   exit(NULL,ResetallTimer(ts),Indefini)
				[]
			[not(isNULL(cs))]->    (* other cases depend on the implementation*)
			     exit(S,ts,call_ref)
		    )
		[]
	[isStatus_Enquiry(pdu)]->  (*5.5.6.11*)
		(  g !AAL_Data_Req !Status(call_ref,S,cause_30);
	             exit(S,ts,call_ref)
		) 
		[]
	[isConnectAck(pdu) or isRestart_Ack(pdu)]->  		 (* 5.5.6.4 Unexpected message*)
		(  g !AAL_Data_Req !Status(call_ref,S,cause_101);
		     exit(S,ts,call_ref)
		)
		[]
	[isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(NULL,ts,Indefini)
		)
		[]
	[isUnregonized_message(pdu)]->   (* 5.5.6.4 Unregonized message *)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
	)
 	[]
	  [ ActiveTimer(T303,ts)  and (ValTimer(T303,ts) eq 0)]->  (*Timeout1 T303*) (*5.5.1.1*)
		L3_Timer !T303_Expired1;    
	         g !AAL_Data_Req !Setup(Call_ref) ;
	           ( let ts:SetofTimer =  StartTimer(T303,ts) in
	              let ts:SetofTimer = IncrTimer(T303,ts) in
	            	exit(S,ts,call_ref)
	            )
	[]
	  [ActiveTimer(T303,ts) and (ValTimer(T303,ts) ne 0)]->  (*Timeout2 T303*) (*5.5.1.1*)
		L3_Timer !T303_Expired2; 
	            (* release call reference *)
	              exit(NULL,ts,Indefini)
     
 	[]
	  g !AAL_Data_Req !Release(call_ref);	(*5.5.4.3*)
	    ( let ts:SetofTimer = ResetTimer(T303,ts) in
		let ts:SetofTimer = StartTimer(T308,ts) in
		     exit(Release_Request_11,ts,call_ref)
	    )
	[]
	  g !AAL_Establish_Ind; (*5.5.6.9*)
	     g !AAL_Data_Req !Release_complete(call_ref,cause_41);
	     ( let ts:SetofTimer = ResetallTimer(ts) in 
		   (*Release call reference*)
	    	   exit(NULL,ResetTimer(T303,ts),Indefini)
	     )
	[]
	 g !AAL_Release_Ind;  (*5.5.6.10*)
	  (*release call reference and connection id*)
	  exit(NULL,ResetTimer(T303,ts),Indefini)

    )
[]
    [ S = Outgoing_Call_Proceeding_3 ] ->
	(
	  g !AAL_Data_Ind ?pdu: Layer3_PDU;
	  (
	  [isConnect(pdu)]-> 
	     ( let cr:Nat = get_call_ref(pdu) in
	   	    [ cr eq Global_call_ref ]->   (*5.5.6.3.2 e*)
			     g !AAL_Data_Req !Status(Global_call_ref,S,cause_81);
			     exit(S,ts,call_ref)
				[]
		    [ cr eq call_ref]->  (*5.5.1.7*)
	       		g !AAL_Data_Req ! ConnectAck(call_ref);
	      		 exit(Active_10,ResetTimer(T310,ts),call_ref)
				[]
		    [(cr ne call_ref) and (cr ne Global_call_ref)]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref,cause_81);
			   exit(NULL,ResetTimer(T310,ts),Indefini)
	     )
		[]
	  [isSetup(pdu)]->   (*5.5.6.3.2 d.  no action is taken*)
		(
		  exit(S,ts,call_ref)
		)
		[]
	[isCall_Proceeding(pdu) or isConnectAck(pdu) or isRestart_Ack(pdu)]->  (*5.5.6.4*)
		(
		  g !AAL_Data_Req !Status(call_ref,S,cause_101);
		  exit(S,ts,call_ref)
		)
		[]
	[isUnregonized_message(pdu)]->   (* 5.5.6.4 Unregonized message *)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
		[]
	  [isRelease(pdu)]->  (*5.5.4.4*)
	     ( let cr:Nat = get_Call_ref(pdu) in
		    [cr eq call_ref]-> 
		        exit(Release_Ind_12,ResetTimer(T310,ts),call_ref)	 		  	   
			[]
		   [cr ne call_ref]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ResetTimer(T310,ts),Indefini) 
		)
	      	  []
	  [isStatus(pdu)]->  (*5.5.6.12*)
	   	    ( let cs:Layer3_States = get_call_state(pdu) in
			 [isNULL(cs)]->  (*5.5.6.12 c*)
			   (*Release call reference*)
	 		   exit(NULL,ResetallTimer(ts),Indefini)
				[]
			[not(isNULL(cs))]->    (* other cases depend on the implementation*)
			     exit(S,ts,call_ref)
		    )
		[]
	  [isRelease_Complete(pdu)]->  (*5.5.6.4 e.g after a bad STATUS*)
	    (  
	      (*release call reference*)
	      exit(NULL,ResetTimer(T310,ts),Indefini)
	    )
		[]
	  [isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(NULL,ResetTimer(T310,ts),Indefini)
		)
		[]
	  [isStatus_Enquiry(pdu)]->    (*5.5.6.11*)
	    ( g !AAL_Data_Req !Status(Call_ref,S,cause_30);
	      exit(S,ts,call_ref)
	    )
		
	   )
	[]
	  g !AAL_Data_Req !Release(call_ref); (*clearing as in 5.5.4.3*)
	     (  let ts:SetofTimer = ResetTimer(T310,ts) in
		   let ts:SetofTimer = StartTimer(T308,ts) in
	       	      exit(Release_Request_11,ts,call_ref)
	      )
	[]
	   [ ActiveTimer(T310,ts) and (ValTimer(T310,ts) eq 0)]-> (*Timeout T310*)  (*5.5.1.5*)
		L3_Timer !T310_Expired1;
	        ( g !AAL_Data_Req !Release(call_ref,cause_102); (*5.5.4.3*)
	          (*Disconnect virtual channel*)
	            exit(Release_Request_11,StartTimer(T308,ts),call_ref)
	        )
	 []
	  g !AAL_Data_Req !Release(call_ref);  (*5.5.4.4*)
	    ( let ts:SetofTimer = ResetTimer(T310,ts) in
		let ts:SetofTimer = StartTimer(T308,ts) in
		     exit(Release_Request_11,ts,call_ref)
	    )
	[]
	  g !AAL_Establish_Ind;(*5.5.6.9*)
	     g !AAL_Data_Req !Release_complete(call_ref,cause_41);
	     ( let ts:SetofTimer = ResetallTimer(ts) in 
		   (*Release call reference*)
	    	   exit(NULL,ResetTimer(T310,ts),Indefini)
	     )
	[]
	 g !AAL_Release_Ind;  (*5.5.6.10*)
	  (*release call reference *)
	  exit(NULL,ResetTimer(T310,ts),Indefini)
   )
 
[]
    [S = Call_Present_6] ->
	(
	    (*Select VCI/VPI*)
	    g !AAL_Data_Req !Call_Proceeding(call_ref); (*5.5.2.5.1.1*)
	        exit(Incoming_Call_Proceeding_9,ts,call_ref)
	[]
	    g !AAL_Data_Req !Release_Complete(call_ref,cause_reject); (*5.5.2.3, 5.5.2.4, 5.5.2.5.1.1*)
			(*cause 17 "User busy")
			(*cause 21 "Call rejected")
			(*cause 23 "User rejects all with calling line ..."*)
			(*cause 35 "Request VPCI/VCI not available"*)
			(*cause 47 "Resource unavailable, unspecified"*)
			(*cause 49 "Quality of service unavailable"*)
			(*cause 88 "Incompatible destination"*)
	          exit(NULL,ts,Indefini)
	[]
	    g !AAL_Data_Req !Connect(call_ref); (*5.5.2.5.1.1, 5.5.2.6*)
	       ( let ts:SetofTimer = StartTimer(T313,ts) in
	      		exit(Connect_Request_8,ts,call_ref)
	       )
	[]
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	   [isSetup(pdu)]->   (*no action is taken*)
		(
		  exit(S,ts,call_ref)
		)
		[]
	[isUnregonized_message(pdu)]->   (*Unregonized message *)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
		[]
	   [isRelease(pdu)]-> (*5.5.4.4*)
	      ( let cr:Nat = get_Call_ref(pdu) in
		    [cr eq call_ref]-> 
		        exit(Release_Ind_12,ts,call_ref)	 		  	   
			[]
		   [cr ne call_ref]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ts,Indefini) 
	       )
	  	[]
	  [isStatus(pdu)]->  (*5.5.6.12*)
	   	    ( let cs:Layer3_States = get_call_state(pdu) in
			 [isNULL(cs)]->  (*5.5.6.12 c*)
			   (*Release call reference*)
	 		   exit(NULL,ResetallTimer(ts),Indefini)
				[]
			[not(isNULL(cs))]->    (* other cases depend on the implementation*)
			     exit(S,ts,call_ref)
		    )
		[]
	  [isRelease_Complete(pdu)]->  (* 5.5.6.4 e.g after a bad STATUS*)
		( 
	         (*release call reference *)
	         exit(NULL,ts,Indefini)
		)
		[]
	  [isStatus_Enquiry(pdu)]-> (*5.5.6.11*)
		( g !AAL_Data_Req !Status(call_ref,S,cause_30);
	   	  exit(S,ts,call_ref)
		)
	  )
	
	[]
	  g !AAL_Establish_Ind; (*5.5.6.9*)
	     g !AAL_Data_Req !Release_complete(call_ref,cause_41);
	     ( let ts:SetofTimer = ResetallTimer(ts) in 
		   (*Release call reference*)
	    	   exit(NULL,ts,Indefini)
	     )
	[]
	 g !AAL_Release_Ind;  (*5.5.6.10*)
	   (*release call reference and connection id*)
	   exit(NULL,ts,Indefini)
	
      )

 []
    
    [ S = Connect_Request_8 ] ->
	(
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	   [isConnectAck(pdu)]->  (*5.5.2.7*)
		(  let cr:Nat = get_call_ref(pdu) in
		    [ cr eq Global_call_ref]->   (*5.5.6.3.2 e*)
			     g !AAL_Data_Req !Status(Global_call_ref,S,cause_81);
			     exit(S,ts,call_ref)
				[]
		    [cr eq call_ref]-> 
		    	   exit(Active_10,ResetTimer(T313,ts),call_ref)
			[]
		    [(cr ne call_ref) and (cr ne Global_call_ref)]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref,cause_81);
			   exit(NULL,ts,Indefini)
		)
		[]
	  [isSetup(pdu)]->   (* no action is taken  5.5.6.3.2 d*)
		(
		  exit(S,ts,call_ref)
		)
		[]
	[isCall_Proceeding(pdu) or isConnect(pdu) or isRestart_Ack(pdu) ]->  (*5.5.6.4*)
		( 
		   g !AAL_Data_Req !Status(call_ref,S,cause_101);
		   exit(S,ts,call_ref)
		)
		[]
	[isUnregonized_message(pdu)]->   (*Unregonized message  5.5.6.4*)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
		[]
	  [isRelease(pdu)]->  (*5.5.4.4*)
		( let cr:Nat = get_Call_ref(pdu) in
		    [cr eq call_ref]-> 
		        exit(Release_Ind_12,ResetTimer(T313,ts),call_ref)	 		  	   
			[]
		   [cr ne call_ref]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ResetTimer(T313,ts),Indefini)
		  )
	  	[]
	  [isStatus(pdu)]->  (*5.5.6.12*)
	   	    ( let cs:Layer3_States = get_call_state(pdu) in
			 [isNULL(cs)]->  (*5.5.6.12 c*)
			   (*Release call reference*)
	 		   exit(NULL,ResetallTimer(ts),Indefini)
				[]
			[not(isNULL(cs))]->    (* other cases depend on the implementation*)
			     exit(S,ts,call_ref)
		    )
		[]
	  [isRelease_Complete(pdu)]-> (*5.5.6.4*)
		( 
	          let ts:SetofTimer = ResetTimer(T313,ts) in
		     (*Release call reference*)
	    	     exit(NULL,ResetTimer(T313,ts),Indefini)
		)
		[]
	  [isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(NULL,ResetTimer(T313,ts),Indefini)
		)
		[]
	  [isStatus_Enquiry(pdu)]->   (*5.5.6.11*)
		( g !AAL_Data_Req !Status(call_ref,S,cause_30);
	    	  exit(S,ts,call_ref)
		)
	  )
	[]
	  [ActiveTimer(T313,ts) and (ValTimer(T313,ts) eq 0)]->  (*Timeout T313*)  (*5.5.2.7*)
	    L3_Timer !T313_Expired1;
	    g !AAL_Data_Req !Release(call_ref,cause_102); (*clearing as in 5.5.4.3*)
	      ( let ts:SetofTimer = StartTimer(T308,ts) in
	      		exit(Release_Request_11,ts,call_ref)
	       )
	  []
	  g !AAL_Data_Req !Release(call_ref); (*5.5.4.1*)
	    ( let ts:SetofTimer = ResetTimer(T313,ts) in
		let ts:SetofTimer = StartTimer(T308,ts) in
		     exit(Release_Request_11,ts,call_ref)
	    )
	 []
	 g !AAL_Establish_Ind;  (*5.5.6.9*)
	     g !AAL_Data_Req !Release_complete(call_ref,cause_41);
	     ( let ts:SetofTimer = ResetallTimer(ts) in 
		   (*Release call reference*)
	    	   exit(NULL,ResetTimer(T313,ts),Indefini)
	     )
	   
	[]
	 g !AAL_Release_Ind;  (*5.5.6.10*)
	  (*release call reference *)
	  exit(NULL,ResetTimer(T313,ts),Indefini)
   )
[]
    [S = Incoming_Call_Proceeding_9 ] ->
	(
	    g !AAL_Data_Req !Connect(call_ref); (*5.5.2.6*)
	      ( let ts:SetofTimer = StartTimer(T313,ts) in
	         exit(Connect_Request_8,ts,call_ref)
	       )
	[]
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	  [isRelease(pdu)]->   (*5.5.4.4*)
		( 		  
	     	  exit(Release_Ind_12,ts,call_ref)
		)
		[]
	  [isRelease_Complete(pdu)]-> (*5.5.6.4*)
		( 
	     	  (*release call reference *)
	     	  exit(NULL,ts,Indefini)
		)
		[]
	  [isSetup(pdu)]->   (* no action is taken*)
		(
		  exit(S,ts,call_ref)
		)
		[]
	[isCall_Proceeding(pdu) or isConnect(pdu) or 
				isConnectAck(pdu) or isRestart_Ack(pdu) ]->  (*5.5.6.4*)
		( 
		   g !AAL_Data_Req !Status(call_ref,S,cause_101);
		   exit(S,ts,call_ref)
		)
		[]
	[isUnregonized_message(pdu)]->   (*Unregonized message *)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
		[]
	  [isStatus_Enquiry(pdu)]->
		( g !AAL_Data_Req !Status(call_ref,S);
	     	  exit(S,ts,call_ref)
		)
	 	 []
	  [isStatus(pdu)]->  (*5.5.6.12*)
	   	 ( let cs:Layer3_States = get_call_state(pdu) in
			 [isNULL(cs)]->  (*5.5.6.12 c*)
			   (*Release call reference*)
	 		   exit(NULL,ts,Indefini)
				[]
			[not(isNULL(cs))]->    (* other cases depend on the implementation*)
			     exit(S,ts,call_ref)
		  )

		[]
	  [isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(NULL,ts,Indefini)
		)

	  )
	[]
	  g !AAL_Data_Req !Release(call_ref);   (*5.5.4.1*)
	    ( let ts:SetofTimer = StartTimer(T308,ts) in
		  exit(Release_Request_11,ts,call_ref)
	    )
	[]
	 g !AAL_Establish_Ind;    (*5.5.6.9*)
	     g !AAL_Data_Req !Release_complete(call_ref,cause_41);
	     ( let ts:SetofTimer = ResetallTimer(ts) in 
		   (*Release call reference*)
	    	   exit(NULL,ts,Indefini)
	     )
	[]
	 g !AAL_Release_Ind;  (*5.5.6.10*)
	    (*release call reference and connection id*)
	    exit(NULL,ts,Indefini)
)

[]
    [ S = Active_10 ] ->
	(
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	   
	   [isStatus_Enquiry(pdu)]->   (*5.5.6.11*)
		( g !AAL_Data_Req ! Status(call_ref,S,cause_30);
	  	  exit(S,ts,call_ref)
		)
		[]
	  [isStatus(pdu)]->  (*5.5.6.11 , 5.5.6.12*)
	   	( let cs:Layer3_States = get_call_state(pdu) in 
		   let c:Cause = get_cause(pdu) in
		     [isNULL(cs)]->
		 	   (*Release Call reference and connection id*)
		 	    exit(NULL,ResetallTimer(ts),Indefini)
	     	     	[]
		      [not(isNULL(cs)) and not(isCause_30(c))]->
			  exit(S,ts,call_ref)
			[]
		     [not(isNULL(cs)) and isCause_30(c)]->  
			    exit(S,ResetTimer(T322,ts),call_ref)
		)
		[]
	  [isRelease(pdu)]->  (*5.5.4.4*)
		( let cr:Nat = get_Call_ref(pdu) in
		    [cr eq call_ref]-> 
			exit(Release_Ind_12,ts,call_ref) 	 		  	   
			[]
		   [cr ne call_ref]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ts,Indefini)
		   
		)
		[]
	  [isRelease_Complete(pdu)]-> (* 5.5.6.4 eg after a bad STATUS*)
		( 
	     	  (*release call reference *)
	     	  exit(NULL,ts,call_ref)
		)
		[]
	  [isSetup(pdu)]->   (* no action is taken 5.5.6.3.2 d *)
		(
		  exit(S,ts,call_ref)
		)
		[]
	[isCall_Proceeding(pdu) or isConnect(pdu) or isConnectAck(pdu) or  isRestart_Ack(pdu)]->  (*5.5.6.4*)
		( 
		   g !AAL_Data_Req !Status(call_ref,S,cause_101);
		   exit(S,ts,call_ref)
		)
		[]
	[isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(NULL,ts,Indefini)
		)
		[]
	[isUnregonized_message(pdu)]->   (*Unregonized message 5.5.6.4  *)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
	)
	 
 	[]
	  g !AAL_Data_Req !Release(call_ref);  (*5.5.4.1*)
	    ( let ts:SetofTimer = StartTimer(T308,ts) in
		  exit(Release_Request_11,ts,call_ref)
	    )
	[]
	  g !AAL_Establish_Ind;      (*5.5.6.9*)  (*Remain in current State*)
	    g !AAL_Data_Req !Status_Enquiry(Call_ref);
	    exit(S,ts,call_ref)
	[]
	  g !AAL_Release_Ind;      (*5.5.6.10*)  (* SALL failure*)
	    g !AAL_Establish_Req;
	    exit(S,StartTimer(T309,ts),call_ref)
	[]
	  [ActiveTimer(T309,ts)]->
	  	g !AAL_Establish_Conf;   (*5.5.6.10*)
	     	( let ts:SetofTimer = ResetTimer(T309,ts) in 
		  	g !AAL_Data_Req !Status_enquiry(call_ref);  (*5.5.6.11*)
		   	     exit(S,StartTimer(T322,ts),call_ref)
	      	)
	[]
	  [ActiveTimer(T309,ts) and (ValTimer(T309,ts) eq 0)]->   
				(*Timeout T309*)    (* 5.5.6.10 ,SALL failure indication *)
		L3_Timer !T309_Expired1;
		  (*Release call reference*)
		  exit(NULL,ResetallTimer(ts),Indefini)
		
	[]
	  [ActiveTimer(T322,ts) and (ValTimer(T322,ts) eq 0)]->  (*Timeout T322*)
		  L3_Timer !T322_Expired1;
		  g !AAL_Data_Req !Status_enquiry(call_ref);  (*5.5.6.11*)
		   exit(S,StartTimer(T322,ts),call_ref) (*the number of retransmission is implementation depend *)
	)
 []
    [S = Release_Request_11 ] ->
	(
	  g !AAL_Data_Ind ?pdu:Layer3_PDU;
	  (
	 [isSetup(pdu)]->   (* no action is taken 5.5.6.3.2 d *)
		(
		  exit(S,ts,call_ref)
		)
		[]
	[isCall_Proceeding(pdu) or isConnect(pdu) or isConnectAck(pdu)]->  (*5.5.6.4*)
		( 
		   g !AAL_Data_Req !Status(call_ref,S,cause_101);
		   exit(S,ts,call_ref)
		)
		[]
	[isUnregonized_message(pdu)]->   (*Unregonized message 5.5.6.4 *)
		(  g !AAL_Data_Req ! Status(call_ref,S,cause_97);
		     exit(S,ts,call_ref)
		)
		[]
	  [isRelease_Complete(pdu)]->  (*5.5.4.3*)
		( 
		  let ts:SetofTimer = ResetTimer(T308,ts) in
	    	     (* release call reference *)
	   	     exit(NULL,ts,Indefini)
		)
		[]
	  [isRelease(pdu)]->  (* Release collision 5.5.4.5*)  
		( let cr:Nat = get_Call_ref(pdu) in
		    [cr eq call_ref]-> 
			(*Release call reference*)
		        exit(NULL,ResetTimer(T308,ts),Indefini)	 		  	   
			[]
		   [cr ne call_ref]-> (*5.5.6.3.2*)
			 g !AAL_Data_Req !Release_Complete(call_ref);
			 (*Release call reference*)
			   exit(NULL,ts,Indefini)
		  )
		[]
	  [isStatus(pdu)]->  (*5.5.6.12*)
	   	( let cs:Layer3_States = get_call_state(pdu) in
	    		[ isNULL(cs) ]->
				(*Release call reference *)
				exit(NULL,ts,Indefini)
			[]
	    		[ not(isNULL(cs))]-> 
	 	 		exit(S,ts,call_ref)
 	  	) 
		[]
	  [isRestart(pdu)]->    (*5.5.5.2*)
		(
		  g !AAL_Data_Req ! Restart_Ack(Global_call_ref);
	  	  exit(NULL,ts,Indefini)
		)
		[]
	  [isStatus_Enquiry(pdu)]->    (*5.5.6.11*)
		( g !AAL_Data_Req !Status(call_ref,S,cause_30);
	     	  exit(S,ts,call_ref)
		)
	  )
	[]
	  [ActiveTimer(T308,ts) and (ValTimer(T308,ts) eq 0)]-> (*Timeout T308*) (*5.5.4.3*)
	           L3_Timer !T308_Expired1;  (*T308!Expired1*)
	           g !AAL_Data_Req !Release(call_ref);
		    ( let ts:SetofTimer = StartTimer(T308,ts) in
			 let ts:SetofTimer = IncrTimer(T308,ts) in	
     	        	 exit(Release_Request_11,ts,call_ref)
		    )
	[]
	  [ ActiveTimer(T308,ts) and (ValTimer(T308,ts) ne 0)]-> 
		 L3_Timer !T308_Expired2;     (*T308!Expired2*) 
	          (* on met le circuit en maintenance *)
		  g !AAL_Data_Req !Restart(Global_call_ref);
	           exit(Restart_Request,StartTimer(T316,ts),call_ref)	  
	[]
	  g !AAL_Establish_Ind;  (*5.5.6.9*)  (*SAAL failure no action is taken*)
	     exit(S,ts,call_ref)
	[]
	  g !AAL_Release_Ind;     (*5.5.6.10*)  (*SAAL failure, clearing internally*)
	    (*release call reference*)
	    exit(NULL,ts,Indefini)
    )
[]
    [S = Release_Ind_12 ]->
	(
	  g !AAL_Data_Req !Release_complete(call_ref); (*5.5.4.4*)
	     exit(NULL,ResetallTimer(ts),Indefini)
	 
	)
[]
    [S = Restart_Request ]->      (*5.5.5.1*)
	(
	  g !AAL_Data_Ind ! Restart_Ack(Global_call_ref);
	     (*Release call reference and virtual channel for reuse*)
	     exit(NULL,ResetTimer(T316,ts),Indefini)
	 []
	   [ActiveTimer(T316,ts) and (ValTimer(T316,ts) eq 0)]->
		L3_Timer !T316_Expired1;
		g !AAL_Data_Req !Restart(Global_call_ref);
		( let ts:SetofTimer = IncrTimer(T308,ts) in	
	      		exit(S,StartTimer(T316,ts),call_ref)
		)
	[]
	  [ ActiveTimer(T316,ts) and (ValTimer(T316,ts) ne 0)]->     (*T316!Expired2*)
	       		(* on met le circuit en maintenance *)
	       	        (* release call reference *)
	       		exit(NULL,ts,Indefini)
	  )


endproc (*Layer3_Action *)

  
  
Endspec (*Q2931_specification*)
