specification SSCF_specification[SAAL_SAP,SSCF_SSCOP]  : noexit

library
	NaturalNumber, Boolean, BitString, Bit, DecDigit
endlib


(*******************************************************************)
(** Datatypes declaration                                         **)
(*******************************************************************)

type SSCF_States is
	sorts SSCF_States
	opns
	  State1_1, 
	  State2_2, 
	  State2_5,
	  State3_4,
	  State4_10	 : -> SSCF_States
endtype (* SSCF_States *)

(* Primitives between the Layer3 and the SSCF *)
type AAL_SP is boolean
	sorts AAL_SP
	opns 
	  AAL_Establish_Req,
	  AAL_Establish_Ind, 
	  AAL_Establish_Conf,  
	  AAL_Release_Req, 
	  AAL_Release_Ind, 
	  AAL_Release_Conf,
	  AAL_Data_Req, 
	  AAL_Data_Ind,
	  AAL_UnitData_Req, 
	  AAL_UnitData_Ind	:	-> AAL_SP
endtype

(* AA-signals between the SSCF and the SSCOP *)
type AA_Signal is boolean,  DecDigit
	sorts AA_Signal
	opns 
	  AA_Establish_Req,
	  AA_Establish_Ind, 
	  AA_Establish_Conf, 
	  AA_Establish_Resp,
 	  AA_Resync_Req,
	  AA_Resync_Ind,
	  AA_Resync_Conf,
	  AA_Resync_Resp,
	  AA_Release_Req,
	  AA_Release_Ind,
	  AA_Release_Conf,
	  AA_Recover_Req, 
	  AA_Recover_Ind, 
	  AA_Recover_Conf,
	  AA_Recover_Resp,
	  AA_Data_Req, 
	  AA_Data_Ind,
	  AA_UnitData_Req, 
	  AA_UnitData_Ind	: -> AA_Signal
endtype

(*******************************************************************)
(** End of data types declaration                                 **)
(*******************************************************************)


behaviour

     (	layer[SAAL_SAP]

		|[SAAL_SAP]| 

	SSCF[SAAL_SAP, SSCF_SSCOP](State1_1)

		|[SSCF_SSCOP]|  

	sscop[SSCF_SSCOP]
    )

where

    	process layer[Q] : noexit :=
 	    Q ?p : primitives_SAAL_SAP;
	    layer[Q]	
 	endproc (* layer *)


 process sscop[G]: noexit :=
    	G ?p : primitives_SAAL_SAP;
	sscop[G]
  endproc (* sscop*)



(*******************************************************************)
(** Specification of the SSCF                                     **)
(*******************************************************************)

 process SSCF[H_SAP,L_SAP] (S : SSCF_States) : noexit :=

  [S = State1_1]->
    (  
      H_SAP !AAL_Establish_Req;(* !Parameter_Data*)
	L_SAP !AA_Establish_Req;(*BR = Yes, SSCOP-UU= Parameter_Data *)
	SSCF[H_SAP,L_SAP](STATE2_2)
    []
      H_SAP !AAL_Release_Req;  (*Parametre_Data *)
	H_SAP !AAL_Release_Conf;
	SSCF[H_SAP,L_SAP](STATE1_1)
    []
      H_SAP !AAL_Data_Req; (*Parametre_Data *)
	SSCF[H_SAP,L_SAP](State1_1)
    []
       H_SAP !AAL_UnitData_Req; (*Parametre_Data *)
	 L_SAP !AA_UnitData_Req; (* MU = Parameter_Data*)
	 SSCF[H_SAP,L_SAP](State1_1)
    []
       L_SAP !AA_Establish_Ind; (* SSCOP-UU*)
	  L_SAP ! AA_Establish_Resp;(* BR= Yes, SSCOP-UU =null*)
	  H_SAP !AAL_Establish_Ind;(* Parameter_data = SSCOP-UU*)
	  SSCF[H_SAP,L_SAP](State4_10)
    []
 	L_SAP !AA_UnitData_Ind;(* MU *)
	  H_SAP !AAL_UnitData_Ind; (* Parameter_Data = MU *)
	  SSCF[H_SAP,L_SAP](State1_1) 
   )
[]
 [S = State2_2]->
   ( 
       H_SAP !AAL_Release_Req; (*Parameter_Data *)
	L_SAP !AA_Release_Req; (* SSCOP-UU = Parameter_Data *)
	SSCF[H_SAP,L_SAP](STATE3_4)
    []
       H_SAP !AAL_UnitData_Req; (*Parameter_Data *)
	 L_SAP !AA_UnitData_Req; (*MU = Parameter_Data *)
	 SSCF[H_SAP,L_SAP](State2_2)
    []
       L_SAP !AA_Establish_Conf; (*SSCOP-UU *)
	  H_SAP !AAL_Establish_Conf; (*Parameter_Data = SSCOP-UU *)
	  SSCF[H_SAP,L_SAP](State4_10)  
    []
 	L_SAP !AA_Release_Ind; (* SSCOP-UU, source*)
	  H_SAP !AAL_Release_Ind; (* Parameter_Data = SSCOP-UU*)
	  SSCF[H_SAP,L_SAP](State1_1) 
     []
       L_SAP !AA_UnitData_Ind; (* MU *) 
	H_SAP !AAL_UnitData_Ind; (* Parameter_Data =MU*)
	SSCF[H_SAP,L_SAP](State2_2)
  )
[]
 [S = State4_10] -> 
   (  
       H_SAP !AAL_Establish_Req; (* Parameter_Data *)
	L_SAP !AA_Resync_Req;    (*SSCOP-UU = Parameter_Data *)
	SSCF[H_SAP,L_SAP](STATE2_5)
     
    []
       H_SAP !AAL_Release_Req; (* Parameter_Data *)
	L_SAP !AA_Release_Req;  (* SSCOP-UU =Parameter_Data *)
	SSCF[H_SAP,L_SAP](STATE3_4)
     
    []
      H_SAP !AAL_Data_Req ;(* ?pdu:Layer3_PDU Parameter_Data *)
	L_SAP !AA_Data_Req; (* MU =Parameter_Data *)
	SSCF[H_SAP,L_SAP](S)
    []
       H_SAP !AAL_UnitData_Req; (* ?pdu:Layer3_PDU Parameter_Data *)
	 L_SAP !AA_UnitData_Req; (* MU = Parameter_Data *)
	 SSCF[H_SAP,L_SAP](S)
    []
       L_SAP !AA_Recover_Ind;
	  L_SAP ! AA_Recover_Resp;
	  H_SAP !AAL_Establish_Ind; (* Parameter_Data = null*)
	  SSCF[H_SAP,L_SAP](S)
    []
       L_SAP !AA_Release_Ind; (* SSCOP-UU, Source*)
	  H_SAP !AAL_Release_Ind; (*?pdu:Layer3_PDU Parameter_Data = SSCOP-UU *)
	  SSCF[H_SAP,L_SAP](State1_1)

     
    []
 	L_SAP !AA_Data_Ind; (* Mu,SN *)
	  H_SAP !AAL_Data_Ind; (*?pdu:Layer_PDUParameter_Data = MU *)
	  SSCF[H_SAP,L_SAP](State4_10)
    []
	L_SAP !AA_Resync_Ind; (* SSCOP-UU *)
	  L_SAP ! AA_Resync_Resp;
	  H_SAP ! AAL_Establish_Ind; (* Parameter_Data = SSCOP-UU*)
	  SSCF[H_SAP,L_SAP](S)
    []
	L_SAP !AA_UnitData_Ind; (*MU*)
	  H_SAP !AAL_UnitData_Ind; (*Parameter_Data = MU *)
	  SSCF[H_SAP,L_SAP](S)
  )
[]
 [ S = State3_4]->
  ( 
      H_SAP !AAL_Establish_Req; (* Parameter_Data *)
	L_SAP !AA_Establish_Req; (* BR = Yes, SSCOP-UU = Parameter_Data*)
	SSCF[H_SAP,L_SAP](STATE2_2)
     

    []
        H_SAP !AAL_UnitData_Req; (* Parameter_Data *)
	 L_SAP !AA_UnitData_Req; (* MU = Parameter_Data *)
	 SSCF[H_SAP,L_SAP](State3_4)
    []
       L_SAP !AA_Release_Conf;
	  H_SAP ! AA_Release_conf;
	  SSCF[H_SAP,L_SAP](State1_1)
    []
 	L_SAP !AA_UnitData_Ind; (*MU*)
	  H_SAP !AAL_UnitData_Ind; (* Parameter_Data = MU *)
	  SSCF[H_SAP,L_SAP](State3_4)
 )

[]
 [S = State2_5]->
   ( 
      H_SAP !AAL_Release_Req; (* Parameter_Data *)
	L_SAP ! AA_Release_Req; (* SSCOP-UU = Parameter_Data *)
	SSCF[H_SAP,L_SAP](State3_4)
    []
       H_SAP !AAL_UnitData_Req; (* Parameter_Data *)
	 L_SAP !AA_UnitData_Req; (* MU Parameter_Data *)
	 SSCF[H_SAP,L_SAP](S)
    []
       L_SAP !AA_Release_Ind; (* SSCOP_UU, Source)
	  H_SAP ! AAL_Release_Ind; (* Parameter_Data = SSCOP-UU *)
	  SSCF[H_SAP,L_SAP](State1_1)
    []
	L_SAP ! AA_Resync_Conf;
	  H_SAP !AAL_Establish_Conf; (*Parameter_Data = null *)
	  SSCF[H_SAP,L_SAP](State4_10)
    []
 	L_SAP !AA_UnitData_Ind; (*MU*)
	  H_SAP !AAL_UnitData_Ind; (* Parameter_Data = Mu *)
	  SSCF[H_SAP,L_SAP](S)
   )

 endproc (* SSCF*)

   
  endspec (* une_etape *)
		
		
		
