Specification SSCOP_spec [SSCF_1, CPCS_1, SSCF_2, CPCS_2] : noexit

library Boolean, BasicNaturalNumber endlib


(*******************************************************************)
(** Datatypes declaration                                         **)
(*******************************************************************)

(* The following DataTypes describe the different primitives 
 * that are provided by each layer / sublayer.
 *)

(*******************************************************************)

type SAAL_SAP_SP is

(* The Signalling AAL Service Access Point Primitives 
 * These primitives are accessible by the layer 3 protocol.
 *)

	sorts SAAL_SAP_SP
	opns 
		AAL_Establish_Req,
		AAL_Establish_Ind, 
		AAL_Establish_Conf,  
		AAL_Release_Req, 
		AAL_Release_Ind, 
		AAL_Release_Conf,
		AAL_Data_Req,
		AAL_Data_Ind,
		AAL_UnitData_Req, 
		AAL_UnitData_Ind	: -> SAAL_SAP_SP
endtype

(*******************************************************************)

Type SSCOP_SP is

(* The SSCOP Service Primitives
 * These prmitives are used to communicate between the SSCOP sublayer
 * and the SSCF sublayer.
 *)

	sorts SSCOP_SP
	opns 
		AA_Establish_Req,
		AA_Establish_Ind, 
		AA_Establish_Resp,
		AA_Establish_Conf, 

		AA_Resync_Req,
		AA_Resync_Ind,
		AA_Resync_Resp,
		AA_Resync_Conf,

		AA_Release_Req,
		AA_Release_Ind,
		AA_Release_Conf,

		AA_Recover_Resp, 
		AA_Recover_Ind, 

		AA_Retrieve_Req,
		AA_Retrieve_Ind,
		AA_RetrieveComplete_Ind,

		AA_Data_Req,
		AA_Data_Ind,

		AA_UnitData_Req, 
		AA_UnitData_Ind	: -> SSCOP_SP
EndType

(*******************************************************************)

Type CPCS_SP is

(* These are the messages that can be exchanged between the SSCOP
 * and the SSCF.
 *)

	sorts CPCS_SP
	opns
		CPCS_UnitData_invoke,
		CPCS_UnitData_signal : -> CPCS_SP
EndType

(*******************************************************************)

Type LM_Error_Message is
	sorts LM_Error_Message
	opns
		A,B,C,D,E,F,G,H,II,J,K,L,M,N,O,P,Q,R,S,T: -> LM_Error_Message
EndType
(*******************************************************************)
(** End of data types declaration                                 **)
(*******************************************************************)

behaviour

hide I_SSCOP_1, I_SSCOP_2 in
(
	SSCOP_sublayer [ SSCF_1, CPCS_1, I_SSCOP_1 ]
		|[CPCS_1]|
	CPCS_Sublayer [ CPCS_1, CPCS_2 ]
		|[CPCS_2]|
	SSCOP_sublayer [ SSCF_2, CPCS_2, I_SSCOP_2 ]
		|[SSCF_2]|
	Layer_3_stub [SSCF_2]
)
where

(*******************************************************************)
(** Specification of the CPCS sublayer                            **)
(*******************************************************************)

Process CPCS_Sublayer [CPCS_1, CPCS_2] : noexit :=
(
	(
		CPCS_1 ! CPCS_UnitData_invoke ? pdu:SSCOP_pdu;
		(
			(	CPCS_2 ! CPCS_UnitData_signal ! pdu; stop)
			|||
			CPCS_Sublayer [CPCS_1, CPCS_2]
		)
	)
	[]
	(
		CPCS_2 ! CPCS_UnitData_invoke ? pdu:SSCOP_pdu;
		(
			(	CPCS_1 ! CPCS_UnitData_signal ! pdu; stop)
			|||
			CPCS_Sublayer [CPCS_1, CPCS_2]
		)
	)
)
EndProc

(*******************************************************************)
(** Specification of the Layer 3 stub                             **)
(*******************************************************************)

Process Layer_3_Stub [SSCF] : noexit :=
(
	(
	SSCF ! AA_Establish_ind ? uu:SSCOP_UU;
		(
		SSCF ! AA_Establish_resp ? uu:SSCOP_UU ? br:Bool;
			Layer_3_Stub [SSCF]
		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;
			Layer_3_Stub [SSCF]
		)
	)
	[]
	(
	SSCF ! AA_Release_ind ? uu:SSCOP_UU ? s:SSCOP_source;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_Release_conf;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_Resync_ind ? uu:SSCOP_UU;
		SSCF ! AA_Resync_resp;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_Resync_conf;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_Data_ind ? uu:SSCOP_UU ? SN:Nat;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_UnitData_ind ? uu:SSCOP_UU;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_Retrieve_ind ? uu:SSCOP_UU;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_RetrieveComplete_ind;
		Layer_3_Stub [SSCF]
	)
	[]
	(
	SSCF ! AA_Recover_ind;
		SSCF ! AA_Recover_resp;
		Layer_3_Stub [SSCF]
	)
)
EndProc

(*******************************************************************)
(** Specification of the SSCOP sublayer                           **)
(*******************************************************************)

(*******************************************************************)
(** Public Data types                                             **)
(*******************************************************************)

type NaturalNumber is BasicNaturalNumber,Boolean

	opns 
		pred : Nat -> Nat
		_eq_, _ne_, _lt_, _le_, _gt_, _ge_  : Nat, Nat -> Bool


	eqns forall n,m : Nat

	ofsort Nat

		pred(0) = 0;
		pred(succ(n)) = n;

	ofsort   Bool

		0 eq 0         = true         ;
		0 eq Succ(m)      = false        ;
		Succ(m) eq 0      = false        ;
		Succ(m) eq Succ(n)   = m eq n    ;
		m ne n         = not(m eq n)     ;
		0 lt 0         = false        ;
		0 lt Succ(n)      = true         ;
		Succ(n) lt 0      = false        ;
		Succ(m) lt Succ(n)   = m lt n    ;
		m le n         = (m lt n) or (m eq n)  ;
		m ge n         = not(m lt n)     ;
		m gt n         = not(m le n)     ;

endtype

Type ListOfNat is Boolean,NaturalNumber

	sorts ListOfNat

	opns
		empty : -> ListOfNat
		put : Nat, ListOfNat -> ListOfNat
		get : ListOfNat -> Nat
		remove : ListOfNat -> ListOfNat
		isempty : ListOfNat -> Bool		
		sizeof: ListOfNat -> Nat

	eqns
		forall e:Nat, q:ListOfNat

			ofsort Nat
				get(empty) = 0;
				get(put(e,empty)) = e;
				not(isempty(q)) => get(put(e,q)) = get(q);

				sizeof(empty) = 0;
				sizeof(put(e,q)) = succ(sizeof(q));

			ofsort ListOfNat
				remove(empty) = empty;
				remove(put(e, empty)) = empty;
				not(isempty(q)) => remove(put(e, q)) = put(e, remove(q));

			ofsort Bool
				isempty(empty) = true;
				isempty(put(e,q)) = false;
EndType


Type SSCOP_PDU is ListOfNat, SSCOP_UU, NaturalNumber, SSCOP_Source, LM_Error_Message

	sorts SSCOP_PDU
	opns

		NO_PDU : -> SSCOP_PDU

	(* Connection establishment PDUs *)

		(* Request Initialization, PDU Type field : 0001 *)
		BGN   :	Nat,	SSCOP_UU,	Nat 		-> SSCOP_PDU
			(*		N(MR)					N(SQ)	*)
		
		(* Request Acknowledgment, PDU Type field : 0010 *)
		BGAK	:	Nat,	SSCOP_UU 				-> SSCOP_PDU
			(*		N(MR)	*)

		(* Connection Reject, PDU Type field : 0111 *)
		BGREJ	:	SSCOP_UU							-> SSCOP_PDU
			(* No parameters *)

	(* Connection release PDUs *)

		(* Disconnect Command, PDU Type field : 0011 *) 
		END	:	SSCOP_UU, 	SSCOP_Source	-> SSCOP_PDU
			(*						S	*)

		(* Disconnect Acknowledgment, PDU Type field : 0100 *)
		ENDAK : 										-> SSCOP_PDU
			(* No parameters *)

	(* Resynchronisation *)
			
		(* Resynchronization Command, PDU Type field : 0101 *)
		RS		:	SSCOP_UU,	Nat,	Nat		-> SSCOP_PDU
			(*						N(MR)	N(SQ)	*)

		(* Resynchronization Acknowledgment, PDU Type field : 0110 *)
		RSAK	:	Nat								-> SSCOP_PDU
			(*		N(MR)	*)

	(* Recovery *)

		(* Recovery command, PDU Type field : 1001 *)
		ER		:	Nat,	Nat						-> SSCOP_PDU
			(*		N(MR)	N(SQ)	*)

		(* Recovery Acknowledgment, PDU Type field : 1111 *)
		ERAK	:	Nat								-> SSCOP_PDU
			(*		N(MR)	*)

	(* Assured Data Transfer *)

		(* Sequenced Connection-mode Data, PDU Type field : 1000 *)
		SD    : 	Nat, SSCOP_UU 				-> SSCOP_PDU
			(*		N(S)	*)	

		(* Transmitter State Information with request for Receiver State 
		 * Information, PDU Type field : 1010 *)
		POLL  : 	Nat,	Nat 						-> SSCOP_PDU
			(*		N(PS)	N(S) *)

		(* Solicited Receiver State Information, PDU Type field : 1011 *)
		STAT  :	ListOfNat,	Nat,	Nat,	Nat 				-> SSCOP_PDU
			(*						N(R)	N(MR)	N(PS)	*)

		(* Unsolicited Receiver State Information, PDU Type field : 1100 *)
		USTAT : 	ListOfNat,	Nat,	Nat 						-> SSCOP_PDU
			(*						N(R)	N(MR)	*)

	(* Unacknowledged Data Transfer *)

		(* Unnumbered User Data, PDU Type field : 1101 *)
		UD		:	SSCOP_UU 						-> SSCOP_PDU

	(* Management Data Transfer *)
		
		(* Unnumbered Management Data, PDU Type field : 1110 *)
		MD		:	SSCOP_UU 						-> SSCOP_PDU

	(*
	 *	The following operands provide some way to manipulate the SSCOP_PDU's.
	 *	among the functions, there are : is_PDUTYPE(pdu) which returns true
	 *	if pdu if of type PDUTYPE, and some functions to access the content of
	 *	the pdu's.
	 *)

		is_SD, is_POLL, is_STAT, is_USTAT, is_UD, is_MD, is_BGN,
		is_BGAK, is_BGREJ, is_ER, is_ERAK, is_END, is_RS, is_ENDAK,
		is_RSAK : SSCOP_PDU -> Bool
		is_general, is_other : SSCOP_PDU -> Bool

		N_MR : SSCOP_PDU -> Nat
		N_SQ : SSCOP_PDU -> Nat
		N_S : SSCOP_PDU -> Nat
		N_PS : SSCOP_PDU -> Nat
		N_R : SSCOP_PDU -> Nat
		UU : SSCOP_PDU	-> SSCOP_UU
		S : SSCOP_PDU -> SSCOP_Source
		list : SSCOP_PDU -> ListOfNat

		Bad_PDU : SSCOP_PDU -> LM_Error_Message

		is_invalid : SSCOP_PDU -> Bool
		bad_length : SSCOP_PDU -> Bool
		is_NO_PDU : SSCOP_PDU -> Bool

	eqns
	
		forall n1,n2,n3:Nat, uu:SSCOP_UU, s:SSCOP_Source,
				ll:ListOfNat, p:SSCOP_PDU

		ofsort Bool
			is_invalid(p) = is_NO_PDU(p);
			bad_length(p) = false;

			is_SD(SD(n1,uu)) = true;
			is_SD(STAT(ll,n1,n2,n3)) = false;
			is_SD(USTAT(ll,n1,n2)) = false;
			is_SD(UD(uu)) = false;
			is_SD(MD(uu)) = false;
			is_SD(BGN(n1,uu,n2)) = false;
			is_SD(BGAK(n1,uu)) = false;
			is_SD(BGREJ(uu)) = false;
			is_SD(ER(n1,n2)) = false;
			is_SD(ERAK(n1)) = false;
			is_SD(END(uu,s)) = false;
			is_SD(ENDAK) = false;
			is_SD(RS(uu,n1,n2)) = false;
			is_SD(RSAK(n1)) = false;
			is_SD(NO_PDU) = false;
			is_SD(POLL(n1,n2)) = false;

			is_POLL(SD(n1,uu)) = false;
			is_POLL(STAT(ll,n1,n2,n3)) = false;
			is_POLL(USTAT(ll,n1,n2)) = false;
			is_POLL(UD(uu)) = false;
			is_POLL(MD(uu)) = false;
			is_POLL(BGN(n1,uu,n2)) = false;
			is_POLL(BGAK(n1,uu)) = false;
			is_POLL(BGREJ(uu)) = false;
			is_POLL(ER(n1,n2)) = false;
			is_POLL(ERAK(n1)) = false;
			is_POLL(END(uu,s)) = false;
			is_POLL(ENDAK) = false;
			is_POLL(RS(uu,n1,n2)) = false;
			is_POLL(RSAK(n1)) = false;
			is_POLL(NO_PDU) = false;
			is_POLL(POLL(n1,n2)) = true;

			is_STAT(SD(n1,uu)) = false;
			is_STAT(STAT(ll,n1,n2,n3)) = true;
			is_STAT(USTAT(ll,n1,n2)) = false;
			is_STAT(UD(uu)) = false;
			is_STAT(MD(uu)) = false;
			is_STAT(BGN(n1,uu,n2)) = false;
			is_STAT(BGAK(n1,uu)) = false;
			is_STAT(BGREJ(uu)) = false;
			is_STAT(ER(n1,n2)) = false;
			is_STAT(ERAK(n1)) = false;
			is_STAT(END(uu,s)) = false;
			is_STAT(ENDAK) = false;
			is_STAT(RS(uu,n1,n2)) = false;
			is_STAT(RSAK(n1)) = false;
			is_STAT(NO_PDU) = false;
			is_STAT(POLL(n1,n2)) = false;

			is_USTAT(SD(n1,uu)) = false;
			is_USTAT(STAT(ll,n1,n2,n3)) = false;
			is_USTAT(USTAT(ll,n1,n2)) = true;
			is_USTAT(UD(uu)) = false;
			is_USTAT(MD(uu)) = false;
			is_USTAT(BGN(n1,uu,n2)) = false;
			is_USTAT(BGAK(n1,uu)) = false;
			is_USTAT(BGREJ(uu)) = false;
			is_USTAT(ER(n1,n2)) = false;
			is_USTAT(ERAK(n1)) = false;
			is_USTAT(END(uu,s)) = false;
			is_USTAT(ENDAK) = false;
			is_USTAT(RS(uu,n1,n2)) = false;
			is_USTAT(RSAK(n1)) = false;
			is_USTAT(NO_PDU) = false;
			is_USTAT(POLL(n1,n2)) = false;

			is_UD(SD(n1,uu)) = false;
			is_UD(STAT(ll,n1,n2,n3)) = false;
			is_UD(USTAT(ll,n1,n2)) = false;
			is_UD(UD(uu)) = true;
			is_UD(MD(uu)) = false;
			is_UD(BGN(n1,uu,n2)) = false;
			is_UD(BGAK(n1,uu)) = false;
			is_UD(BGREJ(uu)) = false;
			is_UD(ER(n1,n2)) = false;
			is_UD(ERAK(n1)) = false;
			is_UD(END(uu,s)) = false;
			is_UD(ENDAK) = false;
			is_UD(RS(uu,n1,n2)) = false;
			is_UD(RSAK(n1)) = false;
			is_UD(NO_PDU) = false;
			is_UD(POLL(n1,n2)) = false;

			is_MD(SD(n1,uu)) = false;
			is_MD(STAT(ll,n1,n2,n3)) = false;
			is_MD(USTAT(ll,n1,n2)) = false;
			is_MD(UD(uu)) = false;
			is_MD(MD(uu)) = true;
			is_MD(BGN(n1,uu,n2)) = false;
			is_MD(BGAK(n1,uu)) = false;
			is_MD(BGREJ(uu)) = false;
			is_MD(ER(n1,n2)) = false;
			is_MD(ERAK(n1)) = false;
			is_MD(END(uu,s)) = false;
			is_MD(ENDAK) = false;
			is_MD(RS(uu,n1,n2)) = false;
			is_MD(RSAK(n1)) = false;
			is_MD(NO_PDU) = false;
			is_MD(POLL(n1,n2)) = false;

			is_BGN(SD(n1,uu)) = false;
			is_BGN(STAT(ll,n1,n2,n3)) = false;
			is_BGN(USTAT(ll,n1,n2)) = false;
			is_BGN(UD(uu)) = false;
			is_BGN(MD(uu)) = false;
			is_BGN(BGN(n1,uu,n2)) = true;
			is_BGN(BGAK(n1,uu)) = false;
			is_BGN(BGREJ(uu)) = false;
			is_BGN(ER(n1,n2)) = false;
			is_BGN(ERAK(n1)) = false;
			is_BGN(END(uu,s)) = false;
			is_BGN(ENDAK) = false;
			is_BGN(RS(uu,n1,n2)) = false;
			is_BGN(RSAK(n1)) = false;
			is_BGN(NO_PDU) = false;
			is_BGN(POLL(n1,n2)) = false;

			is_ERAK(SD(n1,uu)) = false;
			is_ERAK(STAT(ll,n1,n2,n3)) = false;
			is_ERAK(USTAT(ll,n1,n2)) = false;
			is_ERAK(UD(uu)) = false;
			is_ERAK(MD(uu)) = false;
			is_ERAK(BGN(n1,uu,n2)) = false;
			is_ERAK(BGAK(n1,uu)) = false;
			is_ERAK(BGREJ(uu)) = false;
			is_ERAK(ER(n1,n2)) = false;
			is_ERAK(ERAK(n1)) = true;
			is_ERAK(END(uu,s)) = false;
			is_ERAK(ENDAK) = false;
			is_ERAK(RS(uu,n1,n2)) = false;
			is_ERAK(RSAK(n1)) = false;
			is_ERAK(NO_PDU) = false;
			is_ERAK(POLL(n1,n2)) = false;

			is_NO_PDU(SD(n1,uu)) = false;
			is_NO_PDU(STAT(ll,n1,n2,n3)) = false;
			is_NO_PDU(USTAT(ll,n1,n2)) = false;
			is_NO_PDU(UD(uu)) = false;
			is_NO_PDU(MD(uu)) = false;
			is_NO_PDU(BGN(n1,uu,n2)) = false;
			is_NO_PDU(BGAK(n1,uu)) = false;
			is_NO_PDU(BGREJ(uu)) = false;
			is_NO_PDU(ER(n1,n2)) = false;
			is_NO_PDU(ERAK(n1)) = false;
			is_NO_PDU(END(uu,s)) = false;
			is_NO_PDU(ENDAK) = false;
			is_NO_PDU(RS(uu,n1,n2)) = false;
			is_NO_PDU(RSAK(n1)) = false;
			is_NO_PDU(NO_PDU) = true;
			is_NO_PDU(POLL(n1,n2)) = false;

			is_RSAK(SD(n1,uu)) = false;
			is_RSAK(STAT(ll,n1,n2,n3)) = false;
			is_RSAK(USTAT(ll,n1,n2)) = false;
			is_RSAK(UD(uu)) = false;
			is_RSAK(MD(uu)) = false;
			is_RSAK(BGN(n1,uu,n2)) = false;
			is_RSAK(BGAK(n1,uu)) = false;
			is_RSAK(BGREJ(uu)) = false;
			is_RSAK(ER(n1,n2)) = false;
			is_RSAK(ERAK(n1)) = false;
			is_RSAK(END(uu,s)) = false;
			is_RSAK(ENDAK) = false;
			is_RSAK(RS(uu,n1,n2)) = false;
			is_RSAK(RSAK(n1)) = true;
			is_RSAK(NO_PDU) = false;
			is_RSAK(POLL(n1,n2)) = false;

			is_RS(SD(n1,uu)) = false;
			is_RS(STAT(ll,n1,n2,n3)) = false;
			is_RS(USTAT(ll,n1,n2)) = false;
			is_RS(UD(uu)) = false;
			is_RS(MD(uu)) = false;
			is_RS(BGN(n1,uu,n2)) = false;
			is_RS(BGAK(n1,uu)) = false;
			is_RS(BGREJ(uu)) = false;
			is_RS(ER(n1,n2)) = false;
			is_RS(ERAK(n1)) = false;
			is_RS(END(uu,s)) = false;
			is_RS(ENDAK) = false;
			is_RS(RS(uu,n1,n2)) = true;
			is_RS(RSAK(n1)) = false;
			is_RS(NO_PDU) = false;
			is_RS(POLL(n1,n2)) = false;

			is_ENDAK(SD(n1,uu)) = false;
			is_ENDAK(STAT(ll,n1,n2,n3)) = false;
			is_ENDAK(USTAT(ll,n1,n2)) = false;
			is_ENDAK(UD(uu)) = false;
			is_ENDAK(MD(uu)) = false;
			is_ENDAK(BGN(n1,uu,n2)) = false;
			is_ENDAK(BGAK(n1,uu)) = false;
			is_ENDAK(BGREJ(uu)) = false;
			is_ENDAK(ER(n1,n2)) = false;
			is_ENDAK(ERAK(n1)) = false;
			is_ENDAK(END(uu,s)) = false;
			is_ENDAK(ENDAK) = true;
			is_ENDAK(RS(uu,n1,n2)) = false;
			is_ENDAK(RSAK(n1)) = false;
			is_ENDAK(NO_PDU) = false;
			is_ENDAK(POLL(n1,n2)) = false;

			is_END(SD(n1,uu)) = false;
			is_END(STAT(ll,n1,n2,n3)) = false;
			is_END(USTAT(ll,n1,n2)) = false;
			is_END(UD(uu)) = false;
			is_END(MD(uu)) = false;
			is_END(BGN(n1,uu,n2)) = false;
			is_END(BGAK(n1,uu)) = false;
			is_END(BGREJ(uu)) = false;
			is_END(ER(n1,n2)) = false;
			is_END(ERAK(n1)) = false;
			is_END(END(uu,s)) = true;
			is_END(ENDAK) = false;
			is_END(RS(uu,n1,n2)) = false;
			is_END(RSAK(n1)) = false;
			is_END(NO_PDU) = false;
			is_END(POLL(n1,n2)) = false;

			is_ER(SD(n1,uu)) = false;
			is_ER(STAT(ll,n1,n2,n3)) = false;
			is_ER(USTAT(ll,n1,n2)) = false;
			is_ER(UD(uu)) = false;
			is_ER(MD(uu)) = false;
			is_ER(BGN(n1,uu,n2)) = false;
			is_ER(BGAK(n1,uu)) = false;
			is_ER(BGREJ(uu)) = false;
			is_ER(ER(n1,n2)) = true;
			is_ER(ERAK(n1)) = false;
			is_ER(END(uu,s)) = false;
			is_ER(ENDAK) = false;
			is_ER(RS(uu,n1,n2)) = false;
			is_ER(RSAK(n1)) = false;
			is_ER(NO_PDU) = false;
			is_ER(POLL(n1,n2)) = false;

			is_BGREJ(SD(n1,uu)) = false;
			is_BGREJ(STAT(ll,n1,n2,n3)) = false;
			is_BGREJ(USTAT(ll,n1,n2)) = false;
			is_BGREJ(UD(uu)) = false;
			is_BGREJ(MD(uu)) = false;
			is_BGREJ(BGN(n1,uu,n2)) = false;
			is_BGREJ(BGAK(n1,uu)) = false;
			is_BGREJ(BGREJ(uu)) = true;
			is_BGREJ(ER(n1,n2)) = false;
			is_BGREJ(ERAK(n1)) = false;
			is_BGREJ(END(uu,s)) = false;
			is_BGREJ(ENDAK) = false;
			is_BGREJ(RS(uu,n1,n2)) = false;
			is_BGREJ(RSAK(n1)) = false;
			is_BGREJ(NO_PDU) = false;
			is_BGREJ(POLL(n1,n2)) = false;

			is_BGAK(SD(n1,uu)) = false;
			is_BGAK(STAT(ll,n1,n2,n3)) = false;
			is_BGAK(USTAT(ll,n1,n2)) = false;
			is_BGAK(UD(uu)) = false;
			is_BGAK(MD(uu)) = false;
			is_BGAK(BGN(n1,uu,n2)) = false;
			is_BGAK(BGAK(n1,uu)) = true;
			is_BGAK(BGREJ(uu)) = false;
			is_BGAK(ER(n1,n2)) = false;
			is_BGAK(ERAK(n1)) = false;
			is_BGAK(END(uu,s)) = false;
			is_BGAK(ENDAK) = false;
			is_BGAK(RS(uu,n1,n2)) = false;
			is_BGAK(RSAK(n1)) = false;
			is_BGAK(NO_PDU) = false;
			is_BGAK(POLL(n1,n2)) = false;

			is_general(p) = (is_UD(p) or is_MD(p) or is_NO_PDU(p));
			is_other(p) = not(is_general(p));

		ofsort Nat
			N_MR(BGN(n1, uu, n2)) = n1;
			N_MR(RSAK(n1)) = n1;
			N_MR(BGAK(n1, uu)) = n1;
			N_MR(RS(uu, n1, n2)) = n1;
			N_MR(ER(n1, n2)) = n1;
			N_MR(ERAK(n1)) = n1;
			N_MR(STAT(ll,n1,n2,n3)) = n2;
			N_MR(USTAT(ll, n1,n2)) = n2;
			(is_SD(p) or is_ENDAK(p) or is_POLL(p) or is_UD(p) or is_MD(p) 
				or is_NO_PDU(p) or is_BGREJ(p) or is_END(p)) =>
					N_MR(p) = 0;

			N_SQ(BGN(n1, uu, n2)) = n2;
			N_SQ(RS(uu, n1, n2)) = n2;
			N_SQ(ER(n1, n2)) = n2;
			(is_SD(p) or is_ENDAK(p) or is_POLL(p) or is_STAT(p) or is_USTAT(p)
				or is_UD(p) or is_MD(p) or is_BGAK(p) or is_BGREJ(p) or is_ERAK(p)
				or is_RSAK(p) or is_NO_PDU(p) or is_END(p)) =>
					N_SQ(p) = 0;

			N_S(SD(n1,uu)) = n1;
			N_S(POLL(n1,n2)) = n2;
			(is_ENDAK(p) or is_STAT(p) or is_USTAT(p) or is_BGN(p) or is_ER(p)
				or is_UD(p) or is_MD(p) or is_BGAK(p) or is_BGREJ(p) or is_ERAK(p)
				or is_RSAK(p) or is_NO_PDU(p) or is_END(p) or is_RS(p) ) =>
					N_SQ(p) = 0;

			N_PS(POLL(n1,n2)) = n1;
			N_PS(STAT(ll,n1,n2,n3)) = n3;
			(is_ENDAK(p) or is_USTAT(p) or is_BGN(p) or is_ER(p)
				or is_UD(p) or is_MD(p) or is_BGAK(p) or is_BGREJ(p) or is_ERAK(p)
				or is_RSAK(p) or is_NO_PDU(p) or is_END(p) or is_RS(p) 
				or is_SD(p)) =>
					N_SQ(p) = 0;

			N_R(STAT(ll, n1,n2,n3)) = n1;
			N_R(USTAT(ll, n1, n2)) = n1;
			(is_ENDAK(p) or is_BGN(p) or is_ER(p) or is_UD(p) or is_MD(p) 
				or is_BGAK(p) or is_BGREJ(p) or is_ERAK(p) or is_RSAK(p) 
				or is_NO_PDU(p) or is_END(p) or is_RS(p) or is_SD(p) 
				or is_POLL(p)) =>
					N_SQ(p) = 0;

		ofsort SSCOP_UU
			UU(BGN(n1,uu,n2)) = uu;
			UU(END(uu,s)) = uu;
			UU(RS(uu, n1, n2)) = uu;
			UU(BGAK(n1,uu)) = uu;
			UU(BGREJ(uu)) = uu;
			UU(SD(n1,uu)) = uu;
			UU(UD(uu)) = uu;
			UU(MD(uu)) = uu;
			(is_ENDAK(p) or is_ER(p) or is_ERAK(p) or is_RSAK(p) 
				or is_NO_PDU(p) or is_POLL(p) or is_STAT(p) or is_USTAT(p)) =>
					UU(p) = null_uu;

		ofsort SSCOP_Source
			S(END(uu, s)) = s;
			(is_ENDAK(p) or is_BGN(p) or is_ER(p) or is_UD(p) or is_MD(p) 
				or is_BGAK(p) or is_BGREJ(p) or is_ERAK(p) or is_RSAK(p) 
				or is_NO_PDU(p) or is_RS(p) or is_SD(p) or is_STAT(p)
				or is_POLL(p) or is_USTAT(p) ) =>
					S(p) = source_USER;

		ofsort ListOfNat
			list(STAT(ll,n1,n2,n3)) = ll;
			list(USTAT(ll,n1,n2)) = ll;
			(is_ENDAK(p) or is_BGN(p) or is_ER(p) or is_UD(p) or is_MD(p) 
				or is_BGAK(p) or is_BGREJ(p) or is_ERAK(p) or is_RSAK(p) 
				or is_NO_PDU(p) or is_RS(p) or is_SD(p) or is_END(p)
				or is_POLL(p)) =>
					list(p) = empty of ListOfNat;

		ofsort LM_Error_Message
			Bad_PDU(SD(n1,uu)) = A;
			Bad_PDU(BGN(n1, uu, n2)) = B;
			Bad_PDU(BGAK(n1, uu)) = C;
			Bad_PDU(BGREJ(uu)) = D;
			Bad_PDU(END(uu,s)) = E;
			Bad_PDU(ENDAK) = F;
			Bad_PDU(POLL(n1,n2)) = G;
			Bad_PDU(STAT(ll, n1, n2, n3)) = H;
			Bad_PDU(USTAT(ll, n1,n2)) = II;
			Bad_PDU(RS(uu,n1,n2)) = J;
			Bad_PDU(RSAK(n1)) = K;
			Bad_PDU(ER(n1,n2)) = L;
			Bad_PDU(ERAK(n1)) = M;

EndType			


Type SSCOP_UU is
	sorts SSCOP_UU
	opns
		null_uu : -> SSCOP_UU
		uu00,uu01,uu02,uu03,uu04,uu05 : -> SSCOP_UU
(*		uu06,uu07,uu08,uu09: ->SSCOP_UU*)
(*		uu10,uu11,uu12,uu13,uu14,uu15,uu16,uu17,uu18,uu19: ->SSCOP_UU*)
EndType

Type SSCOP_BR is Boolean RenamedBy
	sortnames
		SSCOP_BR for Bool
	opnnames
		yes for true
		no for false
EndType

Type SSCOP_Source is
	sorts SSCOP_Source
	opns
		source_USER,
		source_SSCOP : -> SSCOP_Source
EndType

Type AA_RN is NaturalNumber, Boolean
	sorts
		AA_RN
	opns
		Unknown : -> AA_RN
		Total : -> AA_RN
		val : Nat -> AA_RN
		get : AA_RN -> Nat
		_ eq _ : AA_RN, AA_RN -> Bool

	eqns
		forall a:AA_RN, n:Nat

		ofsort Nat
			get(val(n)) = n;
			get(Unknown) = 0;
			get(Total) = 0;

		ofsort Bool
			Unknown eq Unknown = true;
			Total eq Unknown = false;
			val(n) eq Unknown = false;

			Unknown eq Total = false;
			Total eq Total = true;
			val(n) eq Total = false;
	
EndType

(*******************************************************************)
(** Process definition                                            **)
(*******************************************************************)

Process SSCOP_SubLayer [SSCF, CPCS, I_SSCOP] : NoExit :=

	hide SSCOP_I in
	(
		SSCOP_Internals [SSCOP_I] (0,0,0)
			|[SSCOP_I]|

		SSCOP[SSCF , CPCS, SSCOP_I, I_SSCOP] (Idle_1, (* initial state *)
			SenderSet(0,0,0,0,0,0,0,0),
			RecSet(0,0,0,0),
			ParamSet(succ(succ(0)),succ(succ(0)),succ(succ(0)),false,false,succ(succ(0)),succ(succ(succ(0)))),
			TimerSet(0,false,0,false,0,false,0,false,0,false),
			pdu_set(NO_PDU, NO_PDU, NO_PDU, NO_PDU, NO_PDU, NO_PDU),
			empty of UU_Queue,
			empty of UU_Queue,
			empty of UU_Queue,
			empty of TxBuffer,
			empty of PDU_Queue,
			empty of RxBuffer)
	)
	Where

(****************************************************************************)

	Process SSCOP_Internals [SSCOP_I] (count_SD,count_UD,count_MD:Nat) : noexit :=
	(
		SSCOP_I ! QueueUp_SD_PDU;
			SSCOP_Internals [SSCOP_I] (succ(count_SD), count_UD, count_MD)
		[]
		[ count_SD gt 0 ] ->
			SSCOP_I ! SD_PDU_QueuedUp;
			SSCOP_Internals [SSCOP_I] (pred(count_SD), count_UD, count_MD)
		[]
		SSCOP_I ! QueueUp_UD_PDU;
			SSCOP_Internals [SSCOP_I] (count_SD, succ(count_UD), count_MD)
		[]
		[ count_UD gt 0 ] ->
			SSCOP_I ! UD_PDU_QueuedUp;
			SSCOP_Internals [SSCOP_I] (count_SD, pred(count_UD), count_MD)
		[]
		SSCOP_I ! QueueUp_MD_PDU;
			SSCOP_Internals [SSCOP_I] (count_SD, count_UD, succ(count_MD))
		[]
		[ count_MD gt 0 ] ->
			SSCOP_I ! MD_PDU_QueuedUp;
			SSCOP_Internals [SSCOP_I] (count_SD, count_UD, pred(count_MD))

	)
	EndProc (* SSCOP_Internals *)

(****************************************************************************)

	Process SSCOP [SSCF, CPCS, SSCOP_I, I_SSCOP] (s:SSCOP_state, ss,rs,ps,ts:SSCOP_var,
									sp:Set_of_PDU, SD_Q,UD_Q,MD_Q: UU_Queue,
									TxBuf:TxBuffer, ReTxQ:PDU_Queue, RxBuf:RxBuffer) : 
							noexit :=
	(
			SSCOP_Action [SSCF, CPCS, SSCOP_I, I_SSCOP] 
					(s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			>> accept s:SSCOP_state, ss:SSCOP_var,rs:SSCOP_var,ps:SSCOP_var,
						ts:SSCOP_var, sp:Set_of_PDU,
						SD_Q:UU_Queue, UD_Q:UU_Queue, MD_Q:UU_Queue, TxBuf:TxBuffer,
						ReTxQ:PDU_Queue,
						RxBuf:RxBuffer in
			SSCOP [SSCF, CPCS, SSCOP_I, I_SSCOP] 
					(s, ss, rs, ps, DecAllTimers(ts), sp, SD_Q, UD_Q, MD_Q, TxBuf,
					 ReTxQ, RxBuf)
	)
	EndProc (* SSCOP *)

(****************************************************************************)

	Process SSCOP_Action [SSCF, CPCS, SSCOP_I, I_SSCOP] 
				(s : SSCOP_STATE, ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU,
				SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
				RxBuf:RxBuffer) : 
	exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
			Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
			RxBuffer) :=

	(

	SSCF ! AA_UnitData_req ? mu:SSCOP_UU;
		(* put AA-UnitData_req(MU) into Unassured queue *)
		( let UD_Q:UU_Queue = put(mu,UD_Q) in

		SSCOP_I ! QueueUp_UD_PDU;
		exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
	[]
	(* LM ! MAA_UnitData_req ? mu:SSCOP_UU *)
		(* Put MAA-UnitDataRequest(MU) into management queue *)
		(*let MD_Q:UU_Queue = put(mu, MD_Q) in*)
		(*SSCOP_I ! QueueUp_MD_PDU;*)
		(*exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)*)
	
(*	[]*)
	CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_general(pdu) ];
		(
		[ is_UD(pdu) ] ->
			SSCF ! AA_UnitData_ind ! UU(pdu);
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
		[]
		[ is_MD(pdu) ] ->
			(* LM ! MAA_UnitData_ind ! UU(pdu); *)
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
		[]
		[ is_invalid(pdu) ] ->
			(
			[ bad_length(pdu) ] ->
				(* LM ! MAA_Error_ind ! U; *)
				exit(s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			[]
			[ not(bad_length(pdu)) ] ->
				exit(s, ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			)
		)
	[]
	SSCOP_I ! UD_PDU_QueuedUp;
		(* LowerLayer busy = NO *)
		(* Remove MU from Unassured queue *)
		( let mu:SSCOP_UU = get(UD_Q),
			UD_Q:UU_Queue = remove(UD_Q) in

		CPCS ! CPCS_UnitData_invoke ! UD(mu);
		exit(s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
	[]
	SSCOP_I ! MD_PDU_QueuedUp;
		(* LowerLayer Busy = NO *)
		(* Remove MU from Management queue *)
		(* CPCS ! CPCS_UnitData_invoke ! MD(mu); *)
		( let mu:SSCOP_UU = get(MD_Q),
			MD_Q:UU_Queue = remove(MD_Q) in

		CPCS ! CPCS_UnitData_invoke ! MD(mu);

		exit(s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

	(*************************************************************************)

	[]
	[ s = Idle_1 ] ->
		(
			SSCF ! AA_Establish_Req ? uu:SSCOP_UU ? br:Bool;
			(
				(* Clear Transmitter *)
				Clear_Transmitter  (ps, SD_Q, TxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer in

				( let
				(* Clear-buffers := br *)
					ps:SSCOP_var = SetParamB(ClearBuffers, ps, br),
				(* VT(CC) := 1 *)
				(* VT(SQ) := VT(SQ) + 1 *)
					ss:SSCOP_var = SetVT(CC, IncVT(SQ, ss), succ(0)),
				(* initialize VR(MR) *)
					rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

				(* remember the pdu we send *)
				let
					sp:Set_of_PDU = add(BGN_pdu,BGN(VR(MR,rs),uu,VT(SQ,ss)),sp) in

				CPCS ! CPCS_UnitData_invoke ! BGN(VR(MR,rs), uu, VT(SQ,ss));

				(* Set Timer_CC *)
				( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

				exit (Outgoing_Connection_Pending_2, ss, rs, ps, ts, sp, 
						SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ) )
			)
			[]
			CPCS ! CPCS_UnitData_signal ? pdu: SSCOP_PDU [ is_other(pdu) ];
			(
				
				[ is_BGREJ(pdu) ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)

				[]
				[ is_BGN(pdu) ] ->
				(
					(* Detect Retransmission *)
					Detect_Retransmission  (rs, pdu)
						>> accept rs:SSCOP_var, retransmission:Bool in

					[ retransmission ] ->
					(
						(* If available, resend the BGREJ.SSCOP-UU sent in the last
						 * BGREJ PDU; otherwise, BGREJ.SSCOP UU := null
						 *)
						[ isin(BGREJ_pdu, sp) ] ->
						(
							CPCS ! CPCS_UnitData_invoke ! get(BGREJ_pdu, sp);
							exit (s, ss, rs, ps ,ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
									ReTxQ, RxBuf)
						)
						[]
						[ not(isin(BGREJ_pdu, sp)) ] ->
						(
							(* remember the pdu we send *)
							let sp:Set_of_PDU = add(BGREJ_pdu, BGREJ(null_uu), sp) in

							CPCS ! CPCS_UnitData_invoke ! BGREJ(null_uu);
							exit (s, ss, rs, ps ,ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
									ReTxQ, RxBuf) 
						)
					)
					[]
					[ not(retransmission) ] ->
					(
						(* VT(MS) := BGN.N(MR) *)
						let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

						SSCF ! AA_Establish_ind ! UU(pdu);
						exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, 
								SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
				)
				[]
				[ is_ENDAK(pdu) ] ->
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)

				[]
				
				[ is_END(pdu) ] ->
				(
					CPCS ! CPCS_UnitData_invoke ! ENDAK;
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				)
				[]
				[ is_ER(pdu) or is_POLL(pdu) or is_SD(pdu) or is_BGAK(pdu) 
					or is_ERAK(pdu) or is_STAT(pdu) or is_USTAT(pdu) or is_RS(pdu)
					or is_RSAK(pdu) ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					(* remember the pdu we send *)
					( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

					CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf) )
			)
			[]
			(
		 	SSCOP_I ! SD_PDU_QueuedUp;
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			)
			[]
			SSCF ! AA_Retrieve_req ? aa:AA_RN;
			(
				(* Data Retrieval *)
				Data_Retrieval [SSCF] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
						TxBuf, ReTxQ, RxBuf,aa)
			)
		)	(* End of state 1 *)

	(**************************************************************************)

	[]
	[ s = Outgoing_Connection_Pending_2 ] ->
	(
		CPCS ! CPCS_UnitData_signal ? pdu: SSCOP_PDU [ is_other(pdu) ];
		(
			[ is_ENDAK(pdu) or is_SD(pdu) or is_ERAK(pdu)
				or is_END(pdu) or is_STAT(pdu) or is_USTAT(pdu)
				or is_POLL(pdu) or is_ER(pdu) or is_RSAK(pdu)
				or is_RS(pdu) ] ->
					
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			[]
			
			[ is_BGAK(pdu) ] ->
				(* reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

				(* VT(MS) := BGAK.N(MR) *)
					ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

				SSCF ! AA_Establish_conf ! UU(pdu);
				(* initialize state variable *)
				Initialize_State_Variables  (ss,rs,ps)
					>> accept ss,rs,ps:SSCOP_var in

				(* set data transfer timers *)
				Set_Data_Transfer_Timers  (ps, ts)
					>> accept ts:SSCOP_var in

				exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
						MD_Q, TxBuf, ReTxQ, RxBuf) )
			[]
			[ is_BGREJ(pdu) ] ->
				(* reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				SSCF ! AA_Release_ind ! UU(pdu) ! source_USER;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )
			[]
			[ is_BGN(pdu) ] ->
			(
				(* Detect retransmission *)
				Detect_Retransmission (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* Reset Timer_CC *)
					( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

					(* VT(MS) := BGN.N(MR) *)
						ss:SSCOP_var = SetVT(MS,ss,N_MR(pdu)),

					(* initialize VR(MR) *)
						rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in
					
					[ isin(BGN_pdu, sp) ] ->
						(* remember the pdu we send *)
						( let sp:Set_of_PDU = add(BGAK_pdu, BGAK(VR(MR,rs),
							UU(get(BGN_pdu,sp))), sp) in

						CPCS ! CPCS_UnitData_invoke ! get(BGAK_pdu, sp);

						SSCF ! AA_Establish_conf ! UU(pdu); 
						
						(* initialize state variables *)
						Initialize_State_Variables  (ss,rs,ps)
							>> accept ss,rs,ps:SSCOP_var in

						(* Set data transfer timers *)
						Set_Data_Transfer_Timers  (ps, ts)
							>> accept ts:SSCOP_var in

						exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, 
								UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

					[]
					[ not(isin(BGN_pdu, sp)) ] ->
						(* remember the pdu we send *)
						( let sp:Set_of_PDU=add(BGAK_pdu,BGAK(VR(MR,rs),null_uu),sp) in

						CPCS ! CPCS_UnitData_invoke ! BGAK(VR(MR,rs), null_uu);
						SSCF ! AA_Establish_conf ! UU(pdu); (* sscop_uu = uu *)

						(* initialize state variables *)
						Initialize_State_Variables  (ss, rs, ps)
							>> accept ss, rs, ps:SSCOP_var in

						(* Set data transfer timers *)
						Set_Data_Transfer_Timers  (ps,ts)
							>> accept ts:SSCOP_var in

						exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, 
								UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

					)
			)
		) (* end of received pdu from cpcs *)
		[]
		(
		[ TimerExpired(timer_CC,ts) ] ->
			I_SSCOP ! Timer_CC_Expiry;
			(
			[ VT(CC,ss) ge Param(MaxCC,ps) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)

				(* remember the pdu we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP; 
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
						ReTxQ, RxBuf) )
			[]
			[ not(VT(CC,ss) ge Param(MaxCC,ps)) ] ->
				(* VT(CC) := VT(CC)+1 *)
				( let ss:SSCOP_var = IncVT(CC,ss) in

				(* resend BGN PDU identical to the last BGN PDU *)
				CPCS ! CPCS_UnitData_invoke ! get(BGN_pdu, sp);

				(* Set Timer_CC *)
				( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ) )
			)
			
		)
		[]
		(
			SSCF ! AA_Release_req ? uu:SSCOP_UU;
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

				(* VT(CC) := 1 *)
					ss:SSCOP_var = SetVT(CC, ss, succ(0)),
	
				(* remember the pdu we send *)
					sp:Set_of_PDU = add(END_pdu, END(uu, source_USER), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);

				(* Set Timer_CC *)
				( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

				exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
						UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))
		)
		[]
		(
			SSCOP_I ! SD_PDU_QueuedUp;
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
		)
	)
	(*************************************************************************)

	[]
	[ s = Incoming_Connection_Pending_3 ] ->
	(
		SSCF ! AA_Establish_resp ? uu:SSCOP_UU ? br:Bool;
		(
			(* Clear Transmitter *)
			Clear_Transmitter  (ps, SD_Q, TxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer in

			(* Clear Buffer := br *)
			( let ps:SSCOP_var = SetParamB(ClearBuffers, ps, br),

			(* Initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

			(* remember the pdu we send *)
			let sp:Set_of_PDU = add(BGAK_pdu, BGAK(VR(MR, rs), uu), sp) in

			CPCS ! CPCS_UnitData_invoke ! BGAK(VR(MR, rs), uu);

			(* Initialize state variable *)
			Initialize_State_Variables  (ss,rs,ps)
				>> accept ss,rs,ps:SSCOP_var in

			(* Set Data Timers *)
			Set_Data_Transfer_Timers  (ps, ts)
				>> accept ts:SSCOP_var in

			exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
					MD_Q, TxBuf, ReTxQ, RxBuf) )
		)
		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;

			(* remember the pdu we send *)
			( let sp:Set_of_PDU = add(BGREJ_pdu, BGREJ(uu), sp) in

			CPCS ! CPCS_UnitData_invoke ! BGREJ(uu);
			exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
					RxBuf) )

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
		(
			[ is_BGN(pdu) ] ->
			(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				(
				[ retransmission ] ->
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
								ReTxQ, RxBuf)
				[]
				[ not(retransmission) ] ->
					(* VT(MS) := BGN.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Release_ind ! null_uu ! source_USER;
					SSCF ! AA_Establish_ind ! UU(pdu);
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
									ReTxQ, RxBuf) )
				)
			)
			[]
			[ is_ER(pdu) or is_BGAK(pdu) or is_ERAK(pdu) or is_RSAK(pdu) 
				or is_RS(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)

			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
			(
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
					exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
							ReTxQ, RxBuf)
			)
			[]
			[ is_SD(pdu) or is_USTAT(pdu) or is_STAT(pdu) or is_POLL(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				(* remember the pdu we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
						ReTxQ, RxBuf) )

			[]
			[ is_END(pdu) ] ->
			(
				CPCS ! CPCS_UnitData_invoke ! ENDAK;
				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			)
		)
		[]
		(
		SSCOP_I ! SD_PDU_QueuedUp;
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
					RxBuf)
		)
		[]
		SSCF ! AA_Retrieve_req ? aa:AA_RN;
		(
				(* Data Retrieval *)
				
			Data_Retrieval [SSCF] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
										TxBuf, ReTxQ, RxBuf,aa)
		)

	)
	(**************************************************************************)

	[]
	[ s = Outgoing_Disconnection_Pending_4 ] ->
	(
		SSCF ! AA_Establish_req ? uu:SSCOP_UU ? br:Bool;

			(* Reset Timer_CC *)
			( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

			(* Clear Transmitter *)
			(* Clear-buffers := br *)
				ps:SSCOP_var = SetParamB(ClearBuffers, ps, br),

			(* VT(CC) := 1 *)
			(* VT(SQ) := VT(SQ) + 1 *)
				ss:SSCOP_var = SetVT(CC, IncVT(SQ, ss), succ(0)),

			(* Initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

			(* remember the pdu we send *)
			let sp:Set_of_PDU = add(BGN_pdu,BGN(VR(MR,rs),uu,VT(SQ,ss)), sp) in

			CPCS ! CPCS_UnitData_invoke ! BGN(VR(MR,rs), uu, VT(SQ,ss));

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

			exit (Outgoing_Connection_Pending_2, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
			(
			[ is_SD(pdu) or is_BGAK(pdu) or is_POLL(pdu) or is_STAT(pdu)
				or is_USTAT(pdu) or is_ERAK(pdu) or is_RS(pdu) or is_RSAK(pdu)
				or is_ER(pdu) ] ->

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)

			[]
			[ is_END(pdu) ] ->	
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				CPCS ! CPCS_UnitData_invoke ! ENDAK;
				SSCF ! AA_Release_conf; 

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
						ReTxQ, RxBuf) )

			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				SSCF ! AA_Release_conf;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
						ReTxQ, RxBuf) )

			[]
			[ is_BGN(pdu) ] ->
				(* Detect Retransmission *)
				(
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

					(
					[ retransmission ] ->
						(* remember the pdu we send *)
						( let sp:Set_of_PDU = add(BGAK_pdu, 
												BGAK(VR(MR, rs), null_uu), sp) in

						CPCS ! CPCS_UnitData_invoke ! BGAK(VR(MR, rs),null_uu);

						(* resend END PDU identicla to the last END PDU sent *)

						CPCS ! CPCS_UnitData_invoke ! get(END_pdu, sp);
						exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
								ReTxQ, RxBuf) )
					[]
					[ not(retransmission) ] ->
						(* Reset Timer_CC *)
						( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

						(* VT(MS) := BGN.N(MR) *)
							ss:SSCOP_var = SetVT(MS,ss,N_MR(pdu)) in

						SSCF ! AA_Release_conf;
						SSCF ! AA_Establish_ind ! UU(pdu);

						exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, 
								SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
					)
				)
			)
		[]
		[ TimerExpired(timer_CC,ts) ] -> 
			I_SSCOP ! Timer_CC_Expiry;
		 	(
			[ VT(CC,ss) ge Param(MaxCC,ps) ] ->
				(* LM ! MAA_Error_ind ! O; *)
				SSCF ! AA_Release_conf;
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			[]
			[ not(VT(CC,ss) ge Param(MaxCC, ps)) ] ->
				(* VT(CC) := VT(CC) + 1 *)
				( let ss:SSCOP_var = IncVT(CC, ss) in

				(* resend END PDU identical to the last END PDU sent *)		
				CPCS ! CPCS_UnitData_invoke ! get(END_pdu, sp);

				(* Set Timer_CC *)
				( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ))
			)
		

		[]
		(
		SSCOP_I ! SD_PDU_QueuedUp;
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
					RxBuf)
		)
		[]
		SSCF ! AA_Retrieve_req ? aa:AA_RN;
			(* Data Retrieval *)
			Data_Retrieval [SSCF] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
											TxBuf, ReTxQ, RxBuf,aa)

	)

	(**************************************************************************)

	[]
	[ s = Outgoing_Resynchronization_Pending_5 ] ->
	(
		SSCF ! AA_Release_req ? uu:SSCOP_uu;
			(* Reset Timer_CC *)
			( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

			(* VT(CC) := 1 *)
				ss:SSCOP_var = SetVT(CC, ss, succ(0)),

			(* remember the pdu we send *)
				sp:Set_of_PDU = add(END_pdu, END(uu, source_USER), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

			exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
		(
			[ is_BGN(pdu) ] ->
			(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in
				(
				[ retransmission ] ->
					(* If available, resend the BGAK.SSCOP-UU sent in the 
					 * last BGAK PDU; otherwise, BGAK.SSCOP-UU := null
					 *)
					(* remember the pdu we send *)
					( let sp:Set_of_PDU = addifisin(BGAK_pdu,
							BGAK(VR(MR, rs), UU(get(BGAK_pdu, sp))),
							BGAK(VR(MR, rs), null_uu), sp ) in

					CPCS ! CPCS_UnitData_invoke ! get(BGAK_pdu, sp); 

					(* Resend the RS PDU identical to the last RS PDU sent *)
					CPCS ! CPCS_UnitData_invoke ! get(RS_pdu, sp);

					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf) )
				[]

				[ not(retransmission) ] ->
					(* reset Timer_CC *)
					( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

					(* VT(MS) := BGN.N(MR) *)
						ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Release_ind ! null_uu ! source_USER;
					SSCF ! AA_Establish_ind ! UU(pdu);

					exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, SD_Q, 
							UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)
			)
			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(* reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ is_END(pdu) ] ->
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				CPCS ! CPCS_UnitData_invoke ! ENDAK;
				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ is_RS(pdu) ] ->
			(
				(* Detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* Reset Timer_CC *)
					( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

					(* VT(MS) := RS.N(MR) *)
						ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)),

					(* Initialize VR(MR) *)
						rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

					CPCS ! CPCS_UnitData_invoke ! RSAK(VR(MR,rs));
					SSCF ! AA_Resync_conf; 

					(* Initialize state variables *)
					Initialize_State_Variables  (ss, rs, ps)
						>> accept ss,rs,ps:SSCOP_var in

					(* Set Data Transfer Timers *)
					Set_Data_Transfer_Timers  (ps, ts)
						>> accept ts:SSCOP_var in

					exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
							MD_Q, TxBuf, ReTxQ, RxBuf) )
			)

			[]
			[ is_RSAK(pdu) ] ->
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

				(* VT(MS) := RSAK.N(MR) *)
					ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

				SSCF ! AA_Resync_conf;

				(* Initialize State Variables *)
				Initialize_State_Variables  (ss,rs,ps)
					>> accept ss,rs,ps:SSCOP_var in

				(* Set Data Transfer Timers *)
				Set_Data_Transfer_Timers  (ps, ts)
					>> accept ts:SSCOP_var in

				exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
						MD_Q, TxBuf, ReTxQ, RxBuf) )
			[]
			[ is_ER(pdu) or is_POLL(pdu) or is_STAT(pdu) or is_USTAT(pdu)
				or is_BGAK(pdu) or is_ERAK(pdu) or is_SD(pdu) ] ->
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
		)

		[]
		[ TimerExpired(timer_CC,ts) ] ->
			I_SSCOP ! Timer_CC_Expiry;
			(
			[ VT(CC,ss) ge Param(MaxCC,ps) ] ->
				(*LM ! MAA_Erro_ind ! O;*)
	
				(* remember the PDU we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! Source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ not(VT(CC,ss) ge Param(MaxCC,ps)) ] ->
				(* VT(CC) := VT(CC) + 1 *)
				( let ss:SSCOP_var = IncVT(CC,ss) in
	
				(* Resend RS PDU, identical to the last RS PDU sent *)
				CPCS ! CPCS_UnitData_invoke ! get(RS_pdu, sp);

				(* Set Timer_CC *)
				( let ts:SSCOP_var =  SetTimer(timer_CC, ts, Param(InitCC, ps)) in

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ))
			)

		[]
		SSCOP_I ! SD_PDU_QueuedUp;
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)

	)

	(**************************************************************************)

	[]
	[ s = Incoming_Resynchronization_Pending_6 ] ->
	(
		SSCF ! AA_Resync_resp;
			(* Initialize VR(MR) *)
			( let rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

			CPCS ! CPCS_UnitData_invoke ! RSAK(VR(MR,rs));

			(* Clear Transmitter *)
			Clear_Transmitter  (ps, SD_Q, TxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer in

			(* Initialize State Variables *)
			Initialize_State_Variables  (ss, rs, ps)	
				>> accept ss,rs,ps:SSCOP_var in

			(* Set Data Transfer Timers *)
			Set_Data_Transfer_Timers  (ps, ts)
				>> accept ts:SSCOP_var in

			exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
					TxBuf, ReTxQ, RxBuf) )

		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;
			(* VT(CC) := 1 *)
			( let ss:SSCOP_var = SetVT(CC, ss, succ(0)),

			(* remember the PDU we send *)
				sp:Set_of_PDU = add(END_pdu, END(uu, source_USER), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

			exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
			(
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)
			[]
			[ is_END(pdu) ] ->
				(
				CPCS ! CPCS_UnitData_invoke ! ENDAK;
				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)
			[]
			[ is_BGN(pdu) ] ->
				(
				(* Detect Retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				(
				[ retransmission ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* VT(MS) := BGN.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in
	
					SSCF ! AA_Release_ind ! null_uu ! source_USER;
					SSCF ! AA_Establish_ind ! UU(pdu);

					exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, SD_Q, 
							UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				))

			[]
			[ is_SD(pdu) or is_POLL(pdu) or is_STAT(pdu) or is_USTAT(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)

				(* remember the PDU we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ is_BGAK(pdu) or is_ERAK(pdu) or is_RSAK(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)

			[]
			[ is_RS(pdu) ] ->
				(
				(* Detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				(
				[ retransmission ] ->
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				))
			[]
			[ is_ER(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			)

		[]
		(
		SSCOP_I ! SD_PDU_QueuedUp; 
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
		)
		[]
		SSCF ! AA_Retrieve_req ? rn:AA_RN;
			(* Data Retrieval *)
			Data_Retrieval [SSCF] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, 
											ReTxQ, RxBuf,rn)


	)

	(*************************************************************************)

	[]
	[ s = Outgoing_Recovery_Pending_7 ] ->
		(
		SSCF ! AA_Data_req ? mu:SSCOP_UU;
			(
			[ not(ParamB(ClearBuffers,ps)) ] ->
				(* Put AA_Data_req(MU) into transmission queue *)
				( let SD_Q:UU_Queue = put(mu, SD_Q) in

				(* send SD_PDU queued up *)
				SSCOP_I ! QueueUp_SD_PDU;

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ ParamB(ClearBuffers,ps) ] ->
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			)

		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;
			(* Reset Timer_CC *)
			( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

			(* VT(CC) := 1 *)
				ss:SSCOP_var = SetVT(CC, ss, succ(0)),

			(* remember the PDU we send *)
				sp:Set_of_PDU = add(END_pdu, END(uu, source_USER), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);

			(* Clear Receiver buffer *)
			( let RxBuf:RxBuffer = empty,

			(* Set Timer_CC *)
				ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

			exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		SSCF ! AA_Resync_req ? uu:SSCOP_UU;
			(* Reset Timer_CC *)
			( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

			(* VT(CC) := 1 *)
			(* VT(SQ) := VT(SQ) + 1 *)
				ss:SSCOP_var = SetVT(CC, IncVT(SQ,ss), succ(0)),

			(* Initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

			(* remember the PDU we send *)
			let sp:Set_of_PDU = add(RS_pdu, RS(uu, VR(MR,rs), VT(SQ,ss)), sp) in

			CPCS ! CPCS_UnitData_invoke ! RS(uu, VR(MR,rs), VT(SQ,ss));

			(* Clear transmitter *)
			Clear_Transmitter  (ps, SD_Q, TxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer in

			(* Clear receiver buffer *)
			( let RxBuf:RxBuffer = empty,

			(* Set Timer_CC *)
				ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in 

			exit (Outgoing_Resynchronization_Pending_5, ss, rs, ps, ts, sp, 
					SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
			(
			[ is_BGAK(pdu) or is_RSAK(pdu) ] ->
				(	(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_ERAK(pdu) ] ->
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

				(* VT(MS) := ERAK.N(MR) *)
					ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

				(* Deliver Data *)
				Deliver_Data  (ps,RxBuf)
					>> accept RxBuf:RxBuffer in

				SSCF ! AA_Recover_ind; 

				exit (Recovery_Response_Pending_8, ss, rs, ps, ts, sp, SD_Q, 
						UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

			[]
			[ is_END(pdu) ] ->
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				CPCS ! CPCS_UnitData_invoke ! ENDAK;
				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);

				(* Clear Receiver Buffer *)
				( let RxBuf:RxBuffer = empty in

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ))

			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				(* Reset Timer_CC *)
				( let ts:SSCOP_var = ResetTimer(timer_CC, ts) in

				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;

				(* Clear Receiver Buffer *)
				( let RxBuf:RxBuffer = empty in

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ) )

			[]
			[ is_STAT(pdu) or is_USTAT(pdu) or is_POLL(pdu) or is_SD(pdu) ] ->
				(
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_BGN(pdu) or is_ER(pdu) or is_RS(pdu) ] ->
				(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)

				[]
				[ not(retransmission) ] ->
					(* reset Timer_CC *)
					( let ts:SSCOP_var = ResetTimer(timer_CC, ts),

					(* VT(MS) := pdu.N(MR) *)
						ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in
					
					[ is_BGN(pdu) ] ->
						SSCF ! AA_Release_ind ! null_uu ! source_USER;
						SSCF ! AA_Establish_ind ! UU(pdu);

						(* Clear Receiver buffer *)
						( let RxBuf:RxBuffer = empty in

						exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, 
								SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

					[]
					[ is_ER(pdu) ] ->
						(* Initialize VR(MR) *)
						( let rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

						CPCS ! CPCS_UnitData_invoke ! ERAK(VR(MR,rs));

						(* Deliver Data *)
						Deliver_Data  (ps,RxBuf)
							>> accept RxBuf:RxBuffer in

						SSCF ! AA_Recover_ind;

						exit (Recovery_Response_Pending_8, ss, rs, ps, ts, sp, 
								SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

					[]
					[ is_RS(pdu) ] ->
						SSCF ! AA_Resync_ind ! UU(pdu);
						(* Clear Receiver Buffer *)
						( let RxBuf:RxBuffer = empty in

						exit (Incoming_Resynchronization_Pending_6, 
								ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
								RxBuf) )
					)
				)
			)

		[]
		[ TimerExpired(timer_CC,ts) ] ->
			I_SSCOP ! Timer_CC_Expiry;
			(
			[ VT(CC,ss) ge Param(MaxCC,ps) ] ->
				(* LM ! MAA_Error_ind ! O; *)

				(* remember the PDU we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;

				(* Clear Receiver buffer *)
				( let RxBuf:RxBuffer = empty in

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ) )

			[]
			[ not(VT(CC,ss) ge Param(MaxCC,ps)) ] ->
				(* VT(CC) := VT(CC) + 1 *)
				( let ss:SSCOP_var = IncVT(CC, ss) in

				(* Resend ER PDU identical to the last ER PDU sent *)
				CPCS ! CPCS_UnitData_invoke ! get(ER_pdu, sp);

	 			(* Set Timer_CC *)
				( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ))
			)
	

		(*	[]
		 *		SD_PDU_QueuedUp
		 * Save this signal (until completion of a transition to
		 * a new state)
		 *)

		)

	(*************************************************************************)

	[]
	[ s = Recovery_Response_Pending_8 ] ->
		(
		SSCF ! AA_Retrieve_req  ? rn:AA_RN;
			(* Data Retreival *)
			Data_Retrieval [SSCF] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
											TxBuf, ReTxQ, RxBuf,rn)

		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;
			(* VT(CC) := 1 *)
			( let ss:SSCOP_var = SetVT(CC, ss, succ(0)),

			(* remember the PDU we send *)
				sp:Set_of_PDU = add(END_pdu, END(uu, source_USER), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);
			
			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

			exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		SSCF ! AA_Recover_resp; 
			(
			[ ParamB(ClearBuffers,ps) ] ->
				(* initialize state variables *)
				Initialize_State_Variables  (ss, rs, ps)
					>> accept ss,rs,ps:SSCOP_var in

				(* Set Data Transfer Timers *)
				Set_Data_Transfer_Timers  (ps, ts)
					>> accept ts:SSCOP_var in

				exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
						MD_Q, TxBuf, ReTxQ, RxBuf)

			[]
			[ not(ParamB(ClearBuffers,ps)) ] ->
				(* Clear Transmission Buffer *)
				( let RxBuf:RxBuffer = empty in

				(* initialize state variables *)
				Initialize_State_Variables  (ss, rs, ps)
					>> accept ss,rs,ps:SSCOP_var in

				(* Set Data Transfer Timers *)
				Set_Data_Transfer_Timers  (ps, ts)
					>> accept ts:SSCOP_var in

				exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
						MD_Q, TxBuf, ReTxQ, RxBuf) )
			)

		[]
		SSCF ! AA_Resync_req ? uu:SSCOP_UU;
			(
			(* Clear Transmitter *)
			Clear_Transmitter  (ps, SD_Q, TxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer in

			(* VT(CC) := 1 *)
			(* VT(SQ) := VT(SQ) + 1 *)
			( let ss:SSCOP_var = SetVT(CC, IncVT(SQ,ss), succ(0)),
			
			(* initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

			(* remember the PDU we send *)
			let sp:Set_of_PDU = add(RS_pdu, RS(uu, VR(MR,rs), VT(SQ,ss)), sp) in

			CPCS ! CPCS_UnitData_invoke ! RS(uu, VR(MR,rs), VT(SQ,ss));

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

			exit (Outgoing_Resynchronization_Pending_5, ss, rs, ps, ts, sp, 
					SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))
			)

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
			(
			[ is_BGAK(pdu) or is_RSAK(pdu) ] ->
				(
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_ERAK(pdu) or is_SD(pdu) or is_POLL(pdu) ] ->
				(
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_END(pdu) ] ->
				(
				CPCS ! CPCS_UnitData_invoke ! ENDAK;
				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_RS(pdu) ] ->
				(
				(* detect retransmission *)
				Detect_Retransmission(rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* VT(MS) := RS.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Resync_ind ! UU(pdu);
					exit (Incoming_Resynchronization_Pending_6, ss, rs, ps, ts, 
							sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)
			[]
			[ is_ER(pdu) ] ->
				(
				(* Detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					(
					CPCS ! CPCS_UnitData_invoke ! ERAK(VR(MR,rs));
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
					)
				[]
				[ not(retransmission) ] ->
					(
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
					)
				)

			[]
			[ is_BGN(pdu) ] ->
				(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* VT(MS) := RS.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Release_ind ! null_uu ! source_USER;
					SSCF ! AA_Establish_ind ! UU(pdu);
					exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, 
							SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)

			[]
			[ is_STAT(pdu) or is_USTAT(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)

				(* remember the pdu we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu,source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )
			)

	(*	[]
	 *	SD_PDU Queued up : Save this signal (until completion of a transition to
	 *	a new state)
	 *)

		)

	(**************************************************************************)

	[]
	[ s = Incoming_Recovery_Pending_9 ] ->
		(
		SSCF ! AA_Recover_resp;
			
			(
			[ not(ParamB(ClearBuffers,ps)) ] ->
				(* Clear Transmission Buffers *)
				exit (empty of TxBuffer)
			[]
			[ ParamB(ClearBuffers,ps) ] ->
				exit (TxBuf)
			) >> accept TxBuf:TxBuffer in

			(* Initialize VR(MR) *)
			( let rs:SSCOP_var = SetVR(MR,rs, Param(InitMR,ps)) in

			CPCS ! CPCS_UnitData_invoke ! ERAK(VR(MR,rs));

			(* Initialize state variables *)
			Initialize_State_Variables  (ss, rs, ps)
				>> accept ss,rs,ps:SSCOP_var in

			(* Set Data Transfer Timers *)
			Set_Data_Transfer_Timers  (ps, ts)
				>> accept ts:SSCOP_var in

			exit (Data_Transfer_Ready_10, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
					TxBuf, ReTxQ, RxBuf) )

		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;
			(* VT(CC) := 1 *)
			( let ss:SSCOP_var = SetVT(CC, ss, succ(0)),

			(* remember the pdu we send *)
				sp:Set_of_PDU = add(END_pdu, END(uu,source_USER), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC,ps)) in

			exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		SSCF ! AA_Resync_req ? uu:SSCOP_UU;
			(
			(* Clear Transmitter *)
			Clear_Transmitter  (ps, SD_Q, TxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer in

			(* VT(CC) := 1 *)
			(* VT(SQ) := VT(SQ) + 1 *)
			( let ss:SSCOP_var = SetVT(CC, IncVT(SQ,ss), succ(0)),

			(* Initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR, rs, Param(InitMR,ps)) in

			(* remember the pdu we send *)
			let sp:Set_of_PDU = add(RS_pdu, RS(uu, VR(MR, rs), VT(SQ, ss)), sp) in

			CPCS ! CPCS_UnitData_invoke ! RS(uu, VR(MR, rs), VT(SQ, ss));

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

			exit (Outgoing_Resynchronization_Pending_5, ss, rs, ps, ts, sp, 
					SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))
			)

		[]
		SSCF ! AA_Retrieve_req ? rn:AA_RN;
			(* Data retrieval *)
			Data_Retrieval [SSCF] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, 
											TxBuf, ReTxQ, RxBuf,rn)

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [is_other(pdu)] ;
			(
			[ is_END(pdu) ] ->
				(
				CPCS ! CPCS_UnitData_Invoke ! ENDAK;
				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_USTAT(pdu) or is_STAT(pdu) or is_POLL(pdu) or is_SD(pdu) ] ->
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)

				(* remember the pdu we send *)
				( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

				CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;
				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ is_RSAK(pdu) or is_BGAK(pdu) or is_ERAK(pdu) ] ->
				(
				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_RS(pdu) ] ->
				(
				(* Detect Retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* VT(MS) := RS.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Resync_ind ! UU(pdu);
					exit (Incoming_Resynchronization_Pending_6, ss, rs, ps, ts, 
							sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)

			[]
			[ is_ER(pdu) ] ->
				(
				(* Detect Retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				)

			[]
			[ is_BGN(pdu) ] ->
				(
				(* Detect Retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->
					(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
				[]
				[ not(retransmission) ] ->
					(* VT(MS) := BGN.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Release_ind ! null_uu ! source_USER;
					SSCF ! AA_Establish_ind ! UU(pdu);
					exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, 
							SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)
			)

	(*	[]
	 *	SD_PDU Queued up : Save this signal (until completion of a transition to
	 *	a new state)
	 *)

		)

	(**************************************************************************)

	[]
	[ s = Data_Transfer_Ready_10 ] ->
		(
		SSCF ! AA_Resync_req ? uu:SSCOP_UU;
			(
			(* reset Data TRansfer Timers *)
			Reset_Data_Transfer_Timers  (ts)
				>> accept ts:SSCOP_var in

			(* VT(CC) := 1 *)
			(* VT(SQ) := VT(SQ) + 1 *)
			( let ss:SSCOP_var = SetVT(CC, IncVT(SQ,ss), succ(0)),
		
			(* Initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR, rs, Param(InitMR, ps)) in

			(* remember the PDU we send *)
			let sp:Set_of_PDU = add(RS_pdu, RS(uu, VR(MR, rs),VT(SQ,ss)), sp) in

			CPCS ! CPCS_UnitData_invoke ! RS(uu, VR(MR, rs), VT(SQ, ss));

			(* Release Buffers *)
			Release_Buffers  (SD_Q, TxBuf, ReTxQ, RxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue, 
							RxBuf:RxBuffer in

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

			exit (Outgoing_Resynchronization_Pending_5, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))
			)

		[]
		SSCF ! AA_Release_req ? uu:SSCOP_UU;
			(
			(* Reset Data Transfer Timers *)
			Reset_Data_Transfer_Timers  (ts)
				>> accept ts:SSCOP_var in

			(* VT(CC) := 1 *)
			( let ss:SSCOP_var = SetVT(CC, ss, succ(0)),

			(* remember the PDU we send *)
				sp:Set_of_PDU = add(END_pdu, END(uu, source_USER), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(uu, source_USER);

			(* Prepare retrieval *)
			Prepare_Retrieval  (ps, SD_Q, TxBuf, ReTxQ, RxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue, 
							RxBuf:RxBuffer in

			(* Set Timer_CC *)
			( let ts:SSCOP_var = SetTimer(timer_CC, ts, Param(InitCC, ps)) in

			exit (Outgoing_Disconnection_Pending_4, ss, rs, ps, ts, sp, SD_Q, 
					UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))
			)

		[]
		SSCF ! AA_Data_req ? mu:SSCOP_UU;
			(* Put AA.DATA.request(MU) into transmission queue *)
			( let SD_Q:UU_Queue = put(mu, SD_Q) in

			(* SD_PDU Queued Up *)
			SSCOP_I ! QueueUp_SD_PDU;

			(* The MU parameter of the AA-DATA.requests are queued until served 
			 * by the procedure below the event "SD PDU Queued Up"
			 *)
			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

		[]
		CPCS ! CPCS_UnitData_signal ? pdu:SSCOP_PDU [ is_other(pdu) ];
			(
			[ is_BGAK(pdu) or is_ERAK(pdu) or is_RSAK(pdu) ] ->
				(
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)
			[]
			[ is_ER(pdu) ] ->
				(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->	
					(* Set Timer_NO_RESPONSE *)
					( let ts:SSCOP_var = SetTimer(NO_RESPONSE, ts,
											Param(InitNO_RESPONSE, rs)) in

					CPCS ! CPCS_UnitData_invoke ! ERAK(VR(MR, rs));

					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf) )

				[]
				[ not(retransmission) ] ->
					(* Reset Data Transfer Timers *)
					(* VT(MS) := ER.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					(* Prepare Recovery *)
					Prepare_Recovery  (ps, SD_Q, TxBuf, ReTxQ)
						>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue in

					(* Deliver Data *)
					Deliver_Data  (ps, RxBuf)
						>> accept RxBuf:RxBuffer in

					SSCF ! AA_Recover_ind;

					exit (Incoming_Recovery_Pending_9, ss, rs, ps, ts, sp, SD_Q, 
							UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)

			[]
			[ is_BGN(pdu) ] ->
				(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->	
					(* Set Timer_NO_RESPONSE *)
					( let ts:SSCOP_var = SetTimer(NO_RESPONSE, ts,
											Param(InitNO_RESPONSE, rs)),

					(* If available, resend the BGAK.SSCOP-UU sent in the last
					 * BGAK PDU; otherwise, BGAK.SSCOP-UU := null_uu
					 *)
					(* remember the PDU we send *)
						sp:Set_of_PDU = addifisin(BGAK_pdu,
									BGAK(VR(MR, rs), UU(get(BGAK_pdu, sp))),
									BGAK(VR(MR, rs), null_uu) , sp) in

					CPCS ! CPCS_UnitData_invoke ! get(BGAK_pdu, sp);

					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf) )

				[]
				[ not(retransmission) ] ->
					(
					(* Reset Data Transfer Timers *)
					Reset_Data_Transfer_Timers  (ts)
						>> accept ts:SSCOP_var in

					(* VT(MS) := ER.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Release_ind !null_uu ! source_USER;
					SSCF ! AA_Establish_ind !UU(pdu);

					(* Prepare retrieval *)
					Prepare_Retrieval  (ps, SD_Q, TxBuf, ReTxQ, RxBuf)
						>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
									RxBuf:RxBuffer in

					exit (Incoming_Connection_Pending_3, ss, rs, ps, ts, sp, SD_Q, 
							UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
					)
				)

			[]
			[ is_RS(pdu) ] ->
				(
				(* detect retransmission *)
				Detect_Retransmission  (rs, pdu)
					>> accept rs:SSCOP_var, retransmission:Bool in

				[ retransmission ] ->	
					(* Set Timer_NO_RESPONSE *)
					( let ts:SSCOP_var = SetTimer(NO_RESPONSE, ts,
											Param(InitNO_RESPONSE, rs)) in

					CPCS ! CPCS_UnitData_invoke ! RSAK(VR(MR, rs));

					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf) )

				[]
				[ not(retransmission) ] ->
					(
					(* Reset Data Transfer Timers *)
					Reset_Data_Transfer_Timers  (ts)
						>> accept ts:SSCOP_var in

					(* VT(MS) := ER.N(MR) *)
					( let ss:SSCOP_var = SetVT(MS, ss, N_MR(pdu)) in

					SSCF ! AA_Resync_ind ! UU(pdu);

					(* prepare retrieval *)
					Prepare_Retrieval  (ps, SD_Q, TxBuf, ReTxQ, RxBuf)
						>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
									RxBuf:RxBuffer in

					exit (Incoming_Resynchronization_Pending_6, ss, rs, ps, ts, 
							sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
					)
				)
				

			[]
			[ is_ENDAK(pdu) or is_BGREJ(pdu) ] ->
				(
				(* Reset Data Transfer Timers *)
				Reset_Data_Transfer_Timers  (ts)
					>> accept ts:SSCOP_var in

				(* LM ! MAA_Error_ind ! Bad_PDU(pdu); *)
				SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;

				(* Prepare retrieval *)
				Prepare_Retrieval  (ps, SD_Q, TxBuf, ReTxQ, RxBuf)
					>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
								RxBuf:RxBuffer in

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_END(pdu) ] ->
				(
				(* Reset Data Transfer Timers *)
				Reset_Data_Transfer_Timers (ts)
					>> accept ts:SSCOP_var in

				CPCS ! CPCS_UnitData_invoke ! ENDAK;

				SSCF ! AA_Release_ind ! UU(pdu) ! S(pdu);

				(* Prepare retrieval *)
				Prepare_Retrieval  (ps, SD_Q, TxBuf, ReTxQ, RxBuf)
					>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
								RxBuf:RxBuffer in

				exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
				)

			[]
			[ is_SD(pdu) ] ->
				Treat_SD_PDU [SSCF, CPCS] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
													MD_Q, TxBuf, ReTxQ, RxBuf, pdu)

			[]
			[ is_POLL(pdu) ] ->
				Treat_POLL_PDU [SSCF, CPCS] (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
													MD_Q, TxBuf, ReTxQ, RxBuf, pdu)
 
			[]
			[ is_USTAT(pdu) ] ->
				Treat_USTAT_PDU [SSCF, CPCS, SSCOP_I] (s, ss, rs, ps, ts, sp, 
									SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf, pdu)
 
			[]
			[ is_STAT(pdu) ] ->
				Treat_STAT_PDU [SSCF, CPCS, SSCOP_I] (s, ss, rs, ps, ts, sp, 
									SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf, pdu)

			)

		[]
		[ TimerExpired(POLL, ts) or TimerExpired(KEEP_ALIVE, ts) ] ->
			I_SSCOP ! Timer_POLL_Expiry ! TimerExpired(POLL,ts) ! Timer_KEEP_ALIVE_Expiry ! TimerExpired(KEEP_ALIVE,ts);
			(* VT(PS) := VT(PS) + 1 *)
			( let ss:SSCOP_var = IncVT(var_PS, ss) in

			CPCS ! CPCS_UnitData_invoke !  POLL(VT(var_PS,ss), VT(var_S,ss));

			(* VT(PD) := 0 *)
			( let ss:SSCOP_var = SetVT(PD, ss, 0) in

			(* Set POLL Timer *)
				Set_POLL_Timer  (ss, ps, ts, SD_Q)
					>> accept ts:SSCOP_var in

			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		[ TimerExpired(IDLE, ts) ] ->
			I_SSCOP ! Timer_IDLE_Expiry;
			(* Set Timer NO-RESPONSE *)
			( let ts:SSCOP_var = SetTimer(NO_RESPONSE, ts, 
										Param(InitNO_RESPONSE, ps)),

			(* VT(PS) := VT(PS) + 1 *)
				ss:SSCOP_var = IncVT(var_PS, ss) in

			CPCS ! CPCS_UnitData_invoke !  POLL(VT(var_PS,ss), VT(var_S,ss));

			(* VT(PD) := 0 *)
			( let ss:SSCOP_var = SetVT(PD, ss, 0) in

			(* Set POLL Timer *)
			Set_POLL_Timer  (ss, ps, ts, SD_Q)
				>> accept ts:SSCOP_var in

			exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) ))

		[]
		[ TimerExpired(NO_RESPONSE, ts) ] ->
			I_SSCOP ! Timer_NO_RESPONSE_Expiry;
			(
			(* Reset data Transfer Timers *)
			Reset_Data_Transfer_Timers  (ts)
				>> accept ts:SSCOP_var in

			(* LM ! MAA_Error_ind ! P; *)

			(* remember the pdu we send *)
			( let sp:Set_of_PDU = add(END_pdu, END(null_uu, source_SSCOP), sp) in

			CPCS ! CPCS_UnitData_invoke ! END(null_uu, source_SSCOP);

			SSCF ! AA_Release_ind ! null_uu ! source_SSCOP;

			(* Prepare Retrieval *)
			Prepare_Retrieval  (ps, SD_Q, TxBuf, ReTxQ, RxBuf)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
							RxBuf:RxBuffer in

			exit (Idle_1, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
					RxBuf) )
			)

		[]
		SSCOP_I ! SD_PDU_QueuedUp;
			Treat_SD_PDU_QueuedUp [CPCS, SSCOP_I] (s, ss, rs, ps, ts, sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
	

		)
	
	)
	Where

		(***********************************************************************)

      Process Treat_SD_PDU_QueuedUp [CPCS, SSCOP_I] 
				(s: SSCOP_State, ss,rs,ps,ts:SSCOP_var,
            sp:Set_of_PDU, SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer,
            ReTxQ:PDU_Queue, RxBuf:RxBuffer):
		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=
		(
		(* Retransmission Queue is empty *)
		[ isempty(ReTxQ) ] ->
			(
			(* Transmission Queue is empty *)
			[ isempty(SD_Q) ] ->
				(* Ignore "SD PDU queued up" signal if there is no associated
				 * data in either the transmission queue or the retransmission
				 * queue.
				 *)
				exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			[]
			[ not(isempty(SD_Q)) ] ->
				(
				[ VT(var_S,ss) lt VT(MS,ss) ] ->
					(* Remove MU from Transmission Queue *)
					( let mu:SSCOP_UU = get(SD_Q),
						SD_Q:UU_Queue = remove(SD_Q) in

					CPCS ! CPCS_UnitData_invoke ! SD(VT(var_S,ss),mu);

					(* Save SD PDU in transmission Buffer *)
					(* SD.N(PS) := VT(PS) : A Poll sequence number (SD.N(PS)) is
					 * save with the SD PDU in the Transmission Buffer
					 *)
					( let TxBuf:TxBuffer =add(elt(VT(var_PS,ss),SD(VT(var_S,ss),mu)),TxBuf),

					(* VT(S) := VT(S) + 1 *)
						ss:SSCOP_var = SetVT(var_S, ss, succ(VT(var_S,ss))) in

					Check_For_Poll [CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
															MD_Q, TxBuf, ReTxQ, RxBuf) ))

				[]
				[ not(VT(var_S,ss) lt VT(MS,ss)) ] ->
					SSCOP_I ! QueueUp_SD_PDU;
					(* Reset Timer_IDLE *)
					(* Set Timer_NO_RESPONSE *)
					( let ts:SSCOP_var = ResetTimer(IDLE,
							SetTimer(NO_RESPONSE, ts, Param(InitNO_RESPONSE,ps))) in
					Send_Poll_pdu [CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
															MD_Q, TxBuf, ReTxQ, RxBuf) )
				)
			)
		[]
		[ not(isempty(ReTxQ)) ] ->
			(* Remove SD PDU from Retransmission Queue *)
			( let pdu:SSCOP_PDU = get(ReTxQ),
				ReTxQ:PDU_Queue = remove(ReTxQ) in

			CPCS ! CPCS_UnitData_invoke ! pdu;

			(* Update the stored poll sequence number (in the Transmission 
			 * Buffer) *)
			(* SD.N(PS) := VT(PS) *)

			( let TxBuf:TxBuffer = add(elt(VT(var_PS,ss), pdu), 
											remove(N_S(pdu),TxBuf)) in

		(**************************************
		 * option : Poll after retransmission *
		 **************************************)
	
			(* Retransmission Queue is empty *)
			[ isempty(ReTxQ) ] -> 
				Send_Poll_pdu [CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, 
													TxBuf, ReTxQ, RxBuf)
			[]
			[ not(isempty(ReTxQ)) ] ->
				Check_For_Poll [CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, 
														TxBuf, ReTxQ, RxBuf)
			))

		(*****************************************
		 * option : NO Poll after retransmission *
		 *****************************************)

		(*
		 *	Check_For_Poll [SSCF, CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, 
		 *											TxBuf, ReTxQ, RxBuf)
		 *)
		)
		Where

			(*******************************************************************)

			Process Check_For_Poll [CPCS] (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer) :
			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
				(* VT(PD) := VT(PD) + 1 *)
				let ss:SSCOP_var = SetVT(PD, ss, succ(VT(PD,ss))) in

				(
				[ ActiveTimer(POLL, ts) ] ->
					(
					[ VT(PD,ss) ge Param(MaxPD,ps) ] ->
						Send_Poll_pdu [CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
																MD_Q, TxBuf, ReTxQ, RxBuf)
					[]
					[ not(VT(PD,ss) ge Param(MaxPD,ps)) ] ->
						(
						exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
								RxBuf)
						)
					)
				[]
				[ not(ActiveTimer(POLL, ts)) ] ->
					(
					(
					[ ActiveTimer(IDLE, ts) ] ->
						(* Reset Timer_IDLE *)
						(* Set Timer_NO_RESPONSE *)
						exit ( ResetTimer(IDLE, 
							SetTimer(NO_RESPONSE, ts, Param(InitNO_RESPONSE,ps))) )
					[]
					[ not(ActiveTimer(IDLE, ts)) ] ->
						(* Reset Timer_KEEP_ALIVE *)
						exit (ResetTimer(KEEP_ALIVE,ts))
					) >> accept ts:SSCOP_var in
					(
					[ VT(PD,ss) ge Param(MaxPD,ps) ] ->
						Send_Poll_pdu [CPCS] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
																MD_Q, TxBuf, ReTxQ, RxBuf)
					[]
					[ not(VT(PD,ss) ge Param(MaxPD,ps)) ] ->
						(* Set Timer_POLL *)
						exit (s,ss,rs,ps, SetTimer(POLL, ts, Param(InitPOLL,ps)),sp,
								SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
					)
				)
			)
			EndProc (* Check_For_Poll *)

			(********************************************************************)

			Process Send_Poll_pdu [CPCS] (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU,
						SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer) :
			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
				(* VT(PS) := VT(PS) + 1 *)
				let ss:SSCOP_var = SetVT(var_PS, ss, succ(VT(var_PS,ss))) in

				CPCS ! CPCS_UnitData_invoke ! POLL(VT(var_PS,ss),VT(var_S,ss));

				(* VT(PD) := 0 *)
				( let ss:SSCOP_var = SetVT(PD,ss, 0),

				(* Set Timer_POLL *)
					ts:SSCOP_var = SetTimer(POLL,ts, Param(InitPOLL, ps)) in

				exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
			)
			EndProc (* Send_Poll_pdu *)
				
		EndProc (* Treat_SD_PDU_QueuedUp *)

		(***********************************************************************)

      Process Treat_SD_PDU [SSCF, CPCS] (s: SSCOP_State, ss,rs,ps,ts:SSCOP_var,
            sp:Set_of_PDU, SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer,
            ReTxQ:PDU_Queue, RxBuf:RxBuffer, pdu:SSCOP_PDU):

		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=

		(
		[ not(N_S(pdu) lt VR(MR,rs)) ] ->
			(
			[ VR(H, rs) lt VR(MR, rs) ] ->
				(* List element 1 := VR(H) *)
				(* List element 2 := VR(MR) *)
				( 
				CPCS ! CPCS_UnitData_invoke ! USTAT(put(VR(MR,rs),
					put(VR(H,rs), empty)), VR(R,rs), VR(MR,rs));

				(* VR(H) := VR(MR) *)
				( let rs:SSCOP_var = SetVR(H, rs, VR(MR,rs)) in

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) ))
			[]
			[ not(VR(H, rs) lt VR(MR, rs)) ] ->
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			)

		[]
		[ N_S(pdu) lt VR(MR,rs) ] ->
			(
			[ N_S(pdu) eq VR(R,rs) ] ->
				(
				[ N_S(pdu) eq VR(H,rs) ] ->
					SSCF ! AA_Data_ind ! UU(pdu) ! N_S(pdu);

					(* VR(R) := SD.N(S) + 1 *)
					(* VR(H) := SD.N(S) + 1 *)
					( let rs:SSCOP_var = SetVR(R, SetVR(H,rs,N_S(pdu)),N_S(pdu)) in

					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf) )
				[]
				[ not(N_S(pdu) eq VR(H,rs)) ] ->
					SSCF ! AA_Data_ind ! UU(pdu) ! N_S(pdu);

					(* VR(R) := VR(R) + 1 *)
					( let rs:SSCOP_var = IncVR(R,rs) in

					Give_Received_SD_PDUs [SSCF] (s, ss, rs, ps, ts, sp, SD_Q, 
														UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)
			[]
			[ not(N_S(pdu) eq VR(R, rs)) ] ->
				(
			(*	[ receive buffer is not available ] ->
			 *		exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
			 *				RxBuf)
			 *	[]
			 *	[ receive buffer is available ] ->
			 *)	(
					[ N_S(pdu) eq VR(H,rs) ] ->
						(* save SD PDU in receiver buffer *)
						( let RxBuf:RxBuffer = add(pdu, RxBuf),

						(* VR(H) := VR(H) + 1 *)
							rs:SSCOP_var = IncVR(H,rs) in

						exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
								ReTxQ, RxBuf) )
					[]
					[ N_S(pdu) gt VR(H,rs) ] ->
						(* save SD_PDU in receiver buffer *)
						( let RxBuf:RxBuffer = add(pdu, RxBuf) in

						(* List element 1 := VR(H) *)
						(* List element 2 := SD.N(S) *)

						CPCS ! CPCS_UnitData_invoke ! USTAT(put(N_S(pdu),
								put(VR(H,rs), empty)), VR(R,rs), VR(MR, rs));

						(* VR(H) := SD.N(S) + 1 *)
						( let rs:SSCOP_var = SetVR(H, rs, succ(N_S(pdu))) in

						exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
								ReTxQ, RxBuf) ))
					[]
					[ N_S(pdu) lt VR(H,rs) ] ->
						(
					(*	[ SD.N(S) is already in receiver buffer ] -> *)

						[ isin(N_S(pdu), RxBuf) ] ->
							(* reset Data transfer Timers *)
							Reset_Data_Transfer_Timers  (ts)
								>> accept ts:SSCOP_var in

							(* LM ! MAA_Error_ind ! Q; *)
							Start_Error_Recovery_Procedures [CPCS] 
								(s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
								 ReTxQ, RxBuf)
						[]

						[ not(isin(N_S(pdu),RxBuf)) ] ->
							(* Save SD PDU in receiver buffer *)
							( let RxBuf:RxBuffer = add(pdu, RxBuf) in
							exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
									ReTxQ, RxBuf) )
				
						)
					)
				)
			)				
		)
		Where

			(*******************************************************************)

			Process Give_Received_SD_PDUs [SSCF] (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer) : 
			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=
			(
				(* if there is a pdu with SD.N(S) = VR(R),
				 *		then remove that pdu from the receiver buffer
				 *			give it to the SSCF ( SSCF ! AA_Data_ind ! MU(pdu) ! N_S(pdu); )
				 *			VR(R) := VR(R) + 1
				 *			Give_Received_SD_PDU [SSCF] (s, ss, rs, ps, ts, sp, 
				 *									SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
				 *		else exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, 
				 *						ReTxQ, RxBuf)
				 *)
			[ isin(VR(R,rs), RxBuf) ] ->
				( let pdu:SSCOP_PDU = get( VR(R,rs), RxBuf),
					RxBuf:RxBuffer = remove( VR(R,rs), RxBuf) in

				SSCF ! AA_Data_ind ! UU(pdu) ! N_S(pdu);

				Give_Received_SD_PDUs [SSCF] (s,ss,rs,ps,rs,sp, SD_Q, UD_Q, MD_Q, 
														TxBuf, ReTxQ, RxBuf) )

			[]
			[ not(isin(VR(R,rs), RxBuf)) ] ->
				exit(s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			)
			EndProc (* Give_Received_SD_PDU *)

		EndProc (* Treat_SD_PDU *)

		(**********************************************************************)

      Process Treat_POLL_PDU [SSCF,CPCS] (s:SSCOP_State,ss,rs,ps,ts:SSCOP_var,
               sp:Set_of_PDU, SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer,
               ReTxQ:PDU_Queue, RxBuf:RxBuffer, pdu:SSCOP_PDU) :

		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=

		(
		[ VR(H,rs) gt N_S(pdu) ] ->
			(
			(* reset Data Transfer Timer *)
			Reset_Data_Transfer_Timers  (ts)
				>> accept ts:SSCOP_var in

			(* LM ! MAA_Error_ind ! Q *)
			Start_Error_Recovery_Procedures [CPCS] (s,ss,rs,ps,ts,sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			)

		[]
		[ not(VR(H,rs) gt N_S(pdu)) ] ->
			(
			(
			[ VR(MR,rs) lt N_S(pdu) ] ->
				exit (SetVR(H, rs, VR(MR,rs)))
			[]
			[ not(VR(MR,rs) lt N_S(pdu)) ] ->
				exit (SetVR(H, rs, N_S(pdu)))
			) >> accept rs:SSCOP_var in

			(* List_Length := 0 *)
			(* i := VR(R) *)
			( let List:ListOfNat = empty,
				n:Nat = VR(R,rs),
				List_Length:Nat = 0 in
				
			Fill_and_Send_STAT_pdu [SSCF, CPCS] 
				(s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf,
				List, n, List_Length, pdu) )
			)
		)
		Where

			(*******************************************************************)

			Process Fill_and_Send_STAT_pdu [SSCF, CPCS] (s:SSCOP_State,
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU,
						SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer, l:ListOfNat, n:Nat,
						List_Length:Nat, pdu:SSCOP_PDU) :
			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			[ n eq VR(H,rs) ] ->
				CPCS ! CPCS_UnitData_invoke ! STAT(l, VR(R,rs), 
																VR(MR,rs), N_PS(pdu));
				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf)
			[]
			[ not(n eq VR(H,rs)) ] ->
				(
				i;
				Find_Begining_of_Missing_PDUs [SSCF, CPCS]
					(s,ss,rs,ps,ts,sp,SD_Q,UD_Q,MD_Q,TxBuf,ReTxQ,RxBuf,
					l,n, List_Length, pdu)
				)
			)
			EndProc (* Fill_and_Send_STAT_pdu *)

			(********************************************************************)

			Process Find_Begining_of_Missing_PDUs [SSCF, CPCS] (s:SSCOP_State,
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q,UD_Q,MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer, l:ListOfNat, n:Nat,
						List_Length:Nat, pdu:SSCOP_PDU) :
			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			[ not(n lt VR(H,rs)) ] ->
				(* Append n to list *)
				( let l:ListOfNat = put(n,l) in

				CPCS ! CPCS_UnitData_invoke ! STAT(l, VR(R,rs), 
					VR(MR,rs), N_PS(pdu));

				exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
						RxBuf) )

			[]
			[ n lt VR(H, rs) ] ->
				(
				(* SD.N(S) = n is in Receiver buffer *)
				[ isin(n, RxBuf) ] -> 
					(
					i;
					Find_Begining_of_Missing_PDUs [SSCF, CPCS] (s,ss,rs,ps,ts,sp,
						SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf, l,
						succ(n), List_Length, pdu)
					)
				[]

				(* SD.N(S) = n is not in Receiver buffer *)
				[ not(isin(n, RxBuf)) ] ->

					(* Append n to list *)
					(* List_Lentgh := List_Length + 1 *)
					( let l:ListOfNat = put(n,l),
						List_Length:Nat = succ(List_Length) in
					
					
					[ List_Length ge Param(MaxSTAT,ps) ] ->
					
						CPCS ! CPCS_UnitData_invoke ! STAT(l, VR(R,rs), VR(MR,rs),
																		N_PS(pdu));

						(* Start building a new STAT list *)
						(* Append n to list *)
						(* List Length := 1 *)
						Find_End_of_Missing_PDUs [SSCF, CPCS] (s, ss,rs,ps,ts,sp,
							SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf,
							put(n,empty), n, succ(0), pdu)
					[]
					[ not(List_Length ge Param(MaxSTAT,ps)) ] ->
						(
						i;
						Find_End_of_Missing_PDUs [SSCF, CPCS] (s, ss,rs,ps,ts,sp,
							SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf,
							l, n, List_Length, pdu)
						)
					)
				)
			)
			EndProc (* Find_Begining_of_Missing_PDUs *)

			(*******************************************************************)

			Process Find_End_of_Missing_PDUs [SSCF, CPCS] (s:SSCOP_State,
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer, l:ListOfNat,
						n:Nat, List_Length:Nat, pdu:SSCOP_PDU) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
				(* n := n + 1 *)
				let n:Nat = succ(n) in
				(
				[ n lt VR(H,rs) ] ->
					(
					(* SD.N(S) = n is in Receiver buffer *)
					[ isin(n, RxBuf) ] ->
						(
						i;
						(* Append n to list *)
						(* List_Length := List_Length + 1 *)
						Fill_and_Send_STAT_pdu [SSCF, CPCS] (s, ss,rs,ps,ts,sp,
							SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf,
							put(n,l), n, succ(List_Length), pdu)
						)
					[]
					(*  SD.N(S) = n is in Receiver Buffer *)
					[ not(isin(n, RxBuf)) ] ->
						(i;
						Find_End_of_Missing_PDUs [SSCF, CPCS] (s, ss,rs,ps,ts,sp,
							SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf,
							l, n, List_Length, pdu)
						)
					)
				[]
				[ not(n lt VR(H,rs)) ] ->
					(* Append n to list *)
					(* List_Length := List_Length + 1 *)
					Fill_and_Send_STAT_pdu [SSCF, CPCS] (s, ss,rs,ps,ts,sp,
						SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf,
						put(n,l), n, succ(List_Length), pdu)
				)
			)
			EndProc (* Find_End_of_Missing_PDUs *)
					
		EndProc (* Treat_POLL_pdu *)

		(**********************************************************************)

      Process Treat_USTAT_PDU [SSCF,CPCS,SSCOP_I] (s:SSCOP_State,
               ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, SD_Q,UD_Q,MD_Q:UU_Queue,
               TxBuf:TxBuffer, ReTxQ:PDU_Queue, RxBuf:RxBuffer,
               pdu:SSCOP_PDU) :

		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=

		(
		[ ( VT(A,ss) le N_R(pdu) ) and ( N_R(pdu) lt VT(var_S, ss) ) ] ->
			(* remove SD PDUs from VT(A) to USTAT.N(R) - 1 from 
			 * transmission buffer *)
			( let TxBuf:TxBuffer = remove_from_to(VT(A,ss), pred(N_R(pdu)), TxBuf),

			(* VT(A) := USTAT.N(R) *)
			(* VT(MS) := USTAT.N(MR) *)
				ss:SSCOP_var = SetVT(A, SetVT(MS, ss, N_MR(pdu)), N_R(pdu)),

			(* seq1 := List element 1 *)
			(* seq2 := List element 2 *)
				seq1:Nat = get(List(pdu)),
				seq2:Nat = get(remove(List(pdu))) in

			(
			[ ( VT(A,ss) le seq1 ) and ( seq1 lt seq2 ) 
				and ( seq2 lt VT(var_S,ss) ) ] ->

				Resend_Missing_PDUs [SSCF, CPCS, SSCOP_I] 
											(s, ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q,
											TxBuf, ReTxQ, RxBuf, pdu, seq1, seq2)

			[]
			[ not( ( VT(A,ss) le seq1 ) and ( seq1 lt seq2 ) 
				and ( seq2 lt VT(var_S,ss) ) ) ] ->

				(
				(* Reset Data Transfer Timers *)
				Reset_Data_Transfer_Timers  (ts)
					>> accept ts:SSCOP_var in

				(* LM ! MAA_Error_ind ! T *)
				Start_Error_Recovery_Procedures [CPCS] (s, ss,rs,ps,ts,sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
				)
			)
			)
		[]
		[ not( ( VT(A,ss) le N_R(pdu) ) and ( N_R(pdu) lt VT(var_S, ss) ) ) ] ->
			(
			(* Reset Data Transfer Timers *)
			Reset_Data_Transfer_Timers  (ts)
				>> accept ts:SSCOP_var in

			(* LM ! MAA_Error_ind ! T *)
			Start_Error_Recovery_Procedures [CPCS] (s, ss,rs,ps,ts,sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			)
		)
		Where

			(*******************************************************************)

			Process Resend_Missing_PDUs [SSCF, CPCS, SSCOP_I] (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer, pdu:SSCOP_PDU, seq1,seq2:Nat) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			(* SD.N(S) = seq1 is in transmission buffer *)
			[ isin(seq1, TxBuf) ] ->
				(* retreive SD.N(S) = seq1 from transmission buffer *)
				(* put SD PDU into retransmission queue *)
				( let ReTxQ:PDU_Queue = put(PDU(getTX(seq1, TxBuf)), ReTxQ),
					TxBuf:TxBuffer = remove(seq1, TxBuf) in

				SSCOP_I ! QueueUp_SD_PDU;

				(* Save SD PDU in the transmission buffer *)
				( let TxBuf:TxBuffer = add(elt(VT(var_PS,ss),get(ReTxQ)), TxBuf),
				(* seq1 := seq1 + 1 *)
					seq1:Nat = succ(seq1) in

				[ seq1 eq seq2 ] ->
					(
					(* LM ! MAA_Error_ind ! V(ElemNb(1,List(pdu)) 
					 *									- ElemNb(2,List(pdu))); 
					 *)
					exit (s, ss, rs, ps, ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, 
							RxBuf)
					)
				[]
				[ not(seq1 eq seq2) ] ->
					Resend_Missing_PDUs [SSCF, CPCS, SSCOP_I] (s,ss,rs,ps,ts,sp,SD_Q,
 										UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf, pdu,seq1, seq2)
				) )

			[]
			[ not(isin(seq1, TxBuf)) ] ->
				(
				(* Reset Data Transfer Timers *)
				Reset_Data_Transfer_Timers  (ts)
					>> accept ts:SSCOP_var in

				(* LM ! MAA_Error_ind ! T; *)
				Start_Error_Recovery_Procedures [CPCS] (s,ss,rs,ps,ts,sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
				)
			)
			EndProc (* Resend_Missing_PDUs *)

		EndProc (* Treat_USTAT_PDU *)

		(**********************************************************************)

      Process Treat_STAT_PDU [SSCF,CPCS,SSCOP_I] (s:SSCOP_State,
               ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, SD_Q,UD_Q,MD_Q:UU_Queue,
               TxBuf:TxBuffer, ReTxQ:PDU_Queue, RxBuf:RxBuffer,
               pdu:SSCOP_PDU) :

		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=

		(
		[ ( VT(PA,ss) le N_PS(pdu) ) and ( N_PS(pdu) le VT(var_PS,ss) ) ] ->
			(
			[ ( VT(A,ss) le N_R(pdu) ) and ( N_R(pdu) le VT(var_S,ss) ) ] ->
				(* Remove SD PDUs from VT(A) tp STAT.N(R) - 1 from 
				 * transmission buffer *)
				( let TxBuf:TxBuffer = remove_from_to(VT(A,ss),pred(N_R(pdu)),TxBuf),

				(* VT(A) := STAT.N(R) *)
				(* VT(PA) := STAT.N(PS) *)
				(* VT(MS) := STAT.N(MR) *)
					ss:SSCOP_var = SetVT(A, SetVT(PA, SetVT(MS,ss,N_MR(pdu)), 
											N_PS(pdu)), N_R(pdu)),

				(* n := number of STAT list elements *)
				(* Count := 0 *)
					n:Nat = sizeof(List(pdu)),
					count:Nat = 0 in

				[ n gt succ(0) ] ->
					( let seq1:Nat = get(List(pdu)),
						pdu:SSCOP_PDU = STAT(remove(list(pdu)), N_R(pdu), N_MR(pdu),
													N_PS(pdu)),
						n:Nat = pred(n) in
					
					[ seq1 lt VT(var_S,ss) ] ->
						Resend_All_Missing_PDUs [SSCF, CPCS, SSCOP_I] 
								(s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ,
								RxBuf, pdu,seq1,n,count)
					[]
					[ not(seq1 lt VT(var_S,ss)) ] ->
						(
						(* Reset Data Transfer Timers *)
						Reset_Data_Transfer_Timers  (ts)
							>> accept ts:SSCOP_var in

						(* LM ! MAA_Error_ind ! S; *)
						Start_Error_Recovery_Procedures [CPCS] (s, ss,rs,ps,
										ts, sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
						)
					)
				[]
				[ not(n gt succ(0)) ] ->
					Terminate_Resending (s,ss,rs,ps,ts,sp, SD_Q, 
															UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
				)
			[]
			[ not( ( VT(A,ss) le N_R(pdu) ) and ( N_R(pdu) le VT(var_S,ss) ) ) ] ->
				(
				(* Reset Data Transfer Timers *)
				Reset_Data_Transfer_Timers  (ts)
					>> accept ts:SSCOP_var in

				(* LM ! MAA_Error_ind ! S; *)
				Start_Error_Recovery_Procedures [CPCS] (s, ss,rs,ps,ts, sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
				)
			)
		[]
		[ not( ( VT(PA,ss) le N_PS(pdu) ) and ( N_PS(pdu) le VT(var_PS,ss) )) ] ->
			(
			(* Reset Data Transfer Timers *)
			Reset_Data_Transfer_Timers  (ts)
				>> accept ts:SSCOP_var in

			(* LM ! MAA_Error_ind ! R; *)
			Start_Error_Recovery_Procedures [CPCS] (s, ss,rs,ps,ts, sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			)
		)
		Where

			(*******************************************************************)

			Process Resend_All_Missing_PDUs [SSCF, CPCS, SSCOP_I] (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer, pdu:SSCOP_PDU, seq1:Nat, j,c:Nat) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
				(* seq2 := Next list element *)
				(* j := j - 1 *)
				let seq2:Nat = get(list(pdu)),
					pdu:SSCOP_PDU = STAT(remove(list(pdu)), N_R(pdu), N_MR(pdu),
												N_PS(pdu)),
					j:Nat = pred(j) in

				[ ( seq1 lt seq2 ) and ( seq2 le VT(var_S,ss) ) ] ->
					Resend_Missing_PDUs [SSCF, CPCS, SSCOP_I] 
												(s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q,
												TxBuf, ReTxQ, RxBuf, pdu,seq1,seq2,j,c)
				[]
				[ not( ( seq1 lt seq2 ) and ( seq2 le VT(var_S,ss) ) ) ] ->
					(
					(* reset Data Transfer Timers *)
					Reset_Data_Transfer_Timers (ts)
						>> accept ts:SSCOP_var in

					(* LM ! MAA_Error_ind ! S; *)
					Start_Error_Recovery_Procedures [CPCS] (s,ss,rs,ps,ts,sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
			)
			Where

				(****************************************************************)

				Process Resend_Missing_PDUs [SSCF,CPCS, SSCOP_I] (s:SSCOP_State, 
							ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
							SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
							RxBuf:RxBuffer, pdu:SSCOP_PDU, 
							seq1,seq2,j,c:Nat) :

				exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
						Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
						RxBuffer) :=

				(
				(* SD.N(S) = seq1 is in Transmission Buffer *)
				[ isin(seq1, TxBuf) ] ->
					(* Remove SD.N(S) = seq1 from Transmission buffer *)
					( let sd_pdu:SSCOP_PDU = PDU(getTX(seq1, TxBuf)),
						stored_n_ps:Nat = N_PS(getTX(seq1, TxBuf)),
						TxBuf:TxBuffer = remove(seq1, TxBuf) in

					
					(* Stored SD.N(PS) < N_PS(pdu) and N_PS(pdu) <= VT(PS,ss) *)

					[ ( stored_n_ps lt N_PS(pdu) ) 
						and ( N_PS(pdu) le VT(var_PS, ss) ) ] ->

						(
						(* SD PDU is already in retransmission queue *)
						[ isin(N_S(sd_pdu), ReTxQ) ] ->
							(
							i;
							Check_For_More_PDUs [SSCF,CPCS,SSCOP_I] 
									(s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ,
									RxBuf, pdu,seq1,seq2,j,c,sd_pdu)
							)
						[]
						[ not(isin(N_S(sd_pdu), ReTxQ)) ] ->
							(* Put SD PDU into retransmission queue *)
							( let ReTxQ:PDU_Queue = put(sd_pdu, ReTxQ),
							(* count := count+1 *)
								c:Nat = succ(c) in

							SSCOP_I ! QueueUp_SD_PDU;
							Check_For_More_PDUs [SSCF,CPCS, SSCOP_I] 
									(s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ,
									RxBuf, pdu,seq1,seq2,j,c,sd_pdu) )
						)
					[]
					[ not( ( stored_n_ps lt N_PS(pdu) ) 
						and ( N_PS(pdu) le VT(var_PS, ss) ) ) ] ->
						(
						i;
						Check_For_More_PDUs [SSCF,CPCS,SSCOP_I] 
								(s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ,
								RxBuf, pdu,seq1,seq2,j,c,sd_pdu)
						)
					)
				[]
				[ not(isin(seq1, TxBuf)) ] ->
					(
					(* Reset Data Transfer Timers *)
					Reset_Data_Transfer_Timers  (ts)
						>> accept ts:SSCOP_var in

					(* LM ! MAA_Error_ind ! S; *)
					Start_Error_Recovery_Procedures [CPCS] (s,ss,rs,ps,ts,sp, 
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
				)
				EndProc (* Resend_Missing_PDUs *)

				(****************************************************************)

				Process Check_For_More_PDUs [SSCF, CPCS, SSCOP_I] (s:SSCOP_State, 
							ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
							SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
							RxBuf:RxBuffer, pdu:SSCOP_PDU, seq1,seq2,j,c:Nat, 
							sd_pdu:SSCOP_PDU) :
	
				exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
						Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
						RxBuffer) :=

				(
					(* Save SD PDU in transmission buffer *)
					let TxBuf:TxBuffer = add(elt(VT(var_PS,ss),sd_pdu), TxBuf),
					(* seq1 := seq1 + 1 *)
						seq1:Nat = succ(seq1) in
					(
					[ seq1 eq seq2 ] ->
						(
						i;
						Skip_Already_Sent_PDUs [SSCF, CPCS, SSCOP_I] 
												(s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q, TxBuf,
												ReTxQ, RxBuf, pdu,seq1,seq2,j,c)
						)
					[]
					[ not(seq1 eq seq2) ] ->
						Resend_Missing_PDUs [SSCF, CPCS, SSCOP_I] 
												(s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf,
												ReTxQ, RxBuf, pdu,seq1,seq2,j,c)
					)
				)
				EndProc (* Check_For_More_PDUs *)

			EndProc (* Resend_All_Missing_PDUs *)

			(*******************************************************************)

			Process Skip_Already_Sent_PDUs [SSCF, CPCS, SSCOP_I] (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
						RxBuf:RxBuffer, pdu:SSCOP_PDU, seq1,seq2,j,c:Nat) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			[ j gt 0 ] ->
				(* seq2 := next list element *)
				( let seq2:Nat = get(list(pdu)),
					pdu:SSCOP_PDU = STAT(remove(list(pdu)), N_R(pdu), N_MR(pdu),
												N_PS(pdu)),
					j:Nat = pred(j) in
				
				[ ( seq1 lt seq2 ) and ( seq2 lt VT(var_S,ss) ) ] ->
					Skip_This_PDU [SSCF, CPCS, SSCOP_I] 
										(s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ,
										RxBuf, pdu,seq1,seq2,j,c)
				[]
				[ not( ( seq1 lt seq2 ) and ( seq2 lt VT(var_S,ss) ) ) ] ->
					(
					(* Reset Data Transfer Timers *)
					Reset_Data_Transfer_Timers (ts)
						>> accept ts:SSCOP_var in

					(* LM ! MAA_Error_ind ! S; *)
					Start_Error_Recovery_Procedures [CPCS] (s,ss,rs,ps,ts,sp,
													SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
				)
			[]
			[ not(j gt 0) ] ->
				(* LM ! MAA_Error_ind ! V(c); *)
				Terminate_Resending (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
																MD_Q, TxBuf, ReTxQ, RxBuf)
			)
			Where

				(****************************************************************)

				Process Skip_This_PDU [SSCF, CPCS, SSCOP_I] (s:SSCOP_State, 
							ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU,
							SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
							RxBuf:RxBuffer, pdu:SSCOP_PDU, seq1,seq2,j,c:Nat) :

				exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
						Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
						RxBuffer) :=

				(
				(
				[ ParamB(ClearBuffers,ps) ] ->
					(* Remove SD.N(S) = seq1 from transmission buffer *)
					exit (remove(seq1, TxBuf))
				[]
				[ not(ParamB(ClearBuffers,ps)) ] ->
					exit (TxBuf)
				) >> accept TxBuf:TxBuffer in
				
				(* seq1 := seq1 + 1 *)
				( let seq1:Nat = succ(seq1) in
				
				[ seq1 eq seq2 ] ->
					(
					[ j gt 0 ] ->
						Resend_All_Missing_PDUs [SSCF, CPCS, SSCOP_I] 
														(s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q,
														TxBuf, ReTxQ, RxBuf, pdu, seq1,j,c)
					[]
					[ not(j gt 0) ] ->
						(* LM ! MAA_Error_ind ! V(c); *)
						Terminate_Resending  (s,ss,rs,ps,ts,sp, SD_Q, 
															UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
				[]
				[ not(seq1 eq seq2) ] ->
					Skip_This_PDU [SSCF, CPCS, SSCOP_I] 
										(s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q, TxBuf,
										ReTxQ, RxBuf, pdu,seq1,seq2,j,c)
				)
				)
				EndProc (* Skip_This_PDU *)

			EndProc (* Skip_Already_Sent_PDUs *)

			(********************************************************************)

			Process Terminate_Resending (s:SSCOP_State, 
						ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, 
						SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue, 
						RxBuf:RxBuffer) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			[ VT(var_S,ss) lt VT(MS,ss) ] ->
				(
				[ ParamB(Credit,ps) ] ->
					(
					exit (ps)
					)
				[]
				[ not(ParamB(Credit,ps)) ] ->
					( let ps:SSCOP_var = SetParamB(Credit,ps,True) in
					(* LM ! MAA_Error_ind ! X *)
					exit (ps) )
				)
			[]
			[ not(VT(var_S,ss) lt VT(MS,ss)) ] ->
				(
				[ ParamB(Credit,ps) ] ->
					( let ps:SSCOP_var = SetParamB(Credit,ps,False) in
					(* LM ! MAA_Error_ind ! W *)
					exit (ps) )
				[]
				[ not(ParamB(Credit,ps)) ] ->
					(
					exit(ps)
					)
				)
			) >> accept ps:SSCOP_var in

			(
			[ ActiveTimer(POLL, ts) ] ->
				( let ts:SSCOP_var = SetTimer(NO_RESPONSE, ts, 
													Param(InitNO_RESPONSE,ps)) in

				exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )

			[]
			[ not(ActiveTimer(POLL, ts)) ] ->
				(
				[ ActiveTimer(IDLE,ts) ] ->
					(
					exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
					)
				[]
				[ not(ActiveTimer(IDLE,ts)) ] ->
					( let ts:SSCOP_var = ResetTimer(KEEP_ALIVE, 
												ResetTimer(NO_RESPONSE,
												SetTimer(IDLE,ts,Param(InitIDLE,ps)))) in

					exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf) )
				)
			)
			EndProc (* Terminate_Resending *) 

		EndProc (* Treat_STAT_PDU *)

		(**********************************************************************)

		Process Start_Error_Recovery_Procedures [CPCS] (s:SSCOP_State,
					ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU, SD_Q,UD_Q,MD_Q:UU_Queue,
					TxBuf:TxBuffer, ReTxQ:PDU_Queue, RxBuf:RxBuffer) :

		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=

		(
			(* VT(CC) := 1 *)
			(* VT(SQ) := VT(SQ) + 1 *)
			let ss:SSCOP_var = SetVT(CC,IncVT(SQ,ss),succ(0)),

			(* Initialize VR(MR) *)
				rs:SSCOP_var = SetVR(MR,rs,Param(InitMR, ps)) in

			(* remember the PDU we send *)
			let sp:Set_of_PDU = add(ER_pdu, ER(VR(MR,rs),VT(SQ,ss)), sp) in

			CPCS ! CPCS_UnitData_invoke ! ER(VR(MR,rs),VT(SQ,ss));

			(* Prepare Recovery *)
			Prepare_Recovery  (ps, SD_Q, TxBuf, ReTxQ)
				>> accept SD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue in

			( let ts:SSCOP_var = SetTimer(timer_CC,ts, Param(InitCC,ps)) in
				
			exit (Outgoing_Recovery_Pending_7, ss, rs, ps, ts, sp, SD_Q, UD_Q, 
					MD_Q, TxBuf, ReTxQ, RxBuf) )
		)
		EndProc (* Start_Error_Recovery_Procedures *)

		(**********************************************************************)

		Process Release_Buffers  (TxQ:UU_Queue, TxBuf:TxBuffer,
					ReTxQ:PDU_Queue, RxBuf:RxBuffer) :
		exit (UU_Queue, TxBuffer, PDU_Queue, RxBuffer) := 

		hide I_SSCOP in (
		I_SSCOP ! Proc_Release_Buffers;

		(
			(* Clear Transmission Queue *)
			let TxQ:UU_Queue = empty of UU_Queue,
			(* Clear Transmission Buffer *)
				TxBuf:TxBuffer = empty of TxBuffer,
			(* Clear Retransmission Queue *)
				ReTxQ:PDU_Queue = empty of PDU_Queue,
			(* Clear Receiver Buffer *)
				RxBuf:RxBuffer = empty of RxBuffer in
			exit (TxQ, TxBuf, ReTxQ, RxBuf)
		))
		EndProc (* Release_Buffers *)

		(**********************************************************************)

		Process Prepare_Retrieval  (ps:SSCOP_var, TxQ:UU_Queue, 
					TxBuf:TxBuffer, ReTxQ:PDU_Queue, RxBuf:RxBuffer) :
		exit (UU_Queue, TxBuffer, PDU_Queue, RxBuffer) := 

		hide I_SSCOP in (
		I_SSCOP ! Proc_Prepare_Retrieval;
		(
		[ ParamB(ClearBuffers,ps) ] ->
			(
			(* Clear Transmission Queue *)
			(* Clear Transmission Buffer *)
			(* Clear Retransmission Queue *)
			(* Clear Receiver Buffer *)
			exit (empty of UU_Queue, empty of TxBuffer, empty of PDU_Queue, 
					empty of RxBuffer)
			)
		[]
		[ not(ParamB(ClearBuffers,ps)) ] ->
			(
			(* Clear Retransmission Queue *)
			(* Clear Receiver Buffer *)
			exit (TxQ, TxBuf, empty of PDU_Queue, empty of RxBuffer)
			)
		) )
		EndProc (* Prepare_Retrieval *)

		(**********************************************************************)

		Process Prepare_Recovery  (ps:SSCOP_var, TxQ:UU_Queue, 
					TxBuf:TxBuffer, ReTxQ:PDU_Queue) :
		exit (UU_Queue, TxBuffer, PDU_Queue) :=

		hide I_SSCOP in (
		I_SSCOP ! Proc_Prepare_Recovery;

		(
		[ ParamB(ClearBuffers,ps) ] ->
			(
			(* Clear transmission Queue *)
			(* Clear Transmission Buffer *)
			(* Clear Retransmisison Queue *)
			exit(empty of UU_Queue, empty of TxBuffer, empty of PDU_Queue)
			)
		[]
		[ not(ParamB(ClearBuffers,ps)) ] ->
			(
			(* Clear Retransmisison Queue *)
			exit(TxQ, TxBuf, empty of PDU_Queue)
			)
		))
		EndProc (* Prepare_Recovery *)

		(**********************************************************************)

		Process Clear_Transmitter  (ps:SSCOP_var, TxQ:UU_Queue, 
											TxBuf:TxBuffer) :
		exit (UU_Queue, TxBuffer) :=

		hide I_SSCOP in (
		I_SSCOP ! Proc_Clear_Transmitter;
		(
		[ ParamB(ClearBuffers,ps) ] ->
			(
			exit (TxQ, TxBuf)
			)
		[]
		[ not(ParamB(ClearBuffers,ps)) ] ->
			(
			(* Clear Transmission Queue *)
			(* Clear Transmission Buffer *)
			exit (empty of UU_Queue, empty of TxBuffer)
			)
		))
		EndProc (* Clear_Transmitter *)

		(**********************************************************************)

		Process Deliver_Data  (ps:SSCOP_var, RxBuf:RxBuffer) :
		exit (RxBuffer) :=

		hide I_SSCOP in (
		I_SSCOP ! Proc_Deliver_Data;
		(
		[ ParamB(ClearBuffers,ps) ] ->
			(
			(* Clear Receiver Buffer *)
			exit (empty of RxBuffer)
			)
		[]
		[ not(ParamB(ClearBuffers,ps)) ] ->
			(
			(* Deliver in sequence all data in the Receiver Buffer *)
			(* Clear Receiver Buffer *)
			exit (empty of RxBuffer)
			)
		))
		EndProc (* Deliver_Data *)

		(**********************************************************************)

		Process Initialize_State_Variables (ss, rs,ps:SSCOP_var) :
		exit (SSCOP_var, SSCOP_var, SSCOP_var) :=

		hide I_SSCOP in (
		I_SSCOP ! Proc_Initialize_State_Variables;
		(
			(* VT(S) := 0 *)
			(* VT(PS) := 0 *)
			(* VT(A) := 0 *)
			(* VT(PA) := 1 *)
			(* VT(PD) := 0 *)
			let ss:SSCOP_var = SetVT(var_S, SetVT(var_PS, SetVT(A, SetVT(PA, 
										SetVT(PD, ss, 0), succ(0)), 0), 0), 0),

			(* Credit := Yes *)
				ps:SSCOP_var = SetParamB(Credit, ps, true),

			(* VR(H) := 0 *)
			(* VR(R) := 0 *)
				rs:SSCOP_var = SetVR(H, SetVR(R, rs, 0), 0) in

			exit (ss, rs, ps)
		))
		EndProc (* Initialize_State_Variables *)

		(**********************************************************************)

		Process Data_Retrieval [SSCF] (s:SSCOP_State, ss,rs,ps,ts:SSCOP_var,
											sp:Set_of_PDU, SD_Q, UD_Q, MD_Q:UU_Queue,
											TxBuf:TxBuffer, ReTxQ:PDU_Queue, 
											RxBuf:RxBuffer, aa:AA_RN) :
		
		exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
				Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
				RxBuffer) :=

		hide I_SSCOP in (
		I_SSCOP ! Proc_Data_Retrieval;
		(
		[ aa eq Unknown ] ->
			Empty_Transmission_Queue [SSCF] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, 
														TxBuf, ReTxQ, RxBuf)
		[]
		[ not(aa eq Unknown) ] ->
			(
			[ aa eq Total ] ->
				( let n:Nat = VT(A,ss) in
				Empty_Transmission_Buffer [SSCF] (s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q,
									TxBuf, ReTxQ, RxBuf, n) )
			[]
			[ not(aa eq Total) ] ->
				( let n:Nat = succ(get(aa)) in
				Empty_Transmission_Buffer [SSCF] (s,ss,rs,ps,ts,sp,SD_Q, UD_Q, MD_Q,
									TxBuf, ReTxQ, RxBuf, n) )
			)
		))
		Where

			(********************************************************************)

			Process Empty_Transmission_Buffer [SSCF] (s:SSCOP_State, 
							ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU,
							SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
							RxBuf:RxBuffer, n:Nat) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			[ ( VT(A,ss) le n ) and ( n lt VT(var_S,ss) ) ] ->
				(* Remove SD.N(S) = i from Transmission Buffer *)
				( let pdu:SSCOP_PDU = PDU(getTX(n,TxBuf)),
					TxBuf:TxBuffer = remove(n, TxBuf) in

				SSCF ! AA_Retrieve_ind ! UU(pdu);
				Empty_Transmission_Buffer [SSCF] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q,
												MD_Q, TxBuf, ReTxQ, RxBuf,succ(n)) )
			[]
			[ not((VT(A,ss) le n) and ( n lt VT(var_S,ss))) ] ->
				Empty_Transmission_Queue [SSCF] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
															MD_Q, TxBuf, ReTxQ, RxBuf)
			)
			EndProc (* Empty_Transmission_Buffer *)

			(*******************************************************************)

			Process Empty_Transmission_Queue [SSCF] (s:SSCOP_State, 
							ss,rs,ps,ts:SSCOP_var, sp:Set_of_PDU,
							SD_Q, UD_Q, MD_Q:UU_Queue, TxBuf:TxBuffer, ReTxQ:PDU_Queue,
							RxBuf:RxBuffer) :

			exit (SSCOP_State, SSCOP_var, SSCOP_var, SSCOP_var, SSCOP_var,
					Set_of_PDU, UU_Queue, UU_Queue, UU_Queue, TxBuffer, PDU_Queue,
					RxBuffer) :=

			(
			(* Transmission Queue is empty *)
			[ isempty(SD_Q) ] ->
				SSCF ! AA_RetrieveComplete_ind;
				exit (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, MD_Q, TxBuf, ReTxQ, RxBuf)
			[]
			[ not(isempty(SD_Q)) ] ->
				(* Remove MU from Transmisison Queue *)
				(* ( Retrieve in order received ) *)
				( let mu:SSCOP_UU = get(SD_Q),
					SD_Q:UU_Queue = remove(SD_Q) in

				SSCF ! AA_Retrieve_ind ! mu;
				Empty_Transmission_Queue [SSCF] (s,ss,rs,ps,ts,sp, SD_Q, UD_Q, 
															MD_Q, TxBuf, ReTxQ, RxBuf) )
			)
			EndProc (* Empty_Transmission_Queue *)

		EndProc (* Data_Retrieval *)

		(**********************************************************************)

		Process Detect_Retransmission (rs:SSCOP_var, pdu:SSCOP_PDU) :
		exit (SSCOP_var, Bool) :=
		hide I_SSCOP in
		(
		I_SSCOP ! Proc_Detect_Retransmission;
		(
		[ N_SQ(pdu) eq VR(SQ,rs) ] ->
			(* retransmission := true *)
			exit (rs, true)
		[]
		[ not(N_SQ(pdu) eq VR(SQ,rs)) ] ->
			(* VR(SQ) := pdu.N(SQ) *)
			(* retransmission := false *)
			exit (SetVR(SQ,rs,N_SQ(pdu)), false)
		))
		EndProc (* Detect_Retransmission *)

		(**********************************************************************)

		Process Set_POLL_Timer (ss,ps,ts:SSCOP_var, TxQ:UU_Queue) :
		exit (SSCOP_var) :=
		hide I_SSCOP in
		(
		I_SSCOP ! Proc_Set_POLL_Timer;
		(
		[ isempty(TxQ) ] ->
			(
			[ VT(var_S,ss) eq VT(A,ss) ] ->
				exit ( SetTimer(KEEP_ALIVE, ts, Param(InitKEEP_ALIVE, ps)) )
				[]
				[ not(VT(var_S,ss) eq VT(A,ss)) ] ->
				exit ( SetTimer(POLL, ts, Param(InitPOLL,ps)) )
			)
		[]
		[ not(isempty(TxQ)) ] ->
			exit ( SetTimer(POLL, ts, Param(InitPOLL, ps)) )
		))
		EndProc (* Set_POLL_Timer *)

		(**********************************************************************)

		Process Reset_Data_Transfer_Timers (ts:SSCOP_var) :
		exit (SSCOP_var) :=
		hide I_SSCOP in
		(
			I_SSCOP ! Proc_Reset_Data_Transfer_Timers;
			exit( ResetTimer(POLL, ResetTimer(KEEP_ALIVE, 
					ResetTimer(NO_RESPONSE, ResetTimer(IDLE, ts)))) ) 
		)
		EndProc (* Reset_Data_Transfer_Timers *)

		(**********************************************************************)

		Process Set_Data_Transfer_Timers (ps,ts:SSCOP_var) :
		exit (SSCOP_var) :=
		hide I_SSCOP in
		(
			I_SSCOP ! Proc_Set_Data_Transfer_Timers;
			exit ( SetTimer(POLL, SetTimer(NO_RESPONSE, ts,
						Param(InitNO_RESPONSE, ps)), Param(InitPOLL, ps)) )
		)
		EndProc (* Set_Data_Transfer_Timers *)

		(**********************************************************************)

	EndProc (* SSCOP_Action *)


(*******************************************************************)
(** Private Data types                                            **)
(*******************************************************************)

Type SSCOP_State is
	sorts SSCOP_State
	opns
		Idle_1,
		Outgoing_Connection_Pending_2,
		Incoming_Connection_Pending_3,
		Outgoing_Disconnection_Pending_4,
		Outgoing_Resynchronization_Pending_5,
		Incoming_Resynchronization_Pending_6,
		Outgoing_Recovery_Pending_7,
		Recovery_Response_Pending_8,
		Incoming_Recovery_Pending_9,
		Data_Transfer_Ready_10 : -> SSCOP_State
EndType

Type SSCOP_I_Actions is
	sorts SSCOP_I_Actions
	opns
		Timer_CC_Expiry,
		Timer_POLL_Expiry,
		Timer_KEEP_ALIVE_Expiry,
		Timer_IDLE_Expiry,
		Timer_NO_RESPONSE_Expiry,
		Proc_Release_Buffers,
		Proc_Prepare_Retrieval,
		Proc_Prepare_Recovery,
		Proc_Clear_Transmitter,
		Proc_Deliver_Data,
		Proc_Initialize_State_Variables,
		Proc_Data_Retrieval,
		Proc_Detect_Retransmission,
		Proc_Set_POLL_Timer,
		Proc_Reset_Data_Transfer_Timers,
		Proc_Set_Data_Transfer_Timers,
		QueueUp_SD_PDU,
		SD_PDU_QueuedUp,
		QueueUp_MD_PDU,
		MD_PDU_QueuedUp,
		QueueUp_UD_PDU,
		UD_PDU_QueuedUp: -> SSCOP_I_Actions
EndType

Type GenericQueue is Boolean
	formalsorts Element

	formalopns
		NoElement : -> Element

	sorts Queue

	opns
		empty : -> Queue
		put : Element, Queue -> Queue
		get : Queue -> Element
		remove : Queue -> Queue
		isempty : Queue -> Bool		

	eqns
		forall e:Element, q:Queue

			ofsort Element
				get(empty) = NoElement;
				get(put(e,empty)) = e;
				not(isempty(q)) => get(put(e,q)) = get(q);

			ofsort Queue
				remove(empty) = empty;
				remove(put(e, empty)) = empty;
				not(isempty(q)) => remove(put(e, q)) = put(e, remove(q));

			ofsort Bool
				isempty(empty) = true;
				isempty(put(e,q)) = false;
EndType

(*
Type ListOfNat is GenericQueue ActualizedBy NaturalNumber using
	sortnames
		Nat for Element
		List_Of_Nat for Queue
	opnnames
		0 for NoElement
EndType
*)

Type SSCOP_var_name is
	sorts SSCOP_var_name
	opns
		var_S, var_PS, A, PA, MS, PD, CC, SQ, R, H, MR : -> SSCOP_var_name
EndType

Type SSCOP_timer_name is
	sorts SSCOP_timer_name
	opns
		timer_CC, POLL, NO_RESPONSE, KEEP_ALIVE, IDLE : -> SSCOP_timer_name
EndType

Type SSCOP_param_name is
	sorts SSCOP_param_name
	opns
		MaxCC, MaxPD, MaxSTAT, ClearBuffers, Credit, InitMR, InitCC : -> SSCOP_param_name
		InitIDLE, InitKEEP_ALIVE, InitPOLL, InitNO_RESPONSE : -> SSCOP_param_name
EndType

Type SSCOP_var is NaturalNumber, Boolean, SSCOP_var_name, SSCOP_timer_name, SSCOP_param_name
	sorts SSCOP_var
	opns
		SenderSet : Nat, Nat, Nat, Nat, Nat, Nat, Nat, Nat -> SSCOP_var
		RecSet : Nat, Nat, Nat, Nat -> SSCOP_var
		ParamSet : Nat, Nat, Nat, Bool, Bool, Nat, Nat -> SSCOP_var
		TimerSet : Nat, Bool, Nat, Bool, Nat, Bool, 
						Nat, Bool, Nat, Bool -> SSCOP_var

		VT : SSCOP_var_name, SSCOP_var -> Nat
		SetVT : SSCOP_var_name, SSCOP_var, Nat -> SSCOP_var
		IncVT : SSCOP_var_name, SSCOP_var -> SSCOP_var
		DecVT : SSCOP_var_name, SSCOP_var -> SSCOP_var

		VR : SSCOP_var_name, SSCOP_var -> Nat
		SetVR : SSCOP_var_name, SSCOP_var, Nat -> SSCOP_var
		IncVR : SSCOP_var_name, SSCOP_var -> SSCOP_var
		DecVR : SSCOP_var_name, SSCOP_var -> SSCOP_var

		Timer : SSCOP_timer_name, SSCOP_var -> Nat
		SetTimer : SSCOP_timer_name, SSCOP_var, Nat -> SSCOP_var
		DecTimer : SSCOP_timer_name, SSCOP_var -> SSCOP_var
		ActiveTimer : SSCOP_timer_name, SSCOP_var -> Bool
		SetActiveTimer : SSCOP_timer_name, SSCOP_var, Bool -> SSCOP_var
		DecAllTimers : SSCOP_var -> SSCOP_var
		ResetTimer : SSCOP_timer_name, SSCOP_var -> SSCOP_var
		TimerExpired : SSCOP_timer_name, SSCOP_var -> Bool

		Param : SSCOP_param_name, SSCOP_var -> Nat
		ParamB : SSCOP_param_name, SSCOP_var -> Bool
		SetParam : SSCOP_param_name, SSCOP_var, Nat -> SSCOP_var
		SetParamB : SSCOP_param_name, SSCOP_var, Bool -> SSCOP_var

	eqns
		forall n,n1,n2,n3,n4,n5,n6,n7,n8:Nat, b,b1,b2,b3,b4,b5:Bool, 
				vset:SSCOP_var, vname:SSCOP_var_name, tname:SSCOP_timer_name

		ofsort Nat
			VT(var_S, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n1;
			VT(var_PS, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n2;
			VT(A, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n3;
			VT(PA, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n4;
			VT(MS, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n5;
			VT(PD, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n6;
			VT(CC, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n7;
			VT(SQ, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8)) = n8;

			VR(R, RecSet(n1,n2,n3,n4)) = n1;
			VR(H, RecSet(n1,n2,n3,n4)) = n2;
			VR(MR, RecSet(n1,n2,n3,n4)) = n3;
			VR(SQ, RecSet(n1,n2,n3,n4)) = n4;

			Timer(timer_CC, TimerSet(n1,b1, n2,b2, n3,b3, n4,b4, n5,b5)) = n1;
			Timer(POLL, TimerSet(n1,b1, n2,b2, n3,b3, n4,b4, n5,b5)) = n2;
			Timer(NO_RESPONSE, TimerSet(n1,b1, n2,b2, n3,b3, n4,b4, n5,b5)) = n3;
			Timer(KEEP_ALIVE, TimerSet(n1,b1, n2,b2, n3,b3, n4,b4, n5,b5)) = n4;
			Timer(IDLE, TimerSet(n1,b1, n2,b2, n3,b3, n4,b4, n5,b5)) = n5;

			Param(MaxCC, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n1;
			Param(MaxPD, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n2;
			Param(MaxSTAT, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n3;
			Param(InitMR, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n6;
			Param(InitCC, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n7;
			Param(InitIDLE, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n7;
			Param(InitPOLL, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n7;
			Param(InitNO_RESPONSE, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n7;
			Param(InitKEEP_ALIVE, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = n7;

		ofsort Bool

			ParamB(ClearBuffers, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = b1;
			ParamB(Credit, ParamSet(n1, n2, n3, b1, b2, n6, n7)) = b2;

			ActiveTimer(timer_CC, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5))
				= b1;
			ActiveTimer(POLL, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5))
				= b2;
			ActiveTimer(NO_RESPONSE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5))
				= b3;
			ActiveTimer(KEEP_ALIVE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5))
				= b4;
			ActiveTimer(IDLE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5))
				= b5;

			TimerExpired(tname, vset)
				= (ActiveTimer(tname, vset) and (Timer(tname, vset) eq 0)); 

		ofsort SSCOP_var
		
			SetVT(var_S, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n, n2, n3, n4, n5, n6, n7, n8);
			SetVT(var_PS, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n, n3, n4, n5, n6, n7, n8);
			SetVT(A, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n2, n, n4, n5, n6, n7, n8);
			SetVT(PA, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n2, n3, n, n5, n6, n7, n8);
			SetVT(MS, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n2, n3, n4, n, n6, n7, n8);
			SetVT(PD, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n2, n3, n4, n5, n, n7, n8);
			SetVT(CC, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n2, n3, n4, n5, n6, n, n8);
			SetVT(SQ, SenderSet(n1,n2,n3,n4,n5,n6,n7,n8), n)
				= SenderSet(n1, n2, n3, n4, n5, n6, n7, n);

			SetVR(R, RecSet(n1,n2,n3,n4), n)
				= RecSet(n, n2, n3, n4);
			SetVR(H, RecSet(n1,n2,n3,n4), n)
				= RecSet(n1, n, n3, n4);
			SetVR(MR, RecSet(n1,n2,n3,n4), n)
				= RecSet(n1, n2, n, n4);
			SetVR(SQ, RecSet(n1,n2,n3,n4), n)
				= RecSet(n1, n2, n3, n);

			IncVT(vname, vset) = SetVT(vname, vset, succ(VT(vname, vset)));
			DecVT(vname, vset) = SetVT(vname, vset, pred(VT(vname, vset)));

			IncVR(vname, vset) = SetVR(vname, vset, succ(VR(vname, vset)));
			DecVR(vname, vset) = SetVR(vname, vset, pred(VR(vname, vset)));


			SetTimer(timer_CC, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), n)
				= TimerSet(n, True, n2, b2, n3, b3, n4, b4, n5, b5);
			SetTimer(POLL, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), n)
				= TimerSet(n1, b1, n, True, n3, b3, n4, b4, n5, b5);
			SetTimer(NO_RESPONSE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), n)
				= TimerSet(n1, b1, n2, b2, n, True, n4, b4, n5, b5);
			SetTimer(KEEP_ALIVE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), n)
				= TimerSet(n1, b1, n2, b2, n3, b3, n, True, n5, b5);
			SetTimer(IDLE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), n)
				= TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n, True);

			( ActiveTimer(tname, vset) ) =>
				DecTimer(tname, vset) = SetTimer(tname, vset, pred(Timer(tname, vset)));
			( not(ActiveTimer(tname, vset)) ) =>
				DecTimer(tname, vset) = vset;

			DecAllTimers(vset)
				= DecTimer(timer_CC, DecTimer(POLL, DecTimer(NO_RESPONSE,
					DecTimer(KEEP_ALIVE, DecTimer(IDLE, vset)))));

			SetActiveTimer(timer_CC, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), b)
				= TimerSet(n1, b, n2, b2, n3, b3, n4, b4, n5, b5);					
			SetActiveTimer(POLL, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), b)
				= TimerSet(n1, b1, n2, b, n3, b3, n4, b4, n5, b5);					
			SetActiveTimer(NO_RESPONSE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), b)
				= TimerSet(n1, b1, n2, b2, n3, b, n4, b4, n5, b5);					
			SetActiveTimer(KEEP_ALIVE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), b)
				= TimerSet(n1, b1, n2, b2, n3, b3, n4, b, n5, b5);					
			SetActiveTimer(IDLE, TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b5), b)
				= TimerSet(n1, b1, n2, b2, n3, b3, n4, b4, n5, b);					

			ResetTimer(tname, vset) = SetActiveTimer(tname, vset, False);

			SetParam(MaxCC, ParamSet(n1, n2, n3, b1, b2, n6, n7), n)
				= ParamSet(n, n2, n3, b1, b2, n6, n7);
			SetParam(MaxPD, ParamSet(n1, n2, n3, b1, b2, n6, n7), n)
				= ParamSet(n1, n, n3, b1, b2, n6, n7);
			SetParam(MaxSTAT, ParamSet(n1, n2, n3, b1, b2, n6, n7), n)
				= ParamSet(n1, n2, n, b1, b2, n6, n7);
			SetParamB(ClearBuffers, ParamSet(n1, n2, n3, b1, b2, n6, n7), b)
				= ParamSet(n1, n2, n3, b, b2, n6, n7);
			SetParamB(Credit, ParamSet(n1, n2, n3, b1, b2, n6, n7), b)
			= ParamSet(n1, n2, n3, b1, b, n6, n7);
			SetParam(InitMR, ParamSet(n1, n2, n3, b1, b2, n6, n7), n)
				= ParamSet(n1, n2, n3, b1, b2, n, n7);
			SetParam(InitCC, ParamSet(n1, n2, n3, b1, b2, n6, n7), n)
				= ParamSet(n1, n2, n3, b1, b2, n6, n);

EndType

Type PDU_type is
	sorts PDU_type
	opns
		BGN_pdu,	BGAK_pdu, BGREJ_pdu, ER_pdu, END_pdu, RS_pdu : -> PDU_type
EndType

Type Set_of_PDU is SSCOP_PDU, PDU_Type, Boolean
	sorts Set_of_PDU
	opns
		pdu_set : SSCOP_PDU, SSCOP_PDU, SSCOP_PDU, SSCOP_PDU, 
				SSCOP_PDU, SSCOP_PDU -> Set_of_PDU
		add: PDU_type, SSCOP_PDU, Set_of_PDU -> Set_of_PDU
		isin: PDU_type, Set_of_PDU -> Bool
		get: PDU_type, Set_of_PDU -> SSCOP_PDU
		addifisin : PDU_type, SSCOP_PDU, SSCOP_PDU, Set_of_PDU -> Set_of_PDU

	eqns
		forall t:PDU_type, pdu, p1, p2, p3, p4, p5, p6:SSCOP_PDU, s:Set_of_PDU

		ofsort Set_of_PDU
			add(BGN_pdu, pdu, pdu_set(p1, p2, p3, p4, p5, p6)) 
				= pdu_set(pdu, p2, p3, p4, p5, p6);
			add(BGAK_pdu, pdu, pdu_set(p1, p2, p3, p4, p5, p6)) 
				= pdu_set(p1, pdu, p3, p4, p5, p6);
			add(BGREJ_pdu, pdu, pdu_set(p1, p2, p3, p4, p5, p6)) 
				= pdu_set(p1, p2, pdu, p4, p5, p6);
			add(ER_pdu, pdu, pdu_set(p1, p2, p3, p4, p5, p6)) 
				= pdu_set(p1, p2, p3, pdu, p5, p6);
			add(END_pdu, pdu, pdu_set(p1, p2, p3, p4, p5, p6)) 
				= pdu_set(p1, p2, p3, p4, pdu, p6);
			add(RS_pdu, pdu, pdu_set(p1, p2, p3, p4, p5, p6)) 
				= pdu_set(p1, p2, p3, p4, p5, pdu);

			isin(t, s) => addifisin(t,p1,p2,s) = add(t,p1,s);
			not(isin(t, s)) => addifisin(t,p1,p2,s) = add(t,p2,s);

		ofsort Bool

			isin(BGN_pdu, pdu_set(pdu, p2, p3, p4, p5, p6)) = is_BGN(pdu);
			isin(BGAK_pdu, pdu_set(p1, pdu, p3, p4, p5, p6)) = is_BGAK(pdu);
			isin(BGREJ_pdu, pdu_set(p1, p2, pdu, p4, p5, p6)) = is_BGREJ(pdu);
			isin(ER_pdu, pdu_set(p1, p2, p3, pdu, p5, p6)) = is_ER(pdu);
			isin(END_pdu, pdu_set(p1, p2, p3, p4, pdu, p6)) = is_END(pdu);
			isin(RS_pdu, pdu_set(p1, p2, p3, p4, p5, pdu)) = is_RS(pdu);

		ofsort SSCOP_PDU
			get(BGN_pdu, pdu_set(p1, p2, p3, p4, p5, p6)) = p1;
			get(BGAK_pdu, pdu_set(p1, p2, p3, p4, p5, p6)) = p2;
			get(BGREJ_pdu, pdu_set(p1, p2, p3, p4, p5, p6)) = p3;
			get(ER_pdu, pdu_set(p1, p2, p3, p4, p5, p6)) = p4;
			get(END_pdu, pdu_set(p1, p2, p3, p4, p5, p6)) = p5;
			get(RS_pdu, pdu_set(p1, p2, p3, p4, p5, p6)) = p6;

EndType


Type UU_Queue is GenericQueue ActualizedBy SSCOP_UU using
	sortnames
		SSCOP_UU for Element
		UU_Queue for Queue
	opnnames
		null_uu for NoElement
EndType

Type PDU_Queue is SSCOP_PDU, Boolean, NaturalNumber
	sorts
		PDU_Queue

	opns
		empty : -> PDU_Queue
		put : SSCOP_PDU, PDU_Queue -> PDU_Queue

		isin : Nat, PDU_Queue -> Bool
		get : PDU_Queue -> SSCOP_PDU
		remove : PDU_Queue -> PDU_Queue
		isempty : PDU_Queue -> Bool
		remove_from_to : Nat, Nat, PDU_Queue -> PDU_Queue

	eqns
		forall p:SSCOP_PDU, n,n1,n2:Nat, q:PDU_Queue

		ofsort SSCOP_PDU
			get(empty) = NO_PDU;
			get(put(p,empty)) = p;
			not(isempty(q)) => get(put(p,q)) = get(q);

		ofsort Bool
			isin(n,empty) = false;
			(n eq N_S(p)) => isin(n,put(p,q)) = true;
			not(n eq N_S(p)) => isin(n,put(p,q)) = isin(n,q);

			isempty(empty of PDU_Queue) = true;
			isempty(put(p,q)) = false;

		ofsort PDU_Queue
			remove(empty of PDU_Queue) = empty;
			remove(put(p,empty of PDU_Queue)) = empty;
			not(isempty(q)) => remove(put(p,q)) = put(p, remove(q));

			remove_from_to(n1, n2, empty of PDU_Queue) = empty;
			((n1 le N_S(p)) and (N_S(p) le n2)) =>
				remove_from_to(n1, n2, put(p,q)) = remove_from_to(n1, n2, q);
			not((n1 le N_S(p)) and (N_S(p) le n2)) =>
				remove_from_to(n1, n2, put(p,q)) = put(p,remove_from_to(n1, n2, q));

EndType


Type RxBuffer is SSCOP_PDU, NaturalNumber, Boolean

	sorts RxBuffer

	opns
		empty : -> RxBuffer
		add: SSCOP_PDU, RxBuffer -> RxBuffer

		get: Nat, RxBuffer -> SSCOP_PDU
		isin: Nat, RxBuffer -> Bool
		remove: Nat, RxBuffer -> RxBuffer
		remove_from_to: Nat, Nat, RxBuffer -> RxBuffer

	eqns
		forall c,n1,n2:Nat, e:SSCOP_PDU, b:RxBuffer

		ofsort Bool
			isin(c,empty of RxBuffer) = false;
			(c eq N_S(e))		=> isin(c,add(e,b)) = true;
			not(c eq N_S(e))	=> isin(c,add(e,b)) = isin(c,b);

		ofsort SSCOP_PDU
			get(c,empty of RxBuffer) = NO_PDU;
			(c eq N_S(e)) 		=> get(c,add(e,b)) = e;
			not(c eq N_S(e))	=> get(c,add(e,b)) = get(c,b);

		ofsort RxBuffer
			remove(c, empty of RxBuffer) = empty of RxBuffer;
			(c eq N_S(e))		=> remove(c,add(e,b)) = b;
			not(c eq N_S(e))	=> remove(c,add(e,b)) = add(e,remove(c,b));

			remove_from_to(n1, n2, empty of RxBuffer) = empty of RxBuffer;
			((n1 le N_S(e)) and (N_S(e) le n2)) =>
				remove_from_to(n1, n2, add(e,b)) = remove_from_to(n1, n2, b);
			not((n1 le N_S(e)) and (N_S(e) le n2)) =>
				remove_from_to(n1, n2, add(e,b)) = add(e, remove_from_to(n1,n2,b));

EndType

Type TxBufferElt is SSCOP_PDU, NaturalNumber, Boolean
	sorts
		TxElt
	opns
		no_elt : -> TxElt
		elt: Nat, SSCOP_PDU -> TxElt

		PDU: TxElt -> SSCOP_PDU
		N_PS: TxElt -> Nat

	eqns
		forall n:Nat, p:SSCOP_PDU

		ofsort SSCOP_PDU
			PDU(elt(n,p)) = p;
			PDU(no_elt) = NO_PDU;

		ofsort Nat
			N_PS(elt(n,p)) = n;
			N_PS(no_elt) = 0;
EndType

Type TxBuffer is TxBufferElt, NaturalNumber, Boolean

	sorts TxBuffer

	opns
		empty : -> TxBuffer
		add: TxElt, TxBuffer -> TxBuffer

		getTX: Nat, TxBuffer -> TxElt
		isin: Nat, TxBuffer -> Bool
		remove: Nat, TxBuffer -> TxBuffer
		remove_from_to: Nat, Nat, TxBuffer -> TxBuffer

	eqns
		forall c,n1,n2:Nat, e:TxElt, b:TxBuffer

		ofsort Bool
			isin(c,empty of TxBuffer) = false;
			(c eq N_S(PDU(e)))		=> isin(c,add(e,b)) = true;
			not(c eq N_S(PDU(e)))	=> isin(c,add(e,b)) = isin(c,b);

		ofsort TxElt
			getTX(c,empty of TxBuffer) = no_elt;
			(c eq N_S(PDU(e))) 		=> getTX(c,add(e,b)) = e;
			not(c eq N_S(PDU(e)))	=> getTX(c,add(e,b)) = getTX(c,b);

		ofsort TxBuffer
			remove(c, empty of TxBuffer) = empty of TxBuffer;
			(c eq N_S(PDU(e)))		=> remove(c,add(e,b)) = b;
			not(c eq N_S(PDU(e)))	=> remove(c,add(e,b)) = add(e,remove(c,b));

			remove_from_to(n1, n2, empty of TxBuffer) = empty of TxBuffer;
			((n1 le N_S(PDU(e))) and (N_S(PDU(e)) le n2)) =>
				remove_from_to(n1, n2, add(e,b)) = remove_from_to(n1, n2, b);
			not((n1 le N_S(PDU(e))) and (N_S(PDU(e)) le n2)) =>
				remove_from_to(n1, n2, add(e,b)) = add(e, remove_from_to(n1,n2,b));
EndType

EndProc (* SSCOP *)



EndSpec

